unit pgpSC;

interface

uses
  Windows;

type
  pPGPsc = pointer;  //pPGPContext
  pPGPtls = pointer; //pPGPtlsContext

type
  pFileList = ^TFileList;
  TFileList = record
    Name         : array[0..MAX_PATH-1] of char;
	  IsDirectory  : boolean;
	  next         : pFileList;
  end;

// Library initialization routines
var InitPGPsc: function(Parent: hwnd; var P: pPGPsc; var TLS: pPGPtls): boolean; cdecl;

var UninitPGPsc: function(Parent: hwnd; P: pPGPsc; TLS: pPGPtls): boolean; cdecl;

//BOOL CheckForPurge(UINT uMsg,WPARAM wParam);

// Clipboard routines
var EncryptClipboard: function(Parent: hWnd;
    App          : PChar;
    P            : pPGPsc;
    Tls          : pPGPtls;
    Encrypt      : Longbool;
    Sign         : Longbool): LongBool;cdecl;

var DecryptClipboard: function(Parent: hWnd;
    App          : PChar;
    P            : pPGPsc;
    Tls          : pPGPtls): boolean;cdecl;

var AddKeyClipboard: function(Parent: hWnd;
    P            : pPGPsc;
    Tls          : pPGPtls): boolean;cdecl;

var ClipboardWipe: function(Parent: hWnd; P: pPGPsc): boolean;cdecl;

// Filelist routines
var EncryptFileList: function(Parent: hWnd;
    App          : PChar;
    P            : pPGPsc;
    Tls          : pPGPtls;
    ListHead     : pFileList;
    Encrypt      : boolean;
    Sign         : boolean): boolean; cdecl;

var DecryptFileList: function(Parent: hWnd;
    App          : PChar;
    P            : pPGPsc;
    Tls          : pPGPtls;
    ListHead     : pFileList): boolean; cdecl;

var AddKeyFileList: function(Parent: hWnd;
    P            : pPGPsc;
    Tls          : pPGPtls;
    ListHead     : pFileList): boolean; cdecl;

var WipeFileList: function(Parent: hWnd;
    P            : pPGPsc;
    ListHead     : pFileList;
    DelFilePostWipe : boolean): boolean; cdecl;

// Filelist utility functions (Note that UserCancel can be NULL)
var AddToFileList: function(var FileList: pFileList;
    Filename       : PChar;
    var UserCancel : boolean): boolean; cdecl;

var FreeFileList: function(FileList: pFileList): boolean; cdecl;
{
FILELIST *HDropToFileList(HDROP hDrop);
FILELIST *CmdLineToFileList(char *szCmdLine);

// Call-throughs to PGPcl
UINT PGPscQueryCacheSecsRemaining (void);
BOOL PGPscPreferences(HWND hwnd,void *PGPsc,void *PGPtls);
BOOL PGPscAbout (HWND hwnd,void *PGPsc);

// Returns filename from path
char *JustFile(char *filename);
}
// Grabs preferences from pref file
var GetValidityDrawPrefs: function(P: pPGPsc; var InputBool: boolean): boolean;cdecl;

var GetMarginalInvalidPref: function(P: pPGPsc; var InputBool: boolean): boolean;cdecl;

var GetSecureViewerPref: function(P: pPGPsc): boolean;cdecl;

// Views the clipboard
var LaunchInternalViewer: function(P: pPGPsc; Parent: hWnd): DWORD;cdecl;

// Use secure viewer and/or anti-tempest font
var TempestViewer: function(P: pPGPsc;
    Parent       : hWnd;
    pInput       : PChar;
    InSize       : DWORD;
    Warn         : Boolean
    ): Cardinal;cdecl;

{
// Launches Keys/Tools
BOOL DoLaunchNet(HWND hwnd);
BOOL DoLaunchKeys(HWND hwnd);
BOOL DoLaunchTools(HWND hwnd);
BOOL DoLaunchDisk(HWND hwnd);

// Wipe free space on disk
void FreeSpaceWipeWizard(HWND hwnd, void* PGPsc);
BOOL FreeSpaceWipeCmdLine(HWND hwnd, void* PGPsc, char *Drive);

// Detects if window exists
BOOL WindowExists (LPSTR lpszClassName, LPSTR lpszWindowTitle);
}
// Sends signature info to PGPlog
var SendPGPlogMsg: function(Parent: hWnd; Msg: PChar): boolean; cdecl;

// Returns path for PGPkeys
//procedure PGPpath(Path: PChar);cdecl;
{
// Copy/Paste into current window
BOOL DoCopy(HWND hwnd,void *PGPsc,BOOL bUseCurrent,HWND *hwndFocus);
void DoPaste(BOOL bUseCurrent,HWND hwndFocus);
void DoFocus(BOOL bUseCurrent,HWND hwndFocus);

// Auto-update keyring routines
void StartUpdateTimer(HWND hwnd, UINT *pnLaunchKeysTimer);
VOID CALLBACK UpdateTimerProc(HWND hwnd,
							  UINT uMsg,
							  UINT idEvent,
							  DWORD dwTime);

OUTBUFFLIST *MakeOutBuffItem(OUTBUFFLIST **obl);

BOOL ConcatOutBuffList(void *PGPsc,
					   OUTBUFFLIST *obl,
					   char **pBuff,
					   DWORD *dwBuffSize,
					   BOOL *FYEO);

#ifdef WIN32
#define DDElogServer "PGPlog"
#else
#define DDElogServer "PGPlog16"
#endif
#define DDElogTopic "Operation_Results"
}

function LoadPGPsc: Boolean; //returns True if API successfully loaded
procedure UnLoadPGPsc;
function PGPscLoaded: Boolean; //returns if the API is loaded

implementation

var
  PGPscLib       : HModule;

function PGPscLoaded: boolean;
begin
  result := PGPscLib >= 32;
end;

function LoadPGPsc: boolean;
begin
  // try to load PGP 6.5 version first, then PGP 6.0.x version
  PGPscLib := LoadLibrary('pgpsc.dll');
  if not PGPscLoaded then
    PGPscLib := LoadLibrary('pgp60sc.dll');
   
  if PGPscLoaded then
  begin
    result := True;
    @InitPGPsc            := GetProcAddress(PGPscLib,'InitPGPsc');
    @UninitPGPsc          := GetProcAddress(PGPscLib,'UninitPGPsc');
    @LaunchInternalViewer := GetProcAddress(PGPscLib,'LaunchInternalViewer');
    @TempestViewer        := GetProcAddress(PGPscLib,'TempestViewer');
    //@FileWipe             := GetProcAddress(PGPscLib,'FileWipe');
    @DecryptClipboard     := GetProcAddress(PGPscLib,'DecryptClipboard');
    @AddKeyClipboard      := GetProcAddress(PGPscLib,'AddKeyClipboard');
    @ClipboardWipe        := GetProcAddress(PGPscLib,'ClipboardWipe');
    @EncryptClipboard     := GetProcAddress(PGPscLib,'EncryptClipboard');
    @SendPGPlogMsg        := GetProcAddress(PGPscLib,'SendPGPlogMsg');
    @GetValidityDrawPrefs := GetProcAddress(PGPscLib,'GetValidityDrawPrefs');
    @GetMarginalInvalidPref := GetProcAddress(PGPscLib,'GetMarginalInvalidPref');
    @GetSecureViewerPref  := GetProcAddress(PGPscLib,'GetSecureViewerPref');
    @EncryptFileList      := GetProcAddress(PGPscLib,'EncryptFileList');
    @DecryptFileList      := GetProcAddress(PGPscLib,'DecryptFileList');
    @AddKeyFileList       := GetProcAddress(PGPscLib,'AddKeyFileList');
    @WipeFileList         := GetProcAddress(PGPscLib,'WipeFileList');
    @AddToFileList        := GetProcAddress(PGPscLib,'AddToFileList');
    @FreeFileList         := GetProcAddress(PGPscLib,'FreeFileList');
  end
  else
    result := false;
end;

procedure UnLoadPGPsc;
begin
  if PGPscLoaded then
  begin
    if PGPscLib <> 0 then
      FreeLibrary(PGPscLib);
      PGPscLib := 0;
  end;
end;

initialization
  LoadPGPsc;
  
finalization
  if PGPscLoaded then
    UnLoadPGPsc;

end.

