{************************************************************************}
{ spgpAnalysis - data analyzing functions                                }
{                                                                        }
{ This unit requires one or more units from the Pascal translations of   }
{ the PGPsdk header files: <http://www.oz.net/~srheller/pgpsdk/>         }
{ 								         }
{ The original Pascal code is: spgpAnalysis.pas, released 10 Apr 2000.   }
{ The initial developer of the Pascal code is Steven R. Heller           }
{ <srheller@oz.net>                                                      }
{ 								         }
{ Portions created by Steven R. Heller are    				 }
{ Copyright (C) 2000 Steven R. Heller                                    }
{									 }
{ The contents of this file are used with permission, subject to         }
{ the Mozilla Public License Version 1.1 (the "License"); you may        }
{ not use this file except in compliance with the License. You may       }
{ obtain a copy of the License at                                        }
{ http://www.mozilla.org/MPL/MPL-1.1.html 	                         }
{ 									 }
{ Software distributed under the License is distributed on an 	         }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or         }
{ implied. See the License for the specific language governing           }
{ rights and limitations under the License.                              }
{                                                                        }
{************************************************************************}

unit spgpAnalysis;

interface

uses
  // PGPsdk
  pgpBase, pgpPubTypes, pgpEncodePas, pgpKeys, pgpErrors,
    pgpOptionList, pgpUtilities,
  // SPGP
  spgpUtil, spgpKeyUtil, spgpGlobals;

type
  pRecipsList = ^TRecipsList;
  TRecipsList = packed record
  RecipsGoodCount: PGPUint32;
  RecipsGood     : array[0..127] of shortstring;
  PassphraseCount: PGPUint32;
  RecipsBadCount : PGPUint32;
  RecipsBad      : array[0..127] of shortstring;
  end;

  function spgpanalyze(BufferIn: PChar): LongInt;stdcall;export;
  function spgpanalyzefile(FileIn: PChar): LongInt;stdcall;export;
  function spgpanalyzeex(BufferIn, BufferOut: PChar;BufferOutLen: LongInt):LongInt;stdcall;export;
  function spgpanalyzefileex(FileIn, BufferOut: PChar; BufferOutLen: LongInt):LongInt;stdcall;export;

implementation

uses SysUtils;

var
  FDataAnalysis: Longint;
  FRecips: TRecipsList;
  FWantRecips: boolean;
  FLexicalSectionFound: boolean;
  LibsAllLoaded: boolean;

function EventHandler(context: pPGPContext; event: pPGPEvent; UserValue: PGPUserValue): PGPError;
var
  p:		   		   PChar;
  pid: 	   		   PGPChar;
  ps:		   		   PGPSize;
  validity:   	 PGPVALIDITY;
  s,Passphrase:  string;
  t:             shortstring;
  n: 				     PGPUInt32;
  i: 				     integer;
  filt: 			   pPGPFilter;
  fKeySet: 		   pPGPKeySet;
  keyList: 		   pPGPKeyList;
  Key: 			     pPGPKey;
  iter: 			   pPGPKeyIter;
  keyid:         pPGPKeyID;
  tmp:           TPGPEventRecipientsData;
  memamt:        PGPUInt32;
  LexSecCount:   integer;
  KeyRec:        TKeyData;
begin
result := kPGPError_NoError;
LexSecCount := 0;

  case event^.etype of
 {!}kPGPEvent_ErrorEvent             :result := event^.data.errorData.error;//FError_message := 'Error: '+ PGPErrorMsg(event^.data.errorData.error);

    kPGPEvent_AnalyzeEvent           :
      if LexSecCount = 0 then
        FDataAnalysis := PGPAnalyzeType(event^.data.analyzeData);

    kPGPEvent_RecipientsEvent        :
    begin
  { RecipsGood }
      s := '';
      result := PGPCountKeys(event^.data.RecipientsData.recipientSet, n);
      if result <> 0 then exit;
      FRecips.RecipsGoodCount := n;
      { despite &%@?#! documentation, KeyCount is the total number
        of keys encrypted-to, not just the unavailable ones }
      FRecips.RecipsBadCount := event^.data.RecipientsData.KeyCount - n;
      if n > 0 then
      begin
        result := PGPOrderKeySet(event^.data.RecipientsData.recipientSet, kPGPUserIDOrdering, keylist);
        if result <> 0 then exit else
        try
          result := PGPNewKeyIter(keylist, iter);
          if result <> 0 then exit else
          try
            i := 0;
            while IsntPGPError(PGPKeyIterNext(iter, Key)) do
              begin
                if IsntPGPError(GetKeyData(context,event^.data.RecipientsData.recipientSet,Key,KeyRec))
                then FRecips.RecipsGood[i] := KeyRec.KeyID
                else FRecips.RecipsGood[i] := '';
                inc(i);
              end;
          finally
            PGPFreeKeyIter(iter);
          end;
        finally
          PGPFreeKeyList(keylist);
        end;
      end;

    FRecips.PassphraseCount := event^.data.RecipientsData.ConventionalPassphraseCount;

{    keyid := event^.data.RecipientsData.KeyIDArray;
    result := PGPGetKeyIDString(keyid^,
                                kPGPKeyIDString_Abbreviated,//show abbreviated KeyID
                                pid);
    FRecips.RecipsBad[0] := pid;
 }
    end;

    kPGPEvent_BeginLexEvent          :
      begin
        LexSecCount := event^.data.beginLexData.sectionNumber;
        FLexicalSectionFound := true;
      end;
  end;
end;

function MyEventHandler(context: pPGPContext; event: pPGPEvent; UserValue: PGPUserValue): PGPError; cdecl;
begin
  result := EventHandler(context, event, UserValue);
end;

(* ANALYZE BUFFER *)
function spgpanalyze(BufferIn: PChar): LongInt;stdcall;export;
var
  Context        : pPGPContext;
  KeySet         : pPGPKeySet;
  Options        : pPGPOptionList;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeySet := nil;
  Options := nil;

  result := Init(Context, KeySet, false, false);
  if result <> 0 then exit;

  try
    result :=
    PGPBuildOptionList(context,
      Options,
      [
      PGPOInputBuffer(context, BufferIn, length(BufferIn)),
      PGPODiscardOutput(context, PGPTrue),
      PGPOKeySetRef(context, KeySet),
      PGPOEventHandler(context, MyEventHandler, nil)
      ]);

    if result <> 0 then exit else

    { ignoring result here because (i) we're only trying to
      trigger the analyze event, (ii) it will complain about
      bad passphrase, etc. }
    {result := }PGPDecode(context, [Options]);

    if not FLexicalSectionFound
    then result := 4
    else result := FDataAnalysis;

    finally
      PGPFreeOptionList(options);
      QuitIt(Context, KeySet);
    end;
end;

(* ANALYZE FILE *)
function spgpanalyzefile(FileIn: PChar): LongInt;stdcall;export;
var
  Context        : pPGPContext;
  FileIn_spec    : pPGPFileSpec;
  KeySet         : pPGPKeySet;
  Options        : pPGPOptionList;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  FileIn_spec := nil;
  KeySet := nil;
  Options := nil;

  result := Init(Context, KeySet, false, false);
  if result <> 0 then exit else
  try
    { create input filespec from filename}
    result := PGPNewFileSpecFromFullPath(context,FileIn,FileIn_spec);

    if result <> 0 then exit;

    result :=
    PGPBuildOptionList(context,
      options,
      [
      PGPOInputFile(context, FileIn_spec),
      PGPODiscardOutput(context, PGPTrue),
      PGPOKeySetRef(context, KeySet),
      PGPOEventHandler(context, MyEventHandler, nil)
      ]);

    if result <> 0 then exit else

    PGPDecode(context, [options]);

    if not FLexicalSectionFound
    then result := 4
    else result := FDataAnalysis;

  finally
    PGPFreeOptionList(options);
    PGPFreeFileSpec(FileIn_spec);
    QuitIt(Context, KeySet);
  end;
end;

(* ANALYZE BUFFER - RETURN EXTENDED INFO. *)
function spgpanalyzeex(BufferIn,BufferOut: PChar; BufferOutLen: LongInt):LongInt;stdcall;export;
var
  i              : integer;
  j              : integer;
  s              : string;
  saved_result   : LongInt;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  FWantRecips := True;

  result := spgpanalyze(BufferIn);

  if (result = 4) or (result > 6) or (result < 0) then exit else
  saved_result := result;

    if FRecips.PassphraseCount > 0
    then result := 5//conventional encryption
    else
    if (FRecips.RecipsBadCount > 0)
    and (FRecips.RecipsGoodCount = 0)
    then result := 6//key encryption, but no available keys
    else result := saved_result;

    s := '';

  if (result = 0) or (result = 6) then begin

    j := 0;

    s := s + 'Keys_Unknown: ' + IntToStr(FRecips.RecipsBadCount) + #13#10;

    s := s + 'Keys_Known: ' + IntToStr(FRecips.RecipsGoodCount) + #13#10;

    for j := 0 to FRecips.RecipsGoodCount do
      if FRecips.RecipsGood[j] <> '' then
        s := s + FRecips.RecipsGood[j]+ #9;

    { trim off trailing tab }
    if s <> '' then
      while s[Length(s)] = #9 do
        s := Copy(s,0,Length(s) -1);
        
    s := s + #13#10;
  end;
  if Length(s) > BufferOutLen -1
  then result := Length(s)+1
  else StrPLCopy(BufferOut,s,BufferOutLen-1);

end;

(* ANALYZE FILE - RETURN EXTENDED INFO. *)
function spgpanalyzefileex(FileIn,BufferOut: PChar; BufferOutLen: LongInt):LongInt;stdcall;export;
var
  options: 		    pPGPOptionList;
  FileIn_spec:	  pPGPFileSpec;
  s: 				      string;
  i,j:				    integer;
  saved_result:   LongInt;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  FWantRecips := True;

  if (BufferOutLen <= 0) then BufferOutLen := 1;

  result := spgpanalyzefile(FileIn);

  if (result = 4) or (result > 6) or (result < 0) then exit else
  saved_result := result;

    if FRecips.PassphraseCount > 0
    then result := 5//conventional encryption
    else
    if (FRecips.RecipsBadCount > 0)
    and (FRecips.RecipsGoodCount = 0)
    then result := 6//key encryption, but no available keys
    else result := saved_result;

    s := '';
     
  if (result = 0) or (result = 6) then
  begin
    s := s + 'Keys_Unknown: ' + IntToStr(FRecips.RecipsBadCount) + #13#10;

    s := s + 'Keys_Known: ' + IntToStr(FRecips.RecipsGoodCount) + #13#10;

    for j := 0 to FRecips.RecipsGoodCount do
      if FRecips.RecipsGood[j] <> '' then
        s := s + FRecips.RecipsGood[j]+ #9;

    { trim off trailing tab }
    if s <> '' then
      while s[Length(s)] = #9 do
        s := Copy(s,0,Length(s) -1);

    s := s + #13#10;
  end;

  if Length(s) > BufferOutLen -1
  then result := Length(s)+1
  else StrPLCopy(BufferOut,s,BufferOutLen-1);

end;

initialization
  // check all pgp units used by this unit
  LibsAllLoaded := pgpEncodeLoaded and
                   pgpKeysLoaded and
                   pgpErrorsLoaded and
                   pgpOptionListLoaded and
                   pgpUtilitiesLoaded;

end.
