{************************************************************************}
{ spgpDecrypt - decryption & signature verification functions            }
{                                                                        }
{ This unit requires one or more units from the Pascal translations of   }
{ the PGPsdk header files: <http://www.oz.net/~srheller/pgpsdk/>         }
{ 								                                                       }
{ The original Pascal code is: spgpDecrypt.pas, released 10 Apr 2000.    }
{ The initial developer of the Pascal code is Steven R. Heller           }
{ <srheller@oz.net>                                                      }
{ 								                                                       }
{ Portions created by Steven R. Heller are    				                   }
{ Copyright (C) 2000 Steven R. Heller                                    }
{									                                                       }
{ The contents of this file are used with permission, subject to         }
{ the Mozilla Public License Version 1.1 (the "License"); you may        }
{ not use this file except in compliance with the License. You may       }
{ obtain a copy of the License at                                        }
{ http://www.mozilla.org/MPL/MPL-1.1.html 	                             }
{ <http://www.mozilla.org/MPL/MPL-1.1.html> 	                           }
{ 									                                                     }
{ Software distributed under the License is distributed on an 	         }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or         }
{ implied. See the License for the specific language governing           }
{ rights and limitations under the License.                              }
{                                                                        }
{************************************************************************}

unit spgpDecrypt;

interface

uses
  // Delphi
  SysUtils,
  // PGPsdk
  pgpBase, pgpPubTypes, pgpOptionList, pgpEncodePas, pgpErrors,
    pgpUtilities, pgpKeys, pgpFeatures,
  // SPGP
  spgpGlobals, spgpKeyUtil, spgpUtil,
  // always last
  PGPsdk;

  function spgpdecode(BufferIn, BufferOut: PChar; BufferOutLen: LongInt; Pass, SigProps: PChar): LongInt;stdcall;export;
  function spgpdecodefile(FileIn, FileOut, Pass, SigProps: PChar): LongInt;stdcall;export;
  function spgpdetachedsigverify(SigFile, SignedFile, SigProps: PChar):LongInt;stdcall;export;

implementation

var
  FSigned        : boolean;
  FSignature     : TSig;
  FDataAnalysis  : LongInt;
  LibsAllLoaded  : boolean;

function makepropstring: string;
var
  S : string;
begin
  if not FSigned
  then S := '1'//'SIGNED_NOT'+#9
  else
  begin
  (* check signature *)
    (* generate the key/sig properties string *)
    if (FSignature.KeyID <> '')
    and not FSignature.KeyIsOnRing
    and not FSignature.Checked
    then S := '3'//'SIGNED_NO_KEY'
    else
    if FSignature.Verified then
      S := '0'//'SIGNED_GOOD'
    else
      S := '2';//'SIGNED_BAD';

    (* extended signature information *)
    { IDs }
    S := S + #9 + FSignature.UserID;
    S := S + #9 + FSignature.KeyID;
    S := S + #9 + FSignature.datetimestr;
    S := S + #9 + IntToStr(FSignature.datetimeint);
    S := S + #9 + IntToStr(Integer(FSignature.Checked));
    { signing key is available & message properly formatted? }
    S := S + #9 + IntToStr(Integer(FSignature.Verified));
    { signature is kosher? }

    S := S + #9 + FSignature.keyValidity;

    { key problems }
    S := S + #9 + IntToStr(Integer(FSignature.keyRevoked));
    S := S + #9 + IntToStr(Integer(FSignature.keyDisabled));
    S := S + #9 + IntToStr(Integer(FSignature.keyExpired));
  end;
  result := S;
end;

function EventHandler(Context: pPGPContext; Event: pPGPEvent; UserValue: PGPUserValue):PGPError;
var
  keyid          : pPGPKeyID;
  LexSecCount    : integer;
  p              : PChar;
  pid 	   		   : PGPChar;
  ps		   		   : PGPSize;
  validity    	 : PGPVALIDITY;
begin
  result := kPGPError_NoError;
  LexSecCount := 0;
  p := nil;

  case event^.etype of
    kPGPEvent_ErrorEvent             :result := Event^.data.errorData.error;//FError_message := 'Error: '+ PGPErrorMsg(event^.data.errorData.error);
    kPGPEvent_PassphraseEvent        :result := kPGPError_BadPassphrase;// -11500;

    kPGPEvent_AnalyzeEvent           :
      if LexSecCount = 0 then
        FDataAnalysis := PGPAnalyzeType(event^.data.analyzeData);

    kPGPEvent_SignatureEvent         :
  	begin
			FSignature.KeyID := '';
  		FSignature.UserID := '';
      FSignature.DateTimeStr := '';
  		FSignature.DateTimeInt := 0;
  		FSignature.checked := false;
  		FSignature.verified := false;
  		FSignature.keyRevoked := false;
  		FSignature.keyDisabled := false;
  		FSignature.keyExpired := false;
  		FSignature.keyValidity := '';

      FSignature.keyIsOnRing := false;

      FSigned := true;

      //result :=
    	PGPGetKeyIDString(event^.data.signatureData.signingKeyID,
        kPGPKeyIDString_Abbreviated, pid);
    	FSignature.KeyID := StrPas(pid);

      { check to see if signing key is on ring, exit if not -- all the
        rest is pointless if it's not there }
      if Boolean(KeyIsOnRing(context, pPGPKeySet(UserValue), FSignature.KeyID)) then
      begin
        FSignature.keyIsOnRing := true;
        if result <> 0 then exit;

        GetMem(p, 256);

        try
          //result :=
          PGPGetPrimaryUserIDNameBuffer(event^.data.signatureData.signingKey, 1028, p, ps);
      	  FSignature.UserId := StrPas(p);
        finally
          FreeMem(p);
        end;

      	FSignature.DateTimeInt := PGPGetStdTimeFromPGPTime(event^.data.signatureData.creationtime);
        LocalTime(PGPGetStdTimeFromPGPTime(event^.data.signatureData.creationtime),
          FSignature.DateTimeStr);
        //if result <> 0 then exit;

      	FSignature.keyRevoked := (event^.data.signatureData.keyRevoked = PGPTrue);
      	FSignature.keyDisabled := (event^.data.signatureData.keyDisabled = PGPTrue);
      	FSignature.keyExpired := (event^.data.signatureData.keyExpired = PGPTrue);

      	PGPGetPrimaryUserIDValidity(event^.data.signatureData.signingKey,validity);
        FSignature.keyValidity := IntToStr(validity);

      	FSignature.checked := (event^.data.signatureData.checked = PGPTrue);
      	{ signing key is available & message properly formatted? }
      	FSignature.verified := (event^.data.signatureData.verified = PGPTrue);
      	{ signature is kosher? }
        end
    end;

    kPGPEvent_BeginLexEvent          :
      LexSecCount := event^.data.beginLexData.sectionNumber;
    kPGPEvent_EndLexEvent            :;
    kPGPEvent_DetachedSignatureEvent :;//showmessage(PGPDescribeEvent(event^.etype));
  end;
end;

function MyEventHandler(Context: pPGPContext; Event: pPGPEvent; UserValue: PGPUserValue): PGPError; cdecl;
begin
  result := EventHandler(Context, Event, UserValue);
end;

(* DECRYPT/VERIFY BUFFER *)
function spgpdecode(BufferIn, BufferOut: PChar; BufferOutLen: LongInt; Pass, SigProps: PChar): LongInt;stdcall;export;
var
  Context        : pPGPContext;
  KeySet         : pPGPKeySet;
  Options        : pPGPOptionList;
  Output_size    : PGPSize;
  p              : Pointer;
  s              : shortstring;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeySet := nil;
  Options := nil;
  p := nil;
  s := '';
  FSigned := false;

  result := Init(Context, Keyset, false, false);
  if result <> 0 then exit;

  try
    result :=
    PGPBuildOptionList(context, options,
      [
      PGPOInputBuffer(context, BufferIn, StrLen(BufferIn)),
      PGPOAllocatedOutputBuffer(context, p, BufferOutLen, output_size),
      PGPOKeySetRef(context, KeySet),
      PGPOPassphrase(context, Pass),
      PGPOEventHandler(context, MyEventHandler, KeySet)
      ]);

    if result <> 0 then exit;

    result := PGPDecode(Context, [Options]);

    { sig properties string stuff }
    s := makepropstring;

    { copy string to PChar }
    StrPLCopy(SigProps, s, 255);

    (* write to buffer, check length, etc. *)
    if result = kPGPError_OutputBufferTooSmall then
      result := output_size + 1;
    if result <> 0 then exit;

    if p <> nil then
    begin
      { error code if output is larger than caller's buffer }
      if output_size > BufferOutLen - 1
      then
        result := output_size + 1
      else
        StrLCopy(BufferOut, p, output_size);
    end
    else
      result := -1;

    if (result = 0) or (result = -1) then
    (* LAST CHECK: what did the analysis event say about input? *)
    case FDataAnalysis of
      0:  result := kPGPError_NoError;           { Encrypted }
      1:  result := kPGPError_NoError;           { Signed }
      2:  result := SPGPError_DataIsDetachedSig; { Detached Signature }
      3:  result := SPGPError_DataIsKey;         { Key }
      4:  result := SPGPError_DataIsUnknown;     { Unknown }
    end;

  finally
    PGPFreeData(p);
    PGPFreeOptionList(options);
    QuitIt(Context, KeySet);
  end;
end;

(* DECRYPT FILE *)
function spgpdecodefile(FileIn, FileOut, Pass, SigProps: PChar): LongInt;stdcall;export;
var
  Context        : pPGPContext;
  FileIn_spec    : pPGPFileSpec;
  FileOut_spec   : pPGPFileSpec;
  KeySet         : pPGPKeySet;
  Options        : pPGPOptionList;
  s              : shortstring;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeySet := nil;
  Options := nil;
  FileIn_spec := nil;
  FileOut_spec := nil;
  s := '';
  FSigned := false;

  result := Init(Context, KeySet, false, false);
  if result <> 0 then exit;

  try
    { create input filespec from filename}
    result := PGPNewFileSpecFromFullPath(context, FileIn, FileIn_spec);
    if result <> 0 then exit;

    { create output filespec from filename}
    result := PGPNewFileSpecFromFullPath(context, FileOut, FileOut_spec);
    if result <> 0 then exit;

    result :=
    PGPBuildOptionList(context,
      Options,
      [
      PGPOInputFile(context, FileIn_spec),
      PGPOOutputFile(context, FileOut_spec),
      PGPOKeySetRef(context, KeySet),
      PGPOPassphrase(context, Pass),
      PGPOEventHandler(context, MyEventHandler, KeySet)
      ]);
    if result <> 0 then exit;

    result := PGPDecode(context, [Options]);

    { sig properties string stuff }
    s := makepropstring;

    { copy string to PChar }
    StrPLCopy(SigProps, s, 255);

    if (result = 0) or (result = -1) then
    (* LAST CHECK: what did the analysis event say about input? *)
    case FDataAnalysis of
      0:  result := kPGPError_NoError;           { Encrypted }
      1:  result := kPGPError_NoError;           { Signed }
      2:  result := SPGPError_DataIsDetachedSig; { Detached Signature }
      3:  result := SPGPError_DataIsKey;         { Key }
      4:  result := SPGPError_DataIsUnknown;     { Unknown }
    end;

  finally
    PGPFreeFileSpec(FileIn_spec);
    PGPFreeFileSpec(FileOut_spec);
    PGPFreeOptionList(Options);
    QuitIt(Context, KeySet);
  end;
end;

(* VERIFY DETACHED SIGNATURE *)
function spgpdetachedsigverify(SigFile, SignedFile, SigProps: PChar):LongInt;stdcall;export;
var
  Context        : pPGPContext;
  KeySet         : pPGPKeySet;
  Options  		   : pPGPOptionList;
  SigFile_spec   : pPGPFileSpec;
  SignedFile_spec: pPGPFileSpec;
  s              : shortstring;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeySet := nil;
  Options := nil;
  SigFile_spec := nil;
  SignedFile_spec := nil;
  s := '';
  FSigned := false;

  result := Init(Context, KeySet, false, false);
  if result <> 0 then exit;

  try
    { create input filespec from filename}
    result := PGPNewFileSpecFromFullPath(context, SigFile, SigFile_spec);
    if result <> 0 then exit;

    { create output filespec from filename}
    result := PGPNewFileSpecFromFullPath(context, SignedFile, SignedFile_spec);
    if result <> 0 then exit;

    result :=
    PGPBuildOptionList(context,
      options,
      [
      PGPOInputFile(context, SigFile_spec),
      PGPODetachedSig(context, [PGPOInputFile(context, SignedFile_spec)]),
      PGPOKeySetRef(context, KeySet),
      PGPOEventHandler(context, MyEventHandler, KeySet)
      ]);
    if result <> 0 then exit else

    result := PGPDecode(context, [options]);

    { sig properties string stuff }
    S := makepropstring;

    { copy string to PChar }
    StrPLCopy(SigProps, S, 255);

    if (result = 0) or (result = -1) then
    (* LAST CHECK: what did the analysis event say about input? *)
    case FDataAnalysis of
      0:  result := SPGPError_DataIsEncrypted;{ Encrypted }
      1:  result := SPGPError_DataIsSigned;   { Signed }
      2:  result := kPGPError_NoError;        { Detached Signature }
      3:  result := SPGPError_DataIsKey;      { Key }
      4:  result := SPGPError_DataIsUnknown;  { Unknown }
    end;

  finally
    PGPFreeFileSpec(SigFile_spec);
    PGPFreeFileSpec(SignedFile_spec);
    PGPFreeOptionList(options);
    QuitIt(Context, Keyset);
  end;
end;

initialization
  // check all pgp units used by this unit
  LibsAllLoaded := pgpEncodeLoaded and
                   pgpKeysLoaded and
                   pgpErrorsLoaded and
                   pgpOptionListLoaded and
                   pgpUtilitiesLoaded and
                   pgpFeaturesLoaded;

end.
