{************************************************************************}
{ spgpEncrypt - encryption & signing functions                           }
{                                                                        }
{ This unit requires one or more units from the Pascal translations of   }
{ the PGPsdk header files: <http://www.oz.net/~srheller/pgpsdk/>         }
{ 								                                                       }
{ The original Pascal code is: spgpEncrypt.pas, released 10 Apr 2000.    }
{ The initial developer of the Pascal code is Steven R. Heller           }
{ <srheller@oz.net>                                                      }
{ 								                                                       }
{ Portions created by Steven R. Heller are    				                   }
{ Copyright (C) 2000 Steven R. Heller                                    }
{									                                                       }
{ The contents of this file are used with permission, subject to         }
{ the Mozilla Public License Version 1.1 (the "License"); you may        }
{ not use this file except in compliance with the License. You may       }
{ obtain a copy of the License at                                        }
{ http://www.mozilla.org/MPL/MPL-1.1.html 	                             }
{ 									                                                     }
{ Software distributed under the License is distributed on an 	         }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or         }
{ implied. See the License for the specific language governing           }
{ rights and limitations under the License.                              }
{                                                                        }
{************************************************************************}

unit spgpEncrypt;

interface

uses
  // Delphi
  SysUtils,
  // PGPsdk
  pgpBase, pgpPubTypes, pgpUserInterface, pgpOptionList, pgpClient,
    pgpEncodePas, pgpErrors, pgpUtilities, pgpKeys, pgpFeatures,
  // SPGP
  spgpGlobals, spgpKeyUtil, spgpUtil,
  // always last
  PGPsdk;

{ encode }
  function spgpencode(BufferIn, BufferOut: PChar; BufferOutLen: LongInt;
           Encrypt, Sign, SignAlg, ConventionalEncrypt, ConventionalAlg, Armor,
           TextMode, Clear: LongInt; CryptKeyID, SignKeyID, SignKeyPass,
           ConventionalPass, Comment: PChar): LongInt;stdcall;export;
  function spgpencodefile(FileIn, FileOut: PChar; Encrypt, Sign, SignAlg,
           ConventionalEncrypt, ConventionalAlg, Armor, TextMode,
           Clear: LongInt; CryptKeyID, SignKeyID, SignKeyPass, ConventionalPass,
           Comment: PChar): LongInt;stdcall;export;

{ encode Mk II }
  function spgp_encode(BufferIn, BufferOut: PChar; BufferOutLen: LongInt;
           Encrypt, Sign, SignAlg, ConventionalEncrypt, ConventionalAlg, Armor,
           TextMode, Clear, Compress, EyesOnly, MIME: LongInt; CryptKeyID,
           SignKeyID, SignKeyPass, ConventionalPass, Comment, MIMESeparator: PChar):
           LongInt;stdcall;export;
  function spgp_encodefile(FileIn, FileOut: PChar; Encrypt, Sign, SignAlg,
           ConventionalEncrypt, ConventionalAlg, Armor, TextMode, Clear,
           Compress, EyesOnly, MIME: LongInt; CryptKeyID, SignKeyID,
           SignKeyPass, ConventionalPass, Comment, MIMESeparator: PChar):
           LongInt;stdcall;export;

{ encode UI }
  function spgpuiencode(BufferIn, BufferOut: PChar; BufferOutLen: LongInt;
           Encrypt, Sign, SignAlg, ConventionalEncrypt, ConventionalAlg, Clear,
           Compress, EyesOnly, MIME: LongInt; Comment, MIMESeparator: PChar;
           WinHandle: LongInt): LongInt;stdcall;export;
  function spgpuiencodefile(FileIn, FileOut: PChar; Encrypt, Sign, SignAlg,
           ConventionalEncrypt, ConventionalAlg, Armor, TextMode, Clear,
           Compress, EyesOnly, MIME: LongInt; Comment, MIMESeparator: PChar;
           WinHandle: LongInt): LongInt;stdcall;export;

{ detached signature }
  function spgpdetachedsigcreate(FileIn, SigFile, SignKeyID, SignKeyPass,
           Comment: PChar; SignAlg,Armor: LongInt):LongInt;stdcall;export;

implementation

var
  LibsAllLoaded  : boolean;
  PGPversion5    : boolean;

(* ENCRYPT/SIGN BUFFER - with User Interface *)
function spgpuiencode(
         BufferIn            : PChar;
         BufferOut           : PChar;
         BufferOutLen        : LongInt;
         Encrypt             : LongInt;
         Sign                : LongInt;
         SignAlg             : LongInt;
         ConventionalEncrypt : LongInt;
         ConventionalAlg     : LongInt;
         Clear               : LongInt;
         Compress            : LongInt;
         EyesOnly            : LongInt;
         MIME                : LongInt;
         Comment             : PChar;
         MIMESeparator       : PChar;
         WinHandle           : LongInt
         ): LongInt; stdcall; export;
var
  CipherAlg:       PGPCipherAlgorithm;
  Context:         pPGPContext;
  ConventionalPassphrase: PChar;
  CryptKey:			   pPGPKeySet;
  EncryptOptions:  pPGPOptionList;
  GeneralOptions:  pPGPOptionList;
  HashAlg:         PGPHashAlgorithm;
  HashAlgDefault:  PGPHashAlgorithm;
  Key:             TKeyData;
  LoadRings       : boolean;// load default keyrings?
  LocalKeySet:     pPGPContext;
  MIMEOptions:	   pPGPOptionList;
  MIMESep:         TMimeSepArray;//array[0..80]of char
  mimeBodyOffset:  PGPSIZE;
  OutputSize: 	   PGPSize;
  pOutput: 				 Pointer;
  s:					     string;
  SignKeyPassphrase: PChar;
  SignOptions:	   pPGPOptionList;
  SignKey:				 pPGPKey;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  ConventionalPassphrase := nil;
  CryptKey := nil;
  EncryptOptions := nil;
  GeneralOptions := nil;
  LocalKeySet := nil;
  MIMEOptions := nil;
  pOutput := nil;
  SignKeyPassphrase := nil;
  SignOptions := nil;
  SignKey := nil;
  LoadRings := true;

  // version check: UI not available before PGP 6
  if PGPversion5 then
    result := kPGPError_IncompatibleAPI;
  if result <> 0 then exit;

  result := Init_Alt(Context, LocalKeySet, (not Boolean(ConventionalEncrypt) or Boolean(Sign)), false, false);
  if result <> 0 then exit;
  if Assigned(PGPsdkUILibInit) then
    result := PGPsdkUILibInit;

  if result <> 0 then exit;

  try
    // ENCRYPT
    if Boolean(encrypt) or Boolean(ConventionalEncrypt) then
    begin
      if not Boolean(ConventionalEncrypt) then
      begin
        { get public key for encryption }
        result := PGPRecipientDialog(Context, LocalKeySet, PGPTrue,
          CryptKey,
          [
          PGPOUIParentWindowHandle(Context, WinHandle)
          //PGPOUIEnforceAdditionalRecipientRequests(Context, kPGPARREnforcement_Warn, PGPTrue)
          ]);

        if result <> 0 then exit;

        result :=
        PGPBuildOptionList(Context,
          EncryptOptions,
          [
          PGPOEncryptToKeyset(Context, CryptKey)
          ]);
        end{ if not ConventionalEncrypt }
        else
        begin
          { convert ciphering algorithm to proper type }
          case ConventionalAlg of
            1: CipherAlg := kPGPCipherAlgorithm_IDEA;
            2: CipherAlg := kPGPCipherAlgorithm_3DES;
            3: CipherAlg := kPGPCipherAlgorithm_CAST5
          else
            CipherAlg := kPGPCipherAlgorithm_IDEA;
          end;

          result := PGPBuildOptionList(Context, EncryptOptions,
            [PGPOCipherAlgorithm(Context, CipherAlg)]);

          if result <> 0 then exit;

          { get conventional-encryption passphrase -- use UI }
         { if PGPversion5 then
          begin
            GetMem(ConventionalPassphrase, 256);

            result :=
            PGPcomdlgGetPhrase(
              Context,
              WinHandle,
              PChar('&Enter conventional encryption passphrase:'),
              ConventionalPassphrase,
              nil,//KeySet,
              nil,//@YeKeyID,
              0,
              SignKey,// just using this to fill the slot, it should be nil
              nil,
              PGPCL_ENCRYPTION);
          end
          else
         } result := PGPConventionalEncryptionPassphraseDialog(Context,
            [
            PGPOUIOutputPassphrase(Context, ConventionalPassphrase),
            PGPOUIMinimumPassphraseLength(Context, 1),
            PGPOUIDialogPrompt(Context, PChar('&Enter conventional passphrase')),
            PGPOUIParentWindowHandle(Context, WinHandle),
            PGPOUIWindowTitle(Context, PChar('PGP - Enter Passphrase'))
            ]);

          if result <> 0 then exit;

          result := PGPAppendOptionList(EncryptOptions,
            [
            PGPOConventionalEncrypt(Context, [PGPOPassphrase(Context, ConventionalPassphrase)])
            ])
      end;
    end{ if encrypt }
    else
    EncryptOptions := PGPONullOption(Context);

    if result <> 0 then exit;

   (* -- SIGN -- *)
    if Boolean(sign) then
    begin
      { get signing key & passphrase }
      result := PGPSigningPassphraseDialog(Context,
        LocalKeySet, SignKey,
        [//PGPOUIDefaultKey(Context, SignKey),
        PGPOUIFindMatchingKey(Context, PGPTrue),
        PGPOUIParentWindowHandle(Context, WinHandle),
        PGPOUIOutputPassphrase(Context, SignKeyPassphrase)]);

      if result <> 0 then exit else

      (* DH/DSS keys can only sign with SHA, so all this
      effort is wasted unless you're using RSA *)

      { determine default hash alg for signing key }
      result := GetKeyData(Context, LocalKeyset, SignKey, Key);

      if result <> 0 then exit;

      case Key.KeyAlg of
        kPGPPublicKeyAlgorithm_RSA            : HashAlgDefault := kPGPHashAlgorithm_MD5;
        kPGPPublicKeyAlgorithm_RSAEncryptOnly : HashAlgDefault := kPGPHashAlgorithm_MD5;
        kPGPPublicKeyAlgorithm_RSASignOnly    : HashAlgDefault := kPGPHashAlgorithm_MD5;
        //kPGPPublicKeyAlgorithm_ElGamal:         HashAlgDefault := kPGPHashAlgorithm_SHA;
        kPGPPublicKeyAlgorithm_DSA            : HashAlgDefault := kPGPHashAlgorithm_SHA
      else
        HashAlgDefault := kPGPHashAlgorithm_MD5;
      end;

      { convert hashing algorithm to proper type }
      case SignAlg of
        0: HashAlg := HashAlgDefault;
        1: HashAlg := kPGPHashAlgorithm_MD5;
        2: HashAlg := kPGPHashAlgorithm_SHA;
        3: HashAlg := kPGPHashAlgorithm_RIPEMD160;
        4: HashAlg := kPGPHashAlgorithm_SHADouble
      else
        HashAlg := HashAlgDefault;
      end;

      if result <> 0 then exit;

      result := PGPBuildOptionList(Context, SignOptions,
        [
        PGPOHashAlgorithm(Context, HashAlg),
        PGPOClearSign(Context, Clear)
        ]);

      if result <> 0 then exit;

        result := PGPAppendOptionList(SignOptions,
          [
          PGPOSignWithKey(Context, SignKey, [PGPOPassphrase(Context, SignKeyPassphrase)])
          ])
    end
    else
    SignOptions := PGPONullOption(Context);

    if result <> 0 then exit else
    { MIME options are separate because if they're present in GeneralOptions they'll
      trigger MIME-encoding, even when set to False. }
    if Boolean(MIME) then
    result :=
    PGPBuildOptionList(Context,
      MIMEOptions,
      [
      PGPOOmitMIMEVersion(Context, PGPTrue),
      PGPOPGPMIMEEncoding(Context, PGPTrue, mimeBodyOffset, MIMESep)
      ])
    else
    MIMEOptions := PGPONullOption(Context);

    if result <> 0 then exit else
    result :=
    PGPBuildOptionList(Context,
       GeneralOptions,
       [
       PGPOInputBuffer(Context, BufferIn, length(BufferIn)),
       PGPOAllocatedOutputBuffer(Context, pOutput, BufferOutLen, OutputSize),
       PGPOArmorOutput(Context, PGPTrue), {-a}
       PGPODataIsASCII(Context, PGPTrue), {-t}
       PGPOCompression(Context, Compress),
       PGPOForYourEyesOnly(Context, EyesOnly),
       PGPOVersionString(Context, GlobalVersionString),
       PGPOCommentString(Context, Comment)//,
       //PGPOEventHandler(Context, MyEventHandler, nil)
       ]);

    if result <> 0 then exit else
    result := PGPEncode(Context, [GeneralOptions, EncryptOptions, SignOptions, MIMEOptions]);

    (* write to buffer, check length, etc. *)
    if result = kPGPError_OutputBufferTooSmall then result := OutputSize + 1;
    if result <> 0 then exit;

    if Boolean(MIME) then
    begin
      StrCopy(MimeSeparator, '');
      StrLCopy(MimeSeparator, MimeSep, kPGPMimeSeparatorSize-1);
    end;

    if pOutput <> nil then
    begin
      { error code if output is larger than caller's buffer }
      if OutputSize > BufferOutLen - 1
      then result := OutputSize + 1
      else StrLCopy(BufferOut, pOutput, OutputSize);
    end
    else
      result := -1;

  finally
    PGPFreeData(pOutput);
    //FillChar(SignKeyPass, Length(SignKeyPass), 0);
    //FillChar(ConventionalPass, Length(ConventionalPass), 0);
    PGPFreeData(ConventionalPassphrase);
    PGPFreeData(SignKeyPassphrase);
    PGPFreeKeyset(CryptKey);
    PGPFreeOptionList(GeneralOptions);
    PGPFreeOptionList(EncryptOptions);
    PGPFreeOptionList(SignOptions);
    PGPFreeOptionList(MIMEOptions);
    if Assigned(PGPsdkUILibCleanup) then
      PGPsdkUILibCleanup;
    QuitIt(Context, LocalKeySet);
  end;
end;

(* ENCRYPT/SIGN FILE -- with User Interface *)
function spgpuiencodefile(FileIn, FileOut: PChar; Encrypt, Sign, SignAlg,
         ConventionalEncrypt, ConventionalAlg, Armor, TextMode, Clear, Compress,
         EyesOnly, MIME: LongInt; Comment, MIMESeparator: PChar;
         WinHandle: LongInt): LongInt;stdcall;export;
var
  CipherAlg:       PGPCipherAlgorithm;
  Context:         pPGPContext;
  ConventionalPassphrase: PChar;
  CryptKey: 			 pPGPKeySet;
  EncryptOptions:  pPGPOptionList;
  FileInSpec:		   pPGPFileSpec;
  FileOutSpec:		 pPGPFileSpec;
  GeneralOptions:  pPGPOptionList;
  HashAlg:         PGPHashAlgorithm;
  HashAlgDefault:  PGPHashAlgorithm;
  Key:             TKeyData;
  LocalKeySet:     pPGPKeySet;
  MimeOptions:	   pPGPOptionList;
  mimeBodyOffset:  PGPSIZE;
  MIMESep:         TMimeSepArray;//array[0..80]of char
  s:               string;
  SignOptions:	   pPGPOptionList;
  SignKey:				 pPGPKey;
  SignKeyPassphrase: PChar;
begin
  result := kPGPError_NoError;
  
  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  Context := nil;
  ConventionalPassphrase := nil;
  CryptKey := nil;
  EncryptOptions := nil;
  FileInSpec := nil;
  FileOutSpec := nil;
  GeneralOptions := nil;
  LocalKeySet := nil;
  MimeOptions := nil;
  SignOptions := nil;
  SignKey := nil;
  SignKeyPassphrase := nil;

  // version check: UI not available before PGP 6
  if PGPversion5 then
      result := kPGPError_IncompatibleAPI;
  if result <> 0 then exit;

  //result := Init(Context, LocalKeySet, false, false);
  result := Init_Alt(Context, LocalKeySet, not Boolean(ConventionalEncrypt), false, false);
  if result <> 0 then exit;
  if Assigned(PGPsdkUILibInit) then
    result := PGPsdkUILibInit;

  try
    { create input filespec from filename}
    result := PGPNewFileSpecFromFullPath(Context, FileIn, FileInSpec);

    if result <> 0 then exit else

    { create output filespec from filename}
    result := PGPNewFileSpecFromFullPath(Context, FileOut, FileOutSpec);

    if result <> 0 then exit else

    if Boolean(encrypt) or Boolean(ConventionalEncrypt) then
    begin
      if not Boolean(ConventionalEncrypt) then
      begin
        { get public key for encryption }
        result := PGPRecipientDialog(Context, LocalKeySet, PGPTrue,
          CryptKey,
          [PGPOUIParentWindowHandle(Context, WinHandle)]);

        if result <> 0 then exit;

        result := PGPBuildOptionList(Context, EncryptOptions,
          [PGPOEncryptToKeyset(Context, CryptKey)]);
    end{ if not ConventionalEncrypt }
    else
    begin
      { convert ciphering algorithm to proper type }
      case ConventionalAlg of
        1: CipherAlg := kPGPCipherAlgorithm_IDEA;
        2: CipherAlg := kPGPCipherAlgorithm_3DES;
        3: CipherAlg := kPGPCipherAlgorithm_CAST5
      else
        CipherAlg := kPGPCipherAlgorithm_IDEA;
      end;

      result := PGPBuildOptionList(Context, EncryptOptions,
        [
        PGPOCipherAlgorithm(Context, CipherAlg)
        ]);

      if result <> 0 then exit;

      { get conventional-encryption passphrase -- use UI }
      result := PGPConventionalEncryptionPassphraseDialog(Context,
        [
        PGPOUIOutputPassphrase(Context, ConventionalPassphrase),
        PGPOUIMinimumPassphraseLength(Context, 1),
        PGPOUIDialogPrompt(Context, PChar('&Enter conventional passphrase')),
        PGPOUIParentWindowHandle(Context, WinHandle),
        PGPOUIWindowTitle(Context, PChar('PGP - Enter Passphrase'))
        ]);

      result := PGPAppendOptionList(EncryptOptions,
        [
        PGPOConventionalEncrypt(Context, [PGPOPassphrase(Context, ConventionalPassphrase)])
        ])
    end;//if conventional encrypt
  end{ if encrypt }
  else
    EncryptOptions := PGPONullOption(Context);

  if result <> 0 then exit;

  if Boolean(sign) then
  begin
    { get key & passphrase for signing key }
    result := PGPSigningPassphraseDialog(Context,
      LocalKeySet, SignKey,
      [
      PGPOUIDefaultKey(Context, SignKey),
      PGPOUIFindMatchingKey(Context, PGPTrue),
      PGPOUIParentWindowHandle(Context, WinHandle),
      PGPOUIOutputPassphrase(Context, SignKeyPassphrase)
      ]);

    if result <> 0 then exit else

    { determine default hash algorithm for signing key }
    result := GetKeyData(Context, LocalKeySet, SignKey, Key);

    if result <> 0 then exit;

    case Key.KeyAlg of
      kPGPPublicKeyAlgorithm_RSA:             HashAlgDefault := kPGPHashAlgorithm_MD5;
      kPGPPublicKeyAlgorithm_RSAEncryptOnly:  HashAlgDefault := kPGPHashAlgorithm_MD5;
      kPGPPublicKeyAlgorithm_RSASignOnly:     HashAlgDefault := kPGPHashAlgorithm_MD5;
      //kPGPPublicKeyAlgorithm_ElGamal:         HashAlgDefault := kPGPHashAlgorithm_SHA;
      kPGPPublicKeyAlgorithm_DSA:             HashAlgDefault := kPGPHashAlgorithm_SHA
    else
      HashAlgDefault := kPGPHashAlgorithm_MD5;
    end;

    { convert hashing algorithm to proper type }
    case SignAlg of
      0: HashAlg := HashAlgDefault;
      1: HashAlg := kPGPHashAlgorithm_MD5;
      2: HashAlg := kPGPHashAlgorithm_SHA;
      3: HashAlg := kPGPHashAlgorithm_RIPEMD160;
      4: HashAlg := kPGPHashAlgorithm_SHADouble
    else
      HashAlg := HashAlgDefault;
    end;

    if result <> 0 then exit else

    result := PGPBuildOptionList(Context, SignOptions,
      [
      PGPOHashAlgorithm(Context, HashAlg),
      PGPOClearSign(Context, Clear)
      ]);

    result := PGPAppendOptionList(SignOptions,
      [
      PGPOSignWithKey(Context, SignKey, [PGPOPassphrase(Context, SignKeyPassphrase)])
      ])
  end//if sign
  else
    SignOptions := PGPONullOption(Context);

  if result <> 0 then exit else

  { MIME options are separate because if they're present in GeneralOptions they'll
   trigger MIME-encoding, even when set to False. }
  if Boolean(MIME) then
    result := PGPBuildOptionList(Context, MimeOptions,
      [
      PGPOOmitMIMEVersion(Context, PGPTrue),
      PGPOPGPMIMEEncoding(Context, PGPTrue, mimeBodyOffset, MIMESep)
      ])
  else
    MimeOptions := PGPONullOption(Context);

  if result <> 0 then exit else

  PGPBuildOptionList(Context, GeneralOptions,
    [
    { files instead of buffers }
    PGPOInputFile(Context, FileInSpec),
    PGPOOutputFile(Context, FileOutSpec),
    PGPODataIsASCII(Context, TextMode), {-t}
    PGPOArmorOutput(Context, Armor), {-a}
    PGPOCommentString(Context, Comment),
    PGPOForYourEyesOnly(Context, PGPBoolean(EyesOnly)),
    PGPOCompression(Context, PGPBoolean(Compress)),
    PGPOVersionString(Context, GlobalVersionString)//,
    //PGPOEventHandler(Context, MyEventHandler, nil)
    ]);

  if result <> 0 then exit else

  result := PGPEncode(Context, [GeneralOptions, EncryptOptions, SignOptions, MimeOptions]);

  if result <> 0 then exit;

  if Boolean(MIME) then
  begin
    StrCopy(MimeSeparator, '');
    StrLCopy(MimeSeparator, MimeSep, kPGPMimeSeparatorSize-1);
  end;

  finally
    PGPFreeFileSpec(FileInSpec);
    PGPFreeFileSpec(FileOutSpec);
    //FillChar(SignKeyPass, SizeOf(SignKeyPass), 0);
    //FillChar(ConventionalPass, SizeOf(ConventionalPass), 0);
    PGPFreeData(ConventionalPassphrase);
    PGPFreeData(SignKeyPassphrase);
    PGPFreeKeyset(CryptKey);
    PGPFreeOptionList(GeneralOptions);
    PGPFreeOptionList(EncryptOptions);
    PGPFreeOptionList(SignOptions);
    PGPFreeOptionList(MimeOptions);
    if Assigned(PGPsdkUILibCleanup) then
      PGPsdkUILibCleanup;
    QuitIt(Context, LocalKeySet);
  end;
end;

(* ENCRYPT/SIGN BUFFER *)
function spgp_encode(BufferIn, BufferOut: PChar; BufferOutLen: LongInt; Encrypt,
         Sign, SignAlg, ConventionalEncrypt, ConventionalAlg, Armor, TextMode,
         Clear, Compress, EyesOnly, MIME: LongInt; CryptKeyID, SignKeyID, SignKeyPass,
         ConventionalPass, Comment, MIMESeparator: PChar): LongInt;stdcall;export;
var
  Context:         pPGPContext;
  KeySet:          pPGPKeySet;
  SignKey:				  pPGPKey;
  CryptKey:			    pPGPKeySet;
  p: 					      Pointer;
  output_size: 	    PGPSize;
  sign_options,
  encrypt_options,
  general_options,
  mime_options:	    pPGPOptionList;
  s:					      string;
  CipherAlg:        PGPCipherAlgorithm;
  HashAlg,
  HashAlgDefault:   PGPHashAlgorithm;
  Key:              TKeyData;
  LoadRings         : boolean;
  //debug
  MIMESep:          TMimeSepArray;//array[0..80]of char
  mimeBodyOffset:   PGPSIZE;
  i: integer;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeySet := nil;
  SignKey := nil;
  CryptKey := nil;
  p := nil;
  sign_options := nil;
  encrypt_options := nil;
  general_options := nil;
  mime_options := nil;
  LoadRings := true;

  //result := Init(Context, KeySet, false, false);

  result := Init_Alt(Context, KeySet, (not Boolean(ConventionalEncrypt) or Boolean(Sign)), false, false);

  if result <> 0 then exit;

{ check #1: at least one option }
{if not Boolean(Encrypt) and not Boolean(ConventionalEncrypt) and not Boolean(sign) then
  begin
    result := -101;//no options
    exit;
  end;
}
{ check #2: Key IDs are present & keys are on ring }
  if Boolean(encrypt) then
  begin
  if ZeroLength(CryptKeyID) then
    begin
      result := SPGPError_NoEncryptKey;//no encrypt key specified
      exit;
    end
  else
  if not KeysAreOnRing(context,KeySet,CryptKeyID) then
    begin
      result := SPGPError_EncryptKeyNotFound;//encrypt key not found
      exit;
    end;
end;

  if Boolean(sign) then
  begin
  if ZeroLength(SignKeyID) then
    begin
      result := SPGPError_NoSignKey;//no sign key specified
      exit;
    end
  else
  if not KeyIsOnRing(context,KeySet,SignKeyID) then
    begin
      result := SPGPError_SignKeyNotFound;//sign key not found
      exit;
    end;
  end;

  if result <> 0 then exit else

  { rock and roll! }
  try
    // SIGN
    if Boolean(sign) then
      begin

      if result <> 0 then exit else

      { get private key for signing }
      result := GetKeyByID(context,SignKey,LowerCase(StrPas(SignKeyID)),KeySet);

      if result <> 0 then exit else

      (* DH/DSS keys can only sign with SHA, so all this
         effort is wasted unless you're using RSA *)

      { determine default hash alg for signing key }
      result := GetKeyData(context,KeySet,SignKey,Key);
      if result <> 0 then exit;

      case Key.KeyAlg of
        kPGPPublicKeyAlgorithm_RSA:             HashAlgDefault := kPGPHashAlgorithm_MD5;
        kPGPPublicKeyAlgorithm_RSAEncryptOnly:  HashAlgDefault := kPGPHashAlgorithm_MD5;
        kPGPPublicKeyAlgorithm_RSASignOnly:     HashAlgDefault := kPGPHashAlgorithm_MD5;
        //kPGPPublicKeyAlgorithm_ElGamal:         HashAlgDefault := kPGPHashAlgorithm_SHA;
        kPGPPublicKeyAlgorithm_DSA:             HashAlgDefault := kPGPHashAlgorithm_SHA
        else
        HashAlgDefault := kPGPHashAlgorithm_MD5;
        end;

      { convert hashing algorithm to proper type }
      case SignAlg of
        0: HashAlg := HashAlgDefault;
        1: HashAlg := kPGPHashAlgorithm_MD5;
        2: HashAlg := kPGPHashAlgorithm_SHA;
        3: HashAlg := kPGPHashAlgorithm_RIPEMD160;
        4: HashAlg := kPGPHashAlgorithm_SHADouble
      else
        HashAlg := HashAlgDefault;
      end;

      if result <> 0 then exit else
      result :=
      PGPBuildOptionList(context,
                     sign_options,
                     [
                     PGPOHashAlgorithm(context, HashAlg),
                     PGPOClearSign(context, Clear),
                     PGPOSignWithKey(context, SignKey, [PGPOPassphrase(context, SignKeyPass)])
                     ]);
      end
    else
    sign_options := PGPONullOption(context);

    // ENCRYPT
    if Boolean(encrypt) or Boolean(ConventionalEncrypt) then
      begin

      if result <> 0 then exit else

      if not Boolean(ConventionalEncrypt) then
        begin

        { Signing is done with a key; encryption with a keyset,
          even when encrypting with a single key }
        if result <> 0 then exit else

        { create new empty keyset to hold encrypting key(s) }
        result := PGPNewKeySet(context,CryptKey);

        if result <> 0 then exit else

       { get public key for encryption }
        result := GetKeysByID(context,CryptKey,LowerCase(StrPas(CryptKeyID)),KeySet);

        if result <> 0 then exit else

        result :=
        PGPBuildOptionList(context,
                     encrypt_options,
                     [
                      PGPOEncryptToKeyset(context,CryptKey)
                     ]);
        end{ if not ConventionalEncrypt }
        else begin
        { convert ciphering algorithm to proper type }
        case ConventionalAlg of
          1: CipherAlg := kPGPCipherAlgorithm_IDEA;
          2: CipherAlg := kPGPCipherAlgorithm_3DES;
          3: CipherAlg := kPGPCipherAlgorithm_CAST5
        else
          CipherAlg := kPGPCipherAlgorithm_IDEA;
        end;

        result :=
        PGPBuildOptionList(context,
                     encrypt_options,
                     [
                      PGPOConventionalEncrypt(context, [PGPOPassphrase(context, ConventionalPass)]),
                      PGPOCipherAlgorithm(context, CipherAlg)
                     ]);
        end;
      end{ if encrypt }
    else
    encrypt_options := PGPONullOption(context);

    if result <> 0 then exit else
    { MIME options are separate because if they're present in general_options they'll
      trigger MIME-encoding, even when set to False. }
    if Boolean(MIME) then
    result :=
    PGPBuildOptionList(context,
                  mime_options,
                  [
                  PGPOOmitMIMEVersion(context, PGPTrue),
                  PGPOPGPMIMEEncoding(context, PGPTrue, mimeBodyOffset, MIMESep)
                  ]
                  )
    else
    mime_options := PGPONullOption(context);

    if result <> 0 then exit else
    result :=
    PGPBuildOptionList(context,
                  general_options,
                  [
                  PGPOInputBuffer(context, BufferIn, length(BufferIn)),
                  PGPOAllocatedOutputBuffer(context, p, BufferOutLen, output_size),
                  PGPOArmorOutput(context, Armor),{-a}
                  PGPODataIsASCII(context, TextMode),{-t}
                  PGPOCompression(context, Compress),
                  PGPOForYourEyesOnly(context, EyesOnly),
                  PGPOVersionString(context, GlobalVersionString),
                  PGPOCommentString(context, Comment)//,
                  //PGPOEventHandler(context, MyEventHandler, nil)
                  ]
                  );

    if result <> 0 then exit else
    result := PGPEncode(context, [general_options,encrypt_options,sign_options,mime_options]);

    (* write to buffer, check length, etc. *)
    if result = kPGPError_OutputBufferTooSmall then result := output_size + 1;
    if result <> 0 then exit;

    if Boolean(MIME) then 
      begin
      StrCopy(MimeSeparator,'');
      StrLCopy(MimeSeparator,MimeSep,kPGPMimeSeparatorSize-1);
      end;

    if p <> nil then
      begin
        { error code if output is larger than caller's buffer }
        if output_size > BufferOutLen - 1
        then
          result := output_size + 1
        else
        StrLCopy(BufferOut, p, output_size);
      end
    else
      result := -1;

    finally
      PGPFreeData(p);
      //FillChar(SignKeyPass,SizeOf(SignKeyPass),0);
      //FillChar(ConventionalPass,SizeOf(ConventionalPass),0);
      PGPFreeKeyset(CryptKey);
      PGPFreeOptionList(general_options);
      PGPFreeOptionList(encrypt_options);
      PGPFreeOptionList(sign_options);
      PGPFreeOptionList(mime_options);
      QuitIt(Context,KeySet);
    end;
end;

(* ENCRYPT/SIGN FILE *)
function spgp_encodefile(FileIn, FileOut: PChar; Encrypt, Sign, SignAlg, ConventionalEncrypt, 
         ConventionalAlg, Armor, TextMode, Clear, Compress, EyesOnly, MIME: LongInt;
         CryptKeyID, SignKeyID, SignKeyPass, ConventionalPass, Comment, 
         MIMESeparator: PChar): LongInt;stdcall;export;
var
  Context:          pPGPContext;
  CipherAlg:        PGPCipherAlgorithm;
  CryptKey: 			  pPGPKeySet;
  encrypt_options:	pPGPOptionList;
  FileIn_spec:		  pPGPFileSpec;
  FileOut_spec:		  pPGPFileSpec;
  general_options:	pPGPOptionList;
  HashAlg:          PGPHashAlgorithm;
  HashAlgDefault:   PGPHashAlgorithm;
  Key:              TKeyData;
  KeySet:           pPGPKeySet;
  mime_options:	    pPGPOptionList;
  mimeBodyOffset:   PGPSIZE;
  MIMESep:          TMimeSepArray;//array[0..80]of char
  s:                string;
  sign_options:	    pPGPOptionList;
  SignKey:				  pPGPKey;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  CryptKey := nil;
  encrypt_options := nil;
  FileIn_spec := nil;
  FileOut_spec := nil;
  general_options := nil;
  KeySet:= nil;
  mime_options := nil;
  sign_options := nil;
  SignKey := nil;

  //result := Init(Context,KeySet,false, false);
  result := Init_Alt(Context, KeySet, (not Boolean(ConventionalEncrypt) or Boolean(Sign)), false, false);

{ Check #1: action must be indicated: sign, crypt or conventional }
{  if not Boolean(encrypt) and not Boolean(ConventionalEncrypt)and not Boolean(sign) then
    begin
    result := -101;//error: no action (encrypt or sign) indicated
    exit;
    end;
}
  try
  { check #2: Key IDs are present & keys are on ring }
    if Boolean(encrypt) then
    begin
      if ZeroLength(CryptKeyID) then
      begin
        result := SPGPError_NoEncryptKey;//no encrypt key specified
        exit;
      end
      else
      if not KeysAreOnRing(context,KeySet,CryptKeyID) then
      begin
        result := SPGPError_EncryptKeyNotFound;//encrypt key not found
        exit;
      end;
    end;

    if Boolean(sign) then
    begin
      if ZeroLength(SignKeyID) then
      begin
        result := SPGPError_NoSignKey;//no sign key specified
        exit;
      end
      else
      if not KeyIsOnRing(context,KeySet,SignKeyID) then
      begin
        result := SPGPError_SignKeyNotFound;//sign key not found
        exit;
      end;
    end;

  if result <> 0 then exit else
  { create input filespec from filename}
  result := PGPNewFileSpecFromFullPath(context, FileIn, FileIn_spec);

  if result <> 0 then exit else

  { create output filespec from filename}
  result := PGPNewFileSpecFromFullPath(context, FileOut, FileOut_spec);

  if result <> 0 then exit else

  if Boolean(sign) then
  begin
    { get private key for signing }
    if not ZeroLength(SignKeyID) then
      result := GetKeyByID(context,SignKey,LowerCase(StrPas(SignKeyID)),KeySet);

    if result <> 0 then exit else

    { determine default hash alg for signing key }
    result := GetKeyData(context,KeySet,SignKey,Key);

    if result <> 0 then exit;

    case Key.KeyAlg of
      kPGPPublicKeyAlgorithm_RSA:             HashAlgDefault := kPGPHashAlgorithm_MD5;
      kPGPPublicKeyAlgorithm_RSAEncryptOnly:  HashAlgDefault := kPGPHashAlgorithm_MD5;
      kPGPPublicKeyAlgorithm_RSASignOnly:     HashAlgDefault := kPGPHashAlgorithm_MD5;
      //kPGPPublicKeyAlgorithm_ElGamal:         HashAlgDefault := kPGPHashAlgorithm_SHA;
      kPGPPublicKeyAlgorithm_DSA:             HashAlgDefault := kPGPHashAlgorithm_SHA
    else
      HashAlgDefault := kPGPHashAlgorithm_MD5;
    end;

    { convert hashing algorithm to proper type }
    case SignAlg of
      0: HashAlg := HashAlgDefault;
      1: HashAlg := kPGPHashAlgorithm_MD5;
      2: HashAlg := kPGPHashAlgorithm_SHA;
      3: HashAlg := kPGPHashAlgorithm_RIPEMD160;
      4: HashAlg := kPGPHashAlgorithm_SHADouble
    else
      HashAlg := HashAlgDefault;
    end;

    if result <> 0 then exit else

    result := PGPBuildOptionList(context, sign_options,
      [
      PGPOHashAlgorithm(context, HashAlg),
      PGPOClearSign(context, Clear),
      PGPOSignWithKey(context, SignKey, [PGPOPassphrase(context, SignKeyPass)])
      ]);
  end//if sign
  else
    sign_options := PGPONullOption(context);

  if result <> 0 then exit else

  if Boolean(encrypt) or Boolean(ConventionalEncrypt) then
  begin
    if not Boolean(ConventionalEncrypt) then
    begin
      { create a new empty keyset to hold recipient(s) key(s) }
      result := PGPNewKeySet(context,CryptKey);

      if result <> 0 then exit else

      { get public key for encryption }
      result := GetKeysByID(context,CryptKey,LowerCase(StrPas(CryptKeyID)),KeySet);

      if result <> 0 then exit else

      result := PGPBuildOptionList(context, encrypt_options,
        [PGPOEncryptToKeyset(context,CryptKey)]);
    end{ if not ConventionalEncrypt }
    else
    begin
      { convert ciphering algorithm to proper type }
      case ConventionalAlg of
        1: CipherAlg := kPGPCipherAlgorithm_IDEA;
        2: CipherAlg := kPGPCipherAlgorithm_3DES;
        3: CipherAlg := kPGPCipherAlgorithm_CAST5
      else
        CipherAlg := kPGPCipherAlgorithm_IDEA;
      end;

      result := PGPBuildOptionList(context, encrypt_options,
        [
        PGPOConventionalEncrypt(context, [PGPOPassphrase(context, ConventionalPass)]),
        PGPOCipherAlgorithm(context, CipherAlg)
        ]);
    end;

  end{ if encrypt }
  else
    encrypt_options := PGPONullOption(context);

  if result <> 0 then exit;

  { MIME options are separate because if they're present in general_options they'll
   trigger MIME-encoding, even when set to False. }
  if Boolean(MIME) then
    result := PGPBuildOptionList(context, mime_options,
      [
      PGPOOmitMIMEVersion(context, PGPTrue),
      PGPOPGPMIMEEncoding(context, PGPTrue, mimeBodyOffset, MIMESep)
      ])
  else
    mime_options := PGPONullOption(context);

  if result <> 0 then exit else

  PGPBuildOptionList(context, general_options,
    [
    { files instead of buffers }
    PGPOInputFile(context,FileIn_spec),
    PGPOOutputFile(context,FileOut_spec),
    PGPODataIsASCII(context, TextMode),{-t}
    PGPOArmorOutput(context, Armor), {-a}
    PGPOCommentString(context, Comment),
    PGPOForYourEyesOnly(context, PGPBoolean(EyesOnly)),
    PGPOCompression(context, PGPBoolean(Compress)),
    PGPOVersionString(context, GlobalVersionString)//,
    //PGPOEventHandler(context, MyEventHandler, nil)
    ]);

  if result <> 0 then exit else

  result := PGPEncode(context, [general_options,encrypt_options,sign_options,mime_options]);

  if result <> 0 then exit;

  if Boolean(MIME) then
  begin
    StrCopy(MimeSeparator,'');
    StrLCopy(MimeSeparator,MimeSep,kPGPMimeSeparatorSize-1);
  end;

  finally
    PGPFreeFileSpec(FileIn_spec);
    PGPFreeFileSpec(FileOut_spec);
    //FillChar(SignKeyPass,SizeOf(SignKeyPass),0);
    //FillChar(ConventionalPass,SizeOf(ConventionalPass),0);
    PGPFreeKeyset(CryptKey);
    PGPFreeOptionList(general_options);
    PGPFreeOptionList(encrypt_options);
    PGPFreeOptionList(sign_options);
    PGPFreeOptionList(mime_options);
    QuitIt(Context,KeySet);
  end;
end;

(* ENCRYPT/SIGN BUFFER *)
function spgpencode(BufferIn, BufferOut: PChar; BufferOutLen: LongInt; Encrypt,
         Sign, SignAlg, ConventionalEncrypt, ConventionalAlg, Armor, TextMode,
         Clear: LongInt; CryptKeyID, SignKeyID, SignKeyPass, ConventionalPass,
         Comment: PChar): LongInt;stdcall;export;
var
  p: array[0..kPGPMimeSeparatorSize-1] of char;
begin
result := spgp_encode(
          BufferIn, BufferOut, BufferOutLen, Encrypt,
          Sign, SignAlg, ConventionalEncrypt, ConventionalAlg,
          Armor, TextMode, Clear, 0, 0, 0,
          CryptKeyID, SignKeyID, SignKeyPass, ConventionalPass,
          Comment, p
          );
end;

(* ENCRYPT/SIGN FILE *)
function spgpencodefile(FileIn, FileOut: PChar; Encrypt, Sign, SignAlg, ConventionalEncrypt,
         ConventionalAlg, Armor, TextMode, Clear: LongInt; CryptKeyID, SignKeyID,
         SignKeyPass, ConventionalPass, Comment: PChar): LongInt;stdcall;export;
var
  p: array[0..kPGPMimeSeparatorSize-1] of char;
begin
result := spgp_encodefile(
          FileIn,		  //file to encrypt
          FileOut,    //file for output
          Encrypt,		//(-e)encrypt? 0 or 1
          Sign,			  //(-s)sign? 0 or 1
          SignAlg,
          ConventionalEncrypt, //(-c)conventional encrypt? 0 or 1
          ConventionalAlg,
          Armor,			    //(-a)ASCII-armor? 0 or 1
          TextMode,		    //(-t)data is ASCII? 0 or 1
          Clear,          //clearsign? 0 or 1 (for signing only)
          0,
          0,
          0,
          CryptKeyID,	    //recipient - may be a list
          SignKeyID,		  //signing key ID - one only
          SignKeyPass,	  //passphrase to access signing key
          ConventionalPass,//passphrase for conventional encryption
          Comment,
          p
          );
end;

(* DETACHED SIGNATURE FILE: CREATE *)
function spgpdetachedsigcreate(FileIn, SigFile, SignKeyID, SignKeyPass, Comment: PChar; SignAlg, Armor: LongInt):LongInt;stdcall;export;
var
  Context        : pPGPContext;
  FileIn_spec    : pPGPFileSpec;
  FileOut_spec   : pPGPFileSpec;
  general_options: pPGPOptionList;
  HashAlg        : PGPHashAlgorithm;
  HashAlgDefault : PGPHashAlgorithm;
  Key            : TKeyData;
  KeySet         : pPGPKeySet;
  SignKey        : pPGPKey;
  sign_options   : pPGPOptionList;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeySet := nil;
  SignKey := nil;
  FileIn_spec := nil;
  FileOut_spec := nil;
  sign_options := nil;
  general_options := nil;

  result := Init(Context, KeySet, false, false);

  if result <> 0 then exit else

  try
    if not (SignKeyID <> '') then
    begin
      result := SPGPError_NoSignKey;//no sign key specified
      exit;
    end
    else
    if not KeyIsOnRing(context,KeySet,SignKeyID) then
    begin
      result := SPGPError_SignKeyNotFound;//sign key not found
      exit;
    end;

    { create input filespec from filename}
    result :=
    PGPNewFileSpecFromFullPath(context, FileIn, FileIn_spec);

    if result <> 0 then exit;
    { create output filespec from filename}
    result :=
    PGPNewFileSpecFromFullPath(context, SigFile, FileOut_spec);

    if result <> 0 then exit;

    { get private key for signing }
    if SignKeyID <> '' then
      result := GetKeyByID(context,SignKey,LowerCase(StrPas(SignKeyID)),KeySet);

    if result <> 0 then exit;

    { determine default hash alg for signing key }
    result := GetKeyData(context,KeySet,SignKey,Key);
    if result <> 0 then exit;

    case Key.KeyAlg of
      kPGPPublicKeyAlgorithm_RSA:             HashAlgDefault := kPGPHashAlgorithm_MD5;
      kPGPPublicKeyAlgorithm_RSAEncryptOnly:  HashAlgDefault := kPGPHashAlgorithm_MD5;
      kPGPPublicKeyAlgorithm_RSASignOnly:     HashAlgDefault := kPGPHashAlgorithm_MD5;
      //kPGPPublicKeyAlgorithm_ElGamal:         HashAlgDefault := kPGPHashAlgorithm_SHA;
      kPGPPublicKeyAlgorithm_DSA:             HashAlgDefault := kPGPHashAlgorithm_SHA
    else
      HashAlgDefault := kPGPHashAlgorithm_MD5;
    end;

    { convert hashing algorithm to proper type }
    case SignAlg of
      0: HashAlg := HashAlgDefault;
      1: HashAlg := kPGPHashAlgorithm_MD5;
      2: HashAlg := kPGPHashAlgorithm_SHA;
      3: HashAlg := kPGPHashAlgorithm_RIPEMD160;
      4: HashAlg := kPGPHashAlgorithm_SHADouble
    else
      HashAlg := HashAlgDefault;
    end;

    result :=
    PGPBuildOptionList(context,
      sign_options,
      [
      PGPOHashAlgorithm(context, HashAlg),
      PGPODetachedSig(context, [PGPOLastOption(context)]),
      PGPOSignWithKey(context, SignKey, [PGPOPassphrase(context, SignKeyPass)])
      ]);

    if result <> 0 then exit;
    
    PGPBuildOptionList(context,
      general_options,
      [
      { files instead of buffers }
      PGPOInputFile(context,FileIn_spec),
      PGPOOutputFile(context,FileOut_spec),
      //PGPODataIsASCII(context, TextMode),{-t}
      PGPOArmorOutput(context, Armor), {-a}
      //PGPOOutputLineEndType(context, kPGPLineEnd_CRLF),
      //PGPOCompression(context, PGPTrue),
      PGPOVersionString(context, GlobalVersionString{PChar('NA')}),
      PGPOCommentString(context,Comment)//,
      //PGPOEventHandler(context, MyEventHandler, nil)
      ]);

    if result <> 0 then exit;

    result := PGPEncode(context, [general_options, sign_options]);

  finally
    PGPFreeFileSpec(FileIn_spec);
    PGPFreeFileSpec(FileOut_spec);
    PGPFreeOptionList(general_options);
    PGPFreeOptionList(sign_options);
    QuitIt(Context, KeySet);
  end;
end;

initialization
  // check all pgp units used by this unit
  LibsAllLoaded := //pgpUserInterfaceLoaded and
                   pgpOptionListLoaded and
                   pgpEncodeLoaded and
                   pgpErrorsLoaded and
                   pgpUtilitiesLoaded and
                   pgpKeysLoaded and
                   pgpFeaturesLoaded;
                   
  PGPversion5 := IsPGP5;
end.
