{************************************************************************}
{ spgpFeatures - functions to determine PGPsdk version and availability  }
{ of PGPsdk features                                                     }
{                                                                        }
{ This unit requires one or more units from the Pascal translations of   }
{ the PGPsdk header files: <http://www.oz.net/~srheller/pgpsdk>          }
{ 								                                                       }
{ The original Pascal code is: spgpFeatures.pas, released 10 Apr 2000.   }
{ The initial developer of the Pascal code is Steven R. Heller           }
{ <srheller@oz.net>                                                      }
{ 								                                                       }
{ Portions created by Steven R. Heller are    				                   }
{ Copyright (C) 2000 Steven R. Heller                                    }
{									                                                       }
{ The contents of this file are used with permission, subject to         }
{ the Mozilla Public License Version 1.1 (the "License"); you may        }
{ not use this file except in compliance with the License. You may       }
{ obtain a copy of the License at                                        }
{ http://www.mozilla.org/MPL/MPL-1.1.html 	                             }
{ 									                                                     }
{ Software distributed under the License is distributed on an 	         }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or         }
{ implied. See the License for the specific language governing           }
{ rights and limitations under the License.                              }
{                                                                        }
{************************************************************************}

unit spgpFeatures;

interface

uses
  // Delphi
  SysUtils, Windows,
  // SPGP
  spgpGlobals,
  // PGPsdk
  pgpBase, pgpPubTypes, pgpUtilities, pgpFeatures, pgpErrors,
    pgpClient,
  // always last
  PGPsdk;

type
  pPGPInfoRec = ^TPGPInfoRec;
  TPGPInfoRec = record
    PGPPath        : array[0..MAX_PATH - 1] of char;
    SdkApiVersion  : Longint;
    FeaturesFlags  : Longint;
    PKAlgFlags     : Longint;
    CipherAlgFlags : Longint;
    HashAlgFlags   : Longint;
  end;

  function spgpsdkapiversion: Longint;stdcall;export;
  function spgppgpinfo(Info: pPGPInfoRec): LongInt;stdcall;export;

  { these two just suck, they must be replaced and sent to Siberia }
  function countkeyalgs: LongInt;stdcall;export;
  function countcipheralgs: LongInt;stdcall;export;

implementation

var
  LibsAllLoaded  : boolean;

(* returns the installed sdk API version *)
function spgpsdkapiversion: Longint; stdcall; export;
var
  v              : PGPUint32;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;
  if result <> 0 then exit;

  result := PGPsdkInit;
  if result <> 0 then exit;
  try
    result := PGPGetSDKVersion(v);
    if result <> 0
    then exit
    else result := v;
  finally
    PGPsdkCleanup;
  end;
end;

(* QUERIES SDK CAPABILITIES: CAN THE INSTALLED DLL ENCRYPT? *)
function spgppgpinfo(Info: pPGPInfoRec): LongInt;stdcall;export;
var
  Context        : pPGPContext;
  Flags          : PGPFlags;
  Loaded         : boolean;
  PGPversion5    : boolean;
  Selector       : PGPFeatureSelector;
  TempPath       : array[0..MAX_PATH - 1] of char;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;
  if result <> 0 then exit;

  Context := nil;

  Loaded := false;
  result := PGPsdkInit;
  if result <> 0 then exit;
  Loaded := true;

  if result <> 0 then exit;
  try
    if kPGPsdkAPIVersion < $03000000 then
      PGPversion5 := true;

    Info.SdkApiVersion := kPGPsdkAPIVersion;

    if not PGPversion5 then
    begin
      result := PGPNewContext(kPGPsdkApiVersion, Context);
      PGPclInitLibrary(Context);
    end;

    // PGP PATH
    if PGPversion5 then
      result := PGPcomdlgGetPGPPath(TempPath, MAX_PATH)
    else
      result := PGPclGetPGPPath(TempPath, MAX_PATH);
    if result <> 0 then exit else
      StrCopy(Info.PGPPath, TempPath);

    StrCopy(TempPath, '');
{
type
      PGPFeatureSelector = PGPEnumType;
const
      kPGPFeatures_GeneralSelector        = 1;
      kPGPFeatures_ImplementationSelector =  2;

      kPGPFeatureMask_CanEncrypt   = $1;
      kPGPFeatureMask_CanDecrypt   = $2;
      kPGPFeatureMask_CanSign      = $4;
      kPGPFeatureMask_CanVerify    = $8;

      kPGPFeatureMask_IsDebugBuild = $1;
      kPGPFeatureMask_HasTimeout   = $2;
}
    { set the selector for general features, get the flags }
    Selector := kPGPFeatures_GeneralSelector;
    result := PGPGetFeatureFlags(Selector, Flags);
    if result <> 0 then exit;

    Info.FeaturesFlags := Flags;
    
  finally
    if Assigned(PGPcomdlgCloseLibrary) then
      PGPcomdlgCloseLibrary;
    if Assigned(PGPclCloseLibrary) then
      PGPclCloseLibrary;
    PGPFreeContext(Context);
    if Loaded then PGPsdkCleanup;
  end;
end;

(* QUERY DLL CAPABILITIES: HOW MANY SUPPORTED KEY ALGORITHMS? *)
{ this will be replaced eventually with something which will
  actually identify the algorithms }
function countkeyalgs: longint;stdcall;export;
var
  AlgCount       : PGPUInt32;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  result := PGPsdkInit;
  if result <> 0 then exit else
  try
    result := PGPCountPublicKeyAlgorithms(AlgCount);
    if result <> 0 then exit else
    result := AlgCount;
  finally
    PGPsdkCleanup;
  end;
end;

(* QUERY DLL CAPABILITIES: HOW MANY SUPPORTED CIPHER ALGORITHMS? *)
{ this will be replaced eventually with something which will
  actually identify the algorithms }
function countcipheralgs: longint;stdcall;export;
var
  AlgCount       : PGPUInt32;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  PGPsdkInit;

  if result <> 0 then exit;

  try
    PGPCountSymmetricCiphers(AlgCount);
    result := AlgCount;
  finally
    PGPsdkCleanup;
  end;
end;

initialization
  // check all pgp units used by this unit
  LibsAllLoaded := pgpUtilitiesLoaded and
                   pgpFeaturesLoaded and
                   pgpErrorsLoaded and
                   pgpClientLoaded;

end.
