{************************************************************************}
{ spgpGlobals - misc. internal functions & variables                     }
{                                                                        }
{ This unit requires one or more units from the Pascal translations of   }
{ the PGPsdk header files: <http://www.oz.net/~srheller/pgpsdk/>         }
{ 								                                                       }
{ The original Pascal code is: spgpGlobals.pas, released 10 Apr 2000.    }
{ The initial developer of the Pascal code is Steven R. Heller           }
{ <srheller@oz.net>                                                      }
{ 								                                                       }
{ Portions created by Steven R. Heller are    				                   }
{ Copyright (C) 2000 Steven R. Heller                                    }
{									                                                       }
{ The contents of this file are used with permission, subject to         }
{ the Mozilla Public License Version 1.1 (the "License"); you may        }
{ not use this file except in compliance with the License. You may       }
{ obtain a copy of the License at                                        }
{ http://www.mozilla.org/MPL/MPL-1.1.html 	                             }
{ 									                                                     }
{ Software distributed under the License is distributed on an 	         }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or         }
{ implied. See the License for the specific language governing           }
{ rights and limitations under the License.                              }
{                                                                        }
{************************************************************************}

unit spgpGlobals;

interface

uses
  pgpBase, pgpPubTypes, pgpUtilities, pgpSDKPrefs,
    pgpErrors, pgpKeys, pgpFeatures,
  // always last
  PGPsdk;

const
  GlobalVersionString: array[0..3] of char = 'N/A';

const
  { SPGP error codes }
  SPGPError_MissingParam       = -100;// missing required parameter
  SPGPError_NoEncryptKey       = -102;// no encrypt key specified
  SPGPError_EncryptKeyNotFound = -104;// encrypt key not found
  SPGPError_NoSignKey          = -106;// no sign key specified
  SPGPError_SignKeyNotFound    = -108;// sign key not found
  SPGPError_KeyNotFound        = -110;// key not found (generic)
  SPGPError_KeySearchError     = -112;// key lookup failure

  SPGPError_DataIsEncrypted    = -120;// ciphertext found instead of ?
  SPGPError_DataIsSigned       = -121;// sig found instead of ?
  SPGPError_DataIsDetachedSig  = -122;// d. sig found instead of ciphertext
  SPGPError_DataIsKey          = -123;// key found instead of ciphertext
  SPGPError_DataIsUnknown      = -124;// no ciphertext found

  SPGPError_DateTimeConversion = -130;// error in converting date/time

  SPGPError_NoPGPsdk           = -150;// no PGPsdk DLL found

const
  // Option Flags
  spgpEncodingFlag_UseUI          = -1;

  spgpEncodingFlag_KeyEncrypt     = $1;
  spgpEncodingFlag_PassEncrypt    = $2;
  spgpEncodingFlag_Sign           = $4;
  spgpEncodingFlag_SignDetached   = $8;

  // cipher algorithms
  spgpAlgorithmFlag_IDEA          = $100;
  spgpAlgorithmFlag_DES           = $200;
  spgpAlgorithmFlag_CAST5         = $400;

  // hashing algorithms
  spgpAlgorithmFlag_MD5           = $1000;
  spgpAlgorithmFlag_SHA           = $2000;
  spgpAlgorithmFlag_RIPEMD160     = $4000;

  spgpADKFlag_Warn                = $10000;
  spgpADKFlag_Strict              = $20000;

                              // $40000000;
  // output options
  spgpOutputFlag_Armor            = $1;
  spgpOutputFlag_TextMode         = $2;
  spgpOutputFlag_ClearSign        = $4;
  spgpOutputFlag_EyesOnly         = $8;
  spgpOutputFlag_Compress         = $10;
  spgpOutputFlag_MIME             = $20;

  // key import-export options
  spgpKeyImportFlag_All           = $1;
  spgpKeyImportFlag_None          = $2;

  spgpKeyExportFlag_Private       = $1;
  spgpKeyExportFlag_Compatible    = $2;

const
  spgpKeyPropFlag_OldStyle        = -1;// old style

  // string properties
  spgpKeyPropFlag_KeyID           = $00000001;// 1
  spgpKeyPropFlag_UserID          = $00000002;// 2
  spgpKeyPropFlag_Fingerprint     = $00000004;// 4
  spgpKeyPropFlag_CreationTimeStr = $00000008;// 8
  spgpKeyPropFlag_ExpirationTimeStr = $00000010;// 16
                               // = $00000020;// 32
                               // = $00000040;// 64

  // numeric properties
  spgpKeyPropFlag_Keybits         = $00000080;// 128
  spgpKeyPropFlag_KeyAlg          = $00000100;// 256
  spgpKeyPropFlag_Trust           = $00000200;// 512
  spgpKeyPropFlag_Validity        = $00000400;// 1024
  spgpKeyPropFlag_CreationTime    = $00000800;// 2048
  spgpKeyPropFlag_ExpirationTime  = $00001000;// 4096
                               // = $00002000;
                               // = $00004000;

  // boolean properties
  spgpKeyPropFlag_IsSecret        = $00008000;
  spgpKeyPropFlag_IsAxiomatic     = $00010000;
  spgpKeyPropFlag_IsRevoked       = $00020000;
  spgpKeyPropFlag_IsDisabled      = $00040000;
  spgpKeyPropFlag_IsExpired       = $00080000;
  spgpKeyPropFlag_IsSecretShared  = $00100000;

  spgpKeyPropFlag_CanEncrypt      = $00200000;
  spgpKeyPropFlag_CanDecrypt      = $00400000;
  spgpKeyPropFlag_CanSign         = $00800000;
  spgpKeyPropFlag_CanVerify       = $01000000;

  spgpKeyPropFlag_HasRevoker      = $02000000;
  spgpKeyPropFlag_HasADK          = $04000000;
  spgpKeyPropFlag_HasSubKey       = $08000000;
                               // = $10000000;
                               // = $20000000;
                               // = $40000000;
// Preferences flags
  const
    // for Get/SetPreferences
    PGPPrefsFlag_Default          = 0;
    PGPPrefsFlag_PublicKeyring    = 1;
    PGPPrefsFlag_PrivateKeyring   = 2;
    PGPPrefsFlag_RandomSeedFile   = 4;
    PGPPrefsFlag_DefaultKeyID     = 8;
    PGPPrefsFlag_GroupsFile       = 16;

    // for spgpPreferencesDialog
    PGPPrefsPage_GeneralPrefs     = 0;
    PGPPrefsPage_KeyringPrefs     = 1;
    PGPPrefsPage_EmailPrefs       = 2;
    PGPPrefsPage_HotkeyPrefs      = 3;
    PGPPrefsPage_KeyserverPrefs   = 4;
    PGPPrefsPage_CAPrefs          = 5;
    PGPPrefsPage_AdvancedPrefs    = 6;

type
  pCertificate = ^TCertificate;
  TCertificate = record
    SignersName  : array[0..kPGPMaxUserIDSize - 1] of char;
    Validity     : byte;
  end;

type
  pUserIdExternal = ^TUserIdExternal;
  TUserIdExternal = record
    Name         : array[0..kPGPMaxUserIDSize - 1] of char;
    Validity     : byte;
    //Attribute    : ??;
    Cert         : array[0..31] of TCertificate;
  end;

type
  pSubKeyExternal = ^TSubKeyExternal;
  TSubKeyExternal = record
    KeyID        : array[0..17] of char;// full size
    Fprint       : array[0..19] of char;
    CreationDate : Longint;//array[0..23] of char;
    ExpireDate   : Longint;//array[0..23] of char;
    KeyBits      : Longint;
    KeyAlg       : byte;
    //Trust        : byte;
    //Validity     : byte;
    PropFlags    : Longint;
  end;

type
  pKeyExternal = ^TKeyExternal;
  TKeyExternal = record // NB: not packed
    KeyID        : array[0..17] of char; // full size
    Fprint       : array[0..19] of char;

    KeyBits      : Longint;
    CreationDate : Longint;
    ExpireDate   : Longint;
    PrimaryUserId: byte; // index into UserID array
    KeyAlg       : byte;
    Trust        : byte;
    Validity     : byte;
    PropFlags    : Longint;
    SubKey       : array[0..31] of TSubKeyExternal;
    //UserId       : array[0..11] of TUserIdExternal;
    {
    IsSecret     : byte;
    IsAxiomatic  : byte;
    IsRevoked    : byte;
    IsDisabled   : byte;
    IsExpired    : byte;
    CanEncrypt   : byte;// 48 in 5.5
    CanDecrypt   : byte;{PGP60+}
    {CanSign      : byte;// 47 in 5.5
    CanVerify    : byte;{PGP60+}
    {IsSecretShared : byte;
    HasRevoker : byte;{PGP60+}
    {HasADK : byte;
    HasSubkey : byte;}
  end;

type
  pKeyArray = ^TKeyArray;
  TKeyArray = packed record
    Count        : Longint; // number of used elements
    Keys         : array[0..170] of TKeyExternal;
  end;

type
  pSig = ^TSig;
  TSig = record
    UserID       : shortstring;
    KeyID        : shortstring;
    DateTimeStr  : shortstring;
    DateTimeInt  : Cardinal;
    Checked      : boolean;
    Verified     : boolean;
    KeyValidity  : shortstring;
    KeyRevoked   : boolean;
    KeyDisabled  : boolean;
    KeyExpired   : boolean;
    KeyIsOnRing  : boolean;
    KeySet       : pPGPKeySet;
    Signed       : boolean;
  end;

type
  pSignatureExternal = ^TSignatureExternal;
  TSignatureExternal = record
    signingKeyID: TPGPKeyID;
    checked: PGPBOOLEAN;
    verified: PGPBOOLEAN;
    keyRevoked: PGPBOOLEAN;
    keyDisabled: PGPBOOLEAN;
    keyExpired: PGPBOOLEAN;
    keyMeetsValidityThreshold: PGPBOOLEAN;
    keyValidity: PGPVALIDITY;
    creationTime: PGPTIME;
  end;

{ key data type }
type
  pKeyData = ^TKeyData;
  TKeyData = record
    KeyID        : shortstring;
    UserID       : shortstring;
    Fprint       : shortstring;
    KeyBits      : shortstring;//because of the DH 'XXX/YYY' format
    KeyAlg       : PGPINT32;
    Trust        : PGPINT32;
    Validity     : PGPINT32;
    DateStr      : shortstring;
    DateInt      : PGPINT32;
    ExpireStr    : shortstring;
    ExpireInt    : PGPINT32;
    IsSecret     : PGPBoolean;
    IsAxiomatic  : PGPBoolean;
    IsRevoked    : PGPBoolean;
    IsDisabled   : PGPBoolean;
    IsExpired    : PGPBoolean;
  end;

{ recipients list data type }
type
  pRrecipsList = ^TRecipsList;
  TRecipsList = packed record
    RecipsGoodCount : PGPUINT32;
    RecipsGood      : array[0..127] of shortstring;
    PassphraseCount : PGPUint32;
    RecipsBadCount  : PGPUint32;
    RecipsBad       : array[0..127] of shortstring;
  end;

  function Init(var Context: pPGPContext; var KeySet: pPGPKeySet; const OpenAsMutable, CreateNewRing: Boolean): PGPError;
  function Init_Alt(var Context: pPGPContext; var KeySet: pPGPKeySet; const LoadRings, OpenAsMutable, CreateNewRing: Boolean): PGPError;
  function QuitIt(var Context: pPGPContext; var KeySet: pPGPKeySet): PGPError;
  // fake macro functions
  function IsPGPError(err: PGPError): Boolean;
  function IsntPGPError(err: PGPError): Boolean;

  // checks for SDK version
  function IsPGP5: boolean; 

implementation

var
  FLoaded        : boolean;
  LibsAllLoaded  : boolean;

function Init(var Context: pPGPContext; var KeySet: pPGPKeySet; const OpenAsMutable: Boolean; const CreateNewRing: Boolean): PGPError;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeySet := nil;
  FLoaded := false;

  result := PGPsdkInit;
  if result <> 0 then exit;
  FLoaded := true;

  result := PGPNewContext(kPGPsdkAPIVersion, Context);
  if result <> 0 then exit;

  result := PGPsdkLoadDefaultPrefs(Context);
  if result <> 0 then exit;

  if OpenAsMutable and CreateNewRing then
    result := PGPOpenDefaultKeyRings(context, kPGPKeyRingOpenFlags_Create or kPGPKeyRingOpenFlags_Mutable, KeySet)
  else
  if OpenAsMutable then
    result := PGPOpenDefaultKeyRings(context, kPGPKeyRingOpenFlags_Mutable, KeySet)
  else
    result := PGPOpenDefaultKeyRings(context, 0, KeySet);
end;

function Init_Alt(var Context: pPGPContext; var KeySet: pPGPKeySet; const LoadRings, OpenAsMutable: Boolean; const CreateNewRing: Boolean): PGPError;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeySet := nil;
  FLoaded := false;

  result := PGPsdkInit;
  if result <> 0 then exit;
  FLoaded := true;

  result := PGPNewContext(kPGPsdkAPIVersion, Context);
  if result <> 0 then exit;

  result := PGPsdkLoadDefaultPrefs(Context);
  if result <> 0 then exit;

  if LoadRings then
  begin
    if OpenAsMutable and CreateNewRing then
      result := PGPOpenDefaultKeyRings(context, kPGPKeyRingOpenFlags_Create or kPGPKeyRingOpenFlags_Mutable, KeySet)
    else
    if OpenAsMutable then
      result := PGPOpenDefaultKeyRings(context, kPGPKeyRingOpenFlags_Mutable, KeySet)
    else
      result := PGPOpenDefaultKeyRings(context, 0, KeySet);
  end;
end;

function QuitIt(var Context: pPGPContext; var KeySet: pPGPKeySet): PGPError;
begin
  result := kPGPError_NoError;

  if Assigned(KeySet) then
    result := PGPFreeKeySet(KeySet);

  if Assigned(Context) then
    result := PGPFreeContext(Context);

  if FLoaded then
    result := PGPsdkCleanup;
end;

function IsPGPError(err: PGPError): Boolean;
begin
  if not (err = kPGPError_NoError) then result := true
  else result := false;
end;

function IsntPGPError(err: PGPError): Boolean;
begin
  if (err = kPGPError_NoError) then result := true
  else result := false;
end;

function IsPGP5: boolean;
var
  v              : PGPUint32;
begin
  result := false;
  PGPGetSDKVersion(v);
  if v < $03000000 then result := true;
end;

initialization
  // check all pgp units used by this unit
  LibsAllLoaded := pgpUtilitiesLoaded and
                   pgpSDKPrefsLoaded and
                   pgpErrorsLoaded and
                   pgpKeysLoaded;

end.
