{************************************************************************}
{ spgpKeyIO - Key import/export functions                                }
{                                                                        }
{ This unit requires one or more units from the Pascal translations of   }
{ the PGPsdk header files: <http://www.oz.net/~srheller/pgpsdk/>         }
{ 								                                                       }
{ The original Pascal code is: spgpKeyIO.pas, released 10 Apr 2000.      }
{ The initial developer of the Pascal code is Steven R. Heller           }
{ <srheller@oz.net>                                                      }
{ 								                                                       }
{ Portions created by Steven R. Heller are    				                   }
{ Copyright (C) 2000 Steven R. Heller                                    }
{									                                                       }
{ The contents of this file are used with permission, subject to         }
{ the Mozilla Public License Version 1.1 (the "License"); you may        }
{ not use this file except in compliance with the License. You may       }
{ obtain a copy of the License at                                        }
{ http://www.mozilla.org/MPL/MPL-1.1.html 	                             }
{ 									                                                     }
{ Software distributed under the License is distributed on an 	         }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or         }
{ implied. See the License for the specific language governing           }
{ rights and limitations under the License.                              }
{                                                                        }
{************************************************************************}

unit spgpKeyIO;

interface

uses
  // Delphi
  SysUtils, Classes,
  // PGPsdk
  pgpBase, pgpPubTypes, pgpKeys, pgpUtilities, pgpErrors,
    pgpEncodePas, pgpOptionList, pgpClient, pgpTLS,
  // SPGP
  spgpGlobals, spgpKeyUtil,
  // always last
  PGPsdk;

  function spgpkeyexport(pKeyID,BufferOut: PChar;BufferOutLen,ExportPrivate,ExportCompatible: LongInt):LongInt;stdcall;export;
  function spgpkeyexportfile(pKeyID,FileOut: PChar; ExportPrivate,ExportCompatible: LongInt):LongInt;stdcall;export;
  function spgpkeyimport(BufferIn,KeyProps: PChar; KeyPropsLen: LongInt):LongInt;stdcall;export;
  function spgpkeyimportfile(FileIn,KeyProps: PChar; KeyPropsLen: LongInt):LongInt;stdcall;export;
  function spgp_keyimport(BufferIn,KeyProps: PChar; KeyPropsLen,Import,AllProps: LongInt):LongInt;stdcall;export;
  function spgp_keyimportfile(FileIn,KeyProps: PChar; KeyPropsLen,Import,AllProps: LongInt):LongInt;stdcall;export;

  function spgpkeyimport3(BufferIn, KeyProps: PChar; KeyPropsLen,Import,AllProps, WinHandle: LongInt):LongInt;stdcall;export;

implementation

var
  FDataAnalysis  : LongInt;
  FKeyImport     : boolean;
  FKeyImportCount: LongInt;
  FKeyImportAllProps: boolean;
  FKeyFound      : boolean;
  FKeyProps      : string;
  FSignature     : TSig;
  LibsAllLoaded  : boolean;
  PGPversion5    : boolean;

function EventHandler(Context: pPGPContext; event: pPGPEvent; UserValue: PGPUserValue):PGPError;
var
  p              : pchar;
  pid            : PGPChar;
  ps             : PGPSize;
  validity       : PGPVALIDITY;
  s              : string;
  t              : shortstring;
  n              : PGPUInt32;
  i              : integer;
  filt           : pPGPFilter;
  fKeySet        : pPGPKeySet;
  keyList        : pPGPKeyList;
  Key            : pPGPKey;
  iter           : pPGPKeyIter;
  keyid          : pPGPKeyID;
  LexSecCount    : integer;
  KeyRec         : TKeyData;
begin
  result := kPGPError_NoError;
  LexSecCount := 0;
  p := nil;
  filt := nil;
  fKeySet := nil;
  keyList := nil;
  Key := nil;
  iter := nil;
  keyid := nil;

  case event^.etype of
    kPGPEvent_NullEvent              :;
    kPGPEvent_InitialEvent           :;
    kPGPEvent_FinalEvent             :;
    kPGPEvent_ErrorEvent             :result := event^.data.errorData.error;
    kPGPEvent_WarningEvent           :;
    kPGPEvent_EntropyEvent           :;
    kPGPEvent_PassphraseEvent        :
      if not FKeyImport then result := kPGPError_BadPassphrase;// -11500;
    kPGPEvent_InsertKeyEvent         :;
(**** ANALYZE EVENT -------------------------------------*)
    kPGPEvent_AnalyzeEvent           :
      if LexSecCount = 0 then
        FDataAnalysis := PGPAnalyzeType(event^.data.analyzeData);
(**** KEY FOUND -----------------------------------------*)
    kPGPEvent_KeyFoundEvent          :
    begin
    { for each key in the event's keyset, call getkeyprops }
      if FKeyFound then exit else
      s := '';
      FKeyProps := '';
      FKeyImportCount := 0;

      result := PGPCountKeys(event^.data.KeyFoundData.keyset, n);
      if result <> 0 then exit else
      if n = 0 then exit
      else
      FKeyImportCount := n;
      result := PGPOrderKeySet(event^.data.KeyFoundData.keyset, kPGPAnyOrdering, keylist);
      if result <> 0 then exit;
      try{orderkeyset - creates keylist}
        result := PGPNewKeyIter(keylist, iter);
        if result <> 0 then exit;
        try
          while IsntPGPError(PGPKeyIterNext(iter, Key)) and
            Assigned(Key) do
            begin
            result := GetKeyData(context,event^.data.KeyFoundData.keyset,Key,KeyRec);
            //PGPCountAdditionalRecipientRequests(Key,n);
            if IsntPGPError(result)then begin
              if not FKeyImportAllProps then
                KeyRecToStr(KeyRec,t,3,true)
              else
                KeyRecToStr(KeyRec,t,10,true);
                s := s + t;
              end
            else
            s := s + '' + #13#10;
            end;
        finally
          PGPFreeKeyIter(iter);
        end;{newkeyiter}
      finally
        PGPFreeKeyList(keylist);
      end;{orderkeyset}

    if FKeyImport then
      { add the event's keyset to master keyset }
      result := PGPAddKeys(event^.data.KeyFoundData.keyset, pPGPKeySet(UserValue));

      if result <> 0 then exit else

      FKeyProps := s;//StrPLCopy(FKeyProps,s,FKeyPropsLen);
      s := '';
      FKeyFound := true;
      { One pass through this event will extract the info for
        all the found keys, but this event will be fired once for
        each key found. 3 keys found = getkeyprops called 3*3 times,
        PGPAddKeys called 3 times, etc. FKeyFound keeps
        these from being called more than necessary. Once through is
        enough. }
    end;

    kPGPEvent_OutputEvent            :;
    kPGPEvent_BeginLexEvent          :
      LexSecCount := event^.data.beginLexData.sectionNumber;
    kPGPEvent_EndLexEvent            :;
  end;
end;

function MyEventHandler(context: pPGPContext; event: pPGPEvent; UserValue: PGPUserValue):PGPError; cdecl;
begin
  result := EventHandler(context, event, uservalue);
end;

(* EXPORT KEY(S) TO BUFFER *)
function spgpkeyexport(pKeyID, BufferOut: PChar;BufferOutLen, ExportPrivate, ExportCompatible: LongInt):LongInt;stdcall;export;
var
  Context        : pPGPContext;
  KeySet         : pPGPKeySet;
  KeyID          : string;
  Keys           : pPGPKeySet;
  options        : pPGPOptionList;
  p              : pointer;
  output_size    : PGPSize;
  s              : shortstring;
  numKeys        : PGPUInt32;
  fileRef        : pPGPFileSpec;
  compat         : PGPExportFormat;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeySet := nil;
  Keys := nil;
  options := nil;
  p := nil;
  fileRef := nil;

  result := Init(Context, Keyset, false, false);

  if result <> 0 then exit;

  KeyID := LowerCase(StrPas(pKeyID));

  try
    if ExportCompatible = 1 then
      compat := kPGPExportFormat_Basic
    else
      compat := kPGPExportFormat_Complete;

    if result <> 0 then exit;

    { extract the keys }
    if KeyID <> '' then
    begin
      { create a new, empty keyset to hold key(s) }
      result := PGPNewKeySet(context,Keys);
      if result <> 0 then exit;

      result := GetKeysByID(Context, Keys, KeyID, KeySet);//get key(s)
    end;

    if result <> 0 then exit;
    { build the option list }
    if not PGPversion5 then //kPGPsdkAPIVersion >= $03000000 then // PGP6.x
      result :=
      PGPBuildOptionList(context,
        options,
        [
        PGPOAllocatedOutputBuffer(context, p, BufferOutLen, output_size),
  {pgp6}PGPOExportFormat(context,compat),
        PGPOVersionString(context, GlobalVersionString),
        PGPOExportPrivateKeys(context,ExportPrivate)
        ])
    else
      result :=
      PGPBuildOptionList(context,
        options,
        [
        PGPOAllocatedOutputBuffer(context, p, BufferOutLen, output_size),
        PGPOVersionString(context, GlobalVersionString),
        PGPOExportPrivateKeys(context,ExportPrivate)
        ]);

    if result <> 0 then exit;

    result := PGPExportKeySet(Keys, [Options]);

    (* write to buffer, check length, etc. *)
    if result = kPGPError_OutputBufferTooSmall then
      result := output_size + 1;
    if result <> 0 then exit;

    if p <> nil then
    begin
      { error code if output is larger than caller's buffer }
      if output_size > BufferOutLen - 1 then
        result := output_size + 1
      else
        StrLCopy(BufferOut, p, output_size);
    end
    else
      result := -1;

  finally
    PGPFreeData(p);
    PGPFreeKeySet(Keys);
    PGPFreeOptionList(Options);
    QuitIt(Context, KeySet);
  end;
end;

(* EXPORT KEY(S) TO FILE *)
function spgpkeyexportfile(pKeyID,FileOut: PChar; ExportPrivate,ExportCompatible: LongInt):LongInt;stdcall;export;
var
  Compat         : PGPExportFormat;
  Context        : pPGPContext;
  FileOut_spec   : pPGPFileSpec;
  KeyID          : string;
  Keys           : pPGPKeySet;
  KeySet         : pPGPKeyset;
  Options        : pPGPOptionList;
  s 				     : shortstring;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  Keys := nil;
  KeySet := nil;
  options := nil;
  FileOut_spec := nil;

  result := Init(Context, KeySet, false, false);
  if result <> 0 then exit;

  KeyID := LowerCase(StrPas(pKeyID));

  try
    if ExportCompatible = 1 then
      compat := kPGPExportFormat_Basic
    else
      compat := kPGPExportFormat_Complete;

    if result <> 0 then exit;
    { create output filespec from filename}
    result :=
    PGPNewFileSpecFromFullPath(context, FileOut, FileOut_spec);

    if result <> 0 then exit;

    { extract the keys }
    if KeyID <> '' then
    begin
      { create a new empty keyset to hold recipient(s) key(s) }
      result := PGPNewKeySet(context,Keys);

      if result <> 0 then exit;

      { is Key a KeyID or UserID? }
      s := Copy(KeyID,1,2);
      if LowerCase(s) = '0x' then
        result := GetKeySetByKeyID(Context, Keys, KeyID, KeySet)//get key(s)
      else
        result := GetKeySetByUserID(Context, Keys, KeyID,  KeySet);//get key(s)
    end;

    if result <> 0 then exit;

    { build the option list }
    if not PGPversion5 then //kPGPsdkAPIVersion >= $03000000 then //PGP6.x
      result :=
      PGPBuildOptionList(context,
        options,
        [
        PGPOOutputFile(context, FileOut_spec),
  {pgp6}PGPOExportFormat(context, compat),
        PGPOVersionString(context, GlobalVersionString),
        PGPOExportPrivateKeys(context, ExportPrivate)
        ])
    else
      result :=
      PGPBuildOptionList(context,
        options,
        [
        PGPOOutputFile(context, FileOut_spec),
        PGPOVersionString(context, GlobalVersionString),
        PGPOExportPrivateKeys(context, ExportPrivate)
        ]);

    if result <> 0 then exit;
    result := PGPExportKeySet(Keys, [Options]);

  finally
    PGPFreeFileSpec(FileOut_spec);
    PGPFreeKeySet(Keys);
    PGPFreeOptionList(options);
    QuitIt(Context, KeySet);
  end;
end;

(* import key from buffer *)
function spgpkeyimport(BufferIn, KeyProps: PChar; KeyPropsLen: LongInt):LongInt;stdcall;export;
var
  Context        : pPGPContext;
  KeySet         : pPGPKeySet;
  Options        : pPGPOptionList;
  PubRef         : pPGPFileSpec;
  SecRef         : pPGPFileSpec;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Options := nil;
  KeySet := nil;
  PubRef := nil;
  SecRef := nil;
  FKeyImport := True;
  FKeyFound := False;

  result := Init(Context, KeySet, true, false);
  if result <> 0 then exit;

  try
    result := PGPBuildOptionList(Context,
      Options,
      [
      PGPOInputBuffer(Context, BufferIn, length(BufferIn)),
      PGPODiscardOutput(Context, PGPTrue),
      PGPOEventHandler(Context, MyEventHandler, KeySet),
      PGPOKeySetRef(Context, KeySet),
      PGPOSendEventIfKeyFound(Context, PGPTrue)
      ]
      );

    if result <> 0 then exit;

    result := PGPDecode(Context, [Options]);

    if result <> 0 then exit;

    { save changes to key ring }
    if Boolean(PGPKeySetNeedsCommit(KeySet)) then
      result := PGPCommitKeyringChanges(KeySet);

    if result <> 0 then exit;

    if Length(FKeyProps) > KeyPropsLen - 1
    then result := Length(FKeyProps) + 1
    else StrPLCopy(KeyProps, FKeyProps, KeyPropsLen - 1);

    if (result = 0) or (result = -1) then
      (* LAST CHECK: what did the analysis event say about input? *)
      case FDataAnalysis of
        0:  result := SPGPError_DataIsEncrypted;  { Encrypted }
        1:  result := SPGPError_DataIsSigned;     { Signed }
        2:  result := SPGPError_DataIsDetachedSig;{ Detached Signature }
        3:  result := kPGPError_NoError;          { Key }
        4:  result := SPGPError_DataIsUnknown;    { Unknown }
      end;

  finally
    PGPFreeOptionList(Options);
    QuitIt(Context, KeySet);
  end;
end;

(* IMPORT KEY FROM BUFFER Mk IIa *)
function spgp_keyimport(BufferIn, KeyProps: PChar; KeyPropsLen, Import, AllProps: LongInt):LongInt;stdcall;export;
var
  Context        : pPGPContext;
  FoundKeys      : pPGPKeySet;
  KeySet         : pPGPKeySet;
  Options        : pPGPOptionList;
  s              : string;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  FoundKeys := nil;
  KeySet := nil;
  Options := nil;
  s := '' ;

  // open rings for read/write
  result := Init(Context, KeySet, true, false);
  if result <> 0 then exit;

  try
    // create empty keyset to receive found keys
    result := PGPNewKeySet(Context, FoundKeys);
    if result <> 0 then exit;

    result := PGPImportKeySet(Context,
      FoundKeys,
      [
      PGPOInputBuffer(Context, BufferIn, Length(BufferIn)),
      PGPOEventHandler(Context, MyEventHandler, nil)
      ]
     );
    if result <> 0 then exit;

    if Import = 1 then
    begin
      result := PGPAddKeys(FoundKeys, KeySet);
      if result <> 0 then exit;

      { save changes to key ring }
      if Boolean(PGPKeySetNeedsCommit(KeySet)) then
        result := PGPCommitKeyringChanges(KeySet);
      if result <> 0 then exit;
    end;

    result := GetKeySetPropsOldStyle(Context, FoundKeys, s, Boolean(AllProps));
    if result <> 0 then exit;

    if Length(s) > KeyPropsLen - 1
    then result := Length(s) + 1
    else StrPLCopy(KeyProps, s, KeyPropsLen - 1);
  
    if (result = 0) or (result = -1) then
      // LAST CHECK: what did the analysis event say about input?
      case FDataAnalysis of
        0:  result := SPGPError_DataIsEncrypted;  { Encrypted }
        1:  result := SPGPError_DataIsSigned;     { Signed }
        2:  result := SPGPError_DataIsDetachedSig;{ Detached Signature }
        3:  result := kPGPError_NoError;          { Key }
        4:  result := SPGPError_DataIsUnknown;    { Unknown }
      end;

  finally
    PGPFreeOptionList(Options);
    PGPFreeKeySet(FoundKeys);
    QuitIt(Context, KeySet);
  end;
end;

(* IMPORT KEY FROM BUFFER Mk III *)
function spgpkeyimport3(BufferIn, KeyProps: PChar; KeyPropsLen,Import,AllProps, WinHandle: LongInt):LongInt;stdcall;export;
var
  Context        : pPGPContext;
  KeySet         : pPGPKeySet;
  KeySetToDisplay : pPGPKeySet;
  KeySetSelected : pPGPKeySet;
  Options        : pPGPOptionList;
  PGPversion5    : boolean;
  TlsContext     : pPGPtlsContext;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeySet := nil;
  KeySetToDisplay := nil;
  KeySetSelected := nil;
  Options := nil;
  TlsContext := nil;

  FKeyImport := Boolean(Import);
  FKeyFound := False;

  result := Init(Context, KeySet, true, false);
  if result <> 0 then exit;

  if (AllProps = 1) or (AllProps = 0) then
    FKeyImportAllProps := Boolean(AllProps)
  else
    FKeyImportAllProps := False;

  try
    result := PGPNewTLSContext(Context, tlsContext);
    if result <> 0 then exit;

    result := PGPNewKeySet(Context, KeySetToDisplay);
    if result <> 0 then exit;

    result := GetKeysById(Context, KeySetToDisplay, BufferIn, KeySet);
    if result <> 0 then exit;

    //if kPGPsdkAPIVersion < $03000000 then
    //  PGPversion5 := true;

    if PGPversion5 then
      //result := PGPcomdlgWrapBuffer(Text, Column, OutText)
    else
    begin
      PGPclInitLibrary(Context);

    result :=
    PGPclSelectKeys(Context,
      TlsContext,
      WinHandle,
      PChar('Select the key(s) you would like to import to your keyring:'),
      KeySetToDisplay,
      KeySet,
      KeySetSelected
    );
    end;
    if result <> 0 then exit;
(*
    result := PGPBuildOptionList(context,
      Options,
      [
      PGPOInputBuffer(Context, BufferIn, length(BufferIn)),
      PGPODiscardOutput(Context, PGPTrue),
      PGPOEventHandler(Context, MyEventHandler, KeySet),
      PGPOKeySetRef(Context, KeySet),
      PGPOSendEventIfKeyFound(Context, PGPTrue)
      ]);

    if result <> 0 then exit;

    result := PGPDecode(Context, [Options]);

    if result <> 0 then exit;

    { save changes to key ring }
    if Boolean(PGPKeySetNeedsCommit(KeySet)) then
      result := PGPCommitKeyringChanges(KeySet);

    if result <> 0 then exit;
*)
    if Length(FKeyProps) > KeyPropsLen - 1 then
      result := Length(FKeyProps) + 1
    else
      StrPLCopy(KeyProps, FKeyProps, KeyPropsLen - 1);

    if (result = 0) or (result = -1) then
      (* LAST CHECK: what did the analysis event say about input? *)
      case FDataAnalysis of
        0:  result := SPGPError_DataIsEncrypted;  { Encrypted }
        1:  result := SPGPError_DataIsSigned;     { Signed }
        2:  result := SPGPError_DataIsDetachedSig;{ Detached Signature }
        3:  result := kPGPError_NoError;          { Key }
        4:  result := SPGPError_DataIsUnknown;    { Unknown }
      end;

  finally
    if PGPversion5
    then PGPcomdlgCloseLibrary
    else PGPclCloseLibrary;
    PGPFreeTLSContext(tlsContext);
    PGPFreeOptionList(Options);
    QuitIt(Context, KeySet);
  end;
end;

(* IMPORT KEY FROM FILE Mk II*)
function spgp_keyimportfile(FileIn, KeyProps: PChar; KeyPropsLen, Import, AllProps: LongInt): LongInt;stdcall;export;
var
  Context        : pPGPContext;
  FoundKeys      : pPGPKeySet;
  KeySet         : pPGPKeySet;
  Options        : pPGPOptionList;
  FileIn_spec    : pPGPFileSpec;
  s              : string;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  FoundKeys := nil;
  KeySet := nil;
  Options := nil;
  FileIn_spec := nil;
  s := '';

  result := Init(Context, KeySet, true, false);

  if result <> 0 then exit;

  result :=
  PGPNewFileSpecFromFullPath(Context, FileIn, FileIn_spec);

  if result <> 0 then exit;

  try
    // create empty keyset to receive found keys
    result := PGPNewKeySet(Context, FoundKeys);
    if result <> 0 then exit;

    result := PGPImportKeySet(Context,
      FoundKeys,
      [
      PGPOInputFile(Context, FileIn_spec),
      PGPOEventHandler(Context, MyEventHandler, nil)
      ]
     );
    if result <> 0 then exit;

    if Import = 1 then
    begin
      result := PGPAddKeys(FoundKeys, KeySet);
      if result <> 0 then exit;

      { save changes to key ring }
      if Boolean(PGPKeySetNeedsCommit(KeySet)) then
        result := PGPCommitKeyringChanges(KeySet);
    end;
    if result <> 0 then exit;

    result := GetKeySetPropsOldStyle(Context, FoundKeys, s, Boolean(AllProps));
    if result <> 0 then exit;

    if Length(s) > KeyPropsLen - 1
    then result := Length(s) + 1
    else StrPLCopy(KeyProps, s, KeyPropsLen - 1);

    if (result = 0) or (result = -1) then
      (* LAST CHECK: what did the analysis event say about input? *)
      case FDataAnalysis of
        0:  result := SPGPError_DataIsEncrypted;  { Encrypted }
        1:  result := SPGPError_DataIsSigned;     { Signed }
        2:  result := SPGPError_DataIsDetachedSig;{ Detached Signature }
        3:  result := kPGPError_NoError;          { Key }
        4:  result := SPGPError_DataIsUnknown;    { Unknown }
      end;

  finally
    PGPFreeFileSpec(FileIn_spec);
    PGPFreeKeySet(FoundKeys);
    PGPFreeOptionList(Options);
    QuitIt(Context, KeySet);
  end;
end;

(* import key from file *)
function spgpkeyimportfile(FileIn, KeyProps: PChar; KeyPropsLen: LongInt):LongInt;stdcall;export;
var
  Context        : pPGPContext;
  KeySet         : pPGPKeySet;
  Options        : pPGPOptionList;
  FileIn_spec    : pPGPFileSpec;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeySet := nil;
  Options := nil;
  FileIn_spec := nil;
  FKeyImport := True;
  FKeyFound := False;

  result := Init(Context, KeySet, true, false);
  if result <> 0 then exit;

  result := PGPNewFileSpecFromFullPath(Context, FileIn, FileIn_spec);

  if result <> 0 then exit;

  try
    result := PGPBuildOptionList(Context,
      Options,
      [
      PGPOInputFile(Context, FileIn_spec),
      PGPODiscardOutput(Context, PGPTrue),
      PGPOEventHandler(Context, MyEventHandler, KeySet),
      PGPOKeySetRef(Context, KeySet),
      PGPOSendEventIfKeyFound(Context, PGPTrue)
      ]);

    if result <> 0 then exit;

    result := PGPDecode(Context, [options]);

    if result <> 0 then exit;

    if Boolean(PGPKeySetNeedsCommit(KeySet)) then
      result := PGPCommitKeyRingChanges(KeySet);

    if result <> 0 then exit;

    if Length(FKeyProps) > KeyPropsLen - 1 then
      result := Length(FKeyProps) + 1
    else
      StrPLCopy(KeyProps, FKeyProps, KeyPropsLen - 1);

    if (result = 0) or (result = -1) then
      (* LAST CHECK: what did the analysis event say about input? *)
      case FDataAnalysis of
        0:  result := SPGPError_DataIsEncrypted;   { Encrypted }
        1:  result := SPGPError_DataIsSigned;      { Signed }
        2:  result := SPGPError_DataIsDetachedSig; { Detached Signature }
        3:  result := kPGPError_NoError;           { Key }
        4:  result := SPGPError_DataIsUnknown;     { Unknown }
      end;

  finally
    FKeyProps := '';
    PGPFreeFileSpec(FileIn_spec);
    PGPFreeOptionList(options);
    QuitIt(Context, KeySet);
  end;
end;

initialization
  // check all pgp units used by this unit
  LibsAllLoaded := pgpUtilitiesLoaded and
                   pgpErrorsLoaded and
                   pgpKeysLoaded and
                   pgpOptionListLoaded and
                   pgpEncodeLoaded;
  PGPversion5 := IsPGP5;

end.
