{************************************************************************}
{ spgpKeyMan - misc. key-manipulation functions                          }
{                                                                        }
{ This unit requires one or more units from the Pascal translations of   }
{ the PGPsdk header files: <http://www.oz.net/~srheller/pgpsdk>          }
{ 								                                                       }
{ The original Pascal code is: spgpKeyMan.pas, released 10 Apr 2000.     }
{ The initial developer of the Pascal code is Steven R. Heller           }
{ <srheller@oz.net>                                                      }
{ 								                                                       }
{ Portions created by Steven R. Heller are    				                   }
{ Copyright (C) 2000 Steven R. Heller                                    }
{									                                                       }
{ The contents of this file are used with permission, subject to         }
{ the Mozilla Public License Version 1.1 (the "License"); you may        }
{ not use this file except in compliance with the License. You may       }
{ obtain a copy of the License at                                        }
{ http://www.mozilla.org/MPL/MPL-1.1.html 	                             }
{ 									                                                     }
{ Software distributed under the License is distributed on an 	         }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or         }
{ implied. See the License for the specific language governing           }
{ rights and limitations under the License.                              }
{                                                                        }
{************************************************************************}

unit spgpKeyMan;

interface

uses
  // Delphi
  SysUtils,
  // PGPsdk
  pgpBase, pgpPubTypes, pgpKeys, pgpErrors, pgpUtilities, pgpOptionList,
  // SPGP
  spgpKeyUtil, spgpUtil, spgpGlobals,
  // always last
  PGPsdk;

  function spgpkeyenable(KeyID: PChar): LongInt;stdcall;export;
  function spgpkeydisable(KeyID: PChar): LongInt;stdcall;export;
  function spgpkeyremove(KeyID: PChar): LongInt;export;stdcall;
  function spgpkeypasschange(KeyID, OldPhrase, NewPhrase: PChar): LongInt;stdcall;export;
  function spgpkeysign(KeyHexID, UserID,SignKeyID, SignKeyPass: PChar;Expires,Exportable,Trust,Validity: LongInt): LongInt;export;stdcall;
  function spgpkeysigremove(KeyHexID, UserID, SignKeyID: PChar): LongInt;stdcall;export;
  function spgpkeysigrevoke(UserID, SignKeyID, SignKeyPass: PChar): LongInt;stdcall;export;

implementation

var
  LibsAllLoaded  : boolean;
  PGPversion5    : boolean;

(* ENABLE KEY *)
function spgpkeyenable(KeyID: PChar): LongInt;stdcall;export;
var
  Context        : pPGPContext;
  KeySet         : pPGPKeySet;
  TargetKey      : pPGPKey;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeySet := nil;
  TargetKey := nil;

  result := Init(Context, KeySet, true, false);

  try
    if not ZeroLength(KeyID)// <> ''
    then result := GetKeyByKeyID(context,targetKey,LowerCase(StrPas(KeyID)),KeySet)
    else result := SPGPError_MissingParam;

    if result <> 0 then exit;

    result := PGPEnableKey(targetKey);

    if result <> 0 then exit;

    if Boolean(PGPKeySetNeedsCommit(KeySet))
    then result := PGPCommitKeyringChanges(KeySet);

    finally
      QuitIt(Context, KeySet);
    end;
end;

(* DISABLE KEY *)
function spgpkeydisable(KeyID: PChar): LongInt;stdcall;export;
var
  Context        : pPGPContext;
  KeySet         : pPGPKeySet;
  TargetKey      : pPGPKey;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeySet := nil;
  TargetKey := nil;

  result := Init(Context, KeySet, true, false);
  if result <> 0 then exit;

  try
    if ZeroLength(KeyID)// <> ''
    then result := SPGPError_MissingParam
    else result := GetKeyByID(context, targetKey, LowerCase(StrPas(KeyID)), KeySet);

    if result <> 0 then exit;

    result := PGPDisableKey(targetKey);

    if result <> 0 then exit;

    if Boolean(PGPKeySetNeedsCommit(KeySet))
    then result := PGPCommitKeyringChanges(KeySet);

    finally
      QuitIt(Context, KeySet);
    end;
end;

(* REMOVE KEY FROM DEFAULT KEY-RING *)
function spgpkeyremove(KeyID: PChar):LongInt;export;stdcall;
var
  Context        : pPGPContext;
  KeySet         : pPGPKeySet;
  KeyToKill      : pPGPKeySet;
  s              : shortstring;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeySet := nil;
  KeyToKill := nil;

  result := Init(Context, KeySet, true, false);
  if result <> 0 then exit;

  if not KeysAreOnRing(context, KeySet, KeyID) then
    begin
      result := SPGPError_KeyNotFound;//key not found
      exit;
    end;

  try
    result := PGPNewKeySet(context, KeyToKill);

    s := Copy(StrPas(KeyID),1,2);
    if LowerCase(s) = '0x'
    then result := GetKeySetByKeyID(context, KeyToKill, KeyID, KeySet)
    else result := GetKeySetByUserID(context, KeyToKill, KeyID, KeySet);

    if result <> 0 then exit;

    result := PGPRemoveKeys(KeyToKill,KeySet);
{added these lines to get it working - begin}
    if result <>0 then exit else

    if Boolean(PGPKeySetNeedsCommit(KeySet))
    then result := PGPCommitKeyringChanges(KeySet);
{added these lines to get it working - end}

    finally
      PGPFreeKeySet(KeyToKill);
      QuitIt(Context, KeySet);
    end;
end;

(* CHANGE PASSPHRASE *)
function spgpkeypasschange(KeyID,OldPhrase,NewPhrase: PChar): LongInt;stdcall;export;
var
  Context        : pPGPContext;
  KeySet         : pPGPKeySet;
  TargetKey      : pPGPKey;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeySet := nil;
  TargetKey := nil;

  result := Init(Context, KeySet, true, false);
  if result <> 0 then exit;

  try
    if ZeroLength(KeyID)// <> ''
    then result := SPGPError_MissingParam
    else result := GetKeyByID(context,targetKey,LowerCase(StrPas(KeyID)),KeySet);

    if result <> 0 then exit;

    if PGPversion5 then
      result := PGPChangePassphrase5(targetKey, OldPhrase, NewPhrase)
    else
      result := PGPChangePassphrase(targetKey,
        [PGPOPassphrase(context, OldPhrase),
        PGPOPassphrase(context, NewPhrase)]);

    if result <> 0 then exit;

    if Boolean(PGPKeySetNeedsCommit(KeySet)) then
      result := PGPCommitKeyringChanges(KeySet);

  finally
    QuitIt(Context, KeySet);
  end;
end;

(* SIGN USER ID *)
function spgpkeysign(KeyHexID, UserID, SignKeyID, SignKeyPass: PChar; Expires, Exportable, Trust, Validity: LongInt): LongInt;stdcall;export;
var
  Context        : pPGPContext;
  Filt           : pPGPFilter;
  FKeySet        : pPGPKeySet;
  KeySet         : pPGPKeySet;
  KeyList        : pPGPKeyList;
  KeyID          : TPGPKeyID;
  Iter           : pPGPKeyIter;
  n              : PGPUInt32;
  TargetKeyID    : TPGPKeyID;
  Key            : pPGPKey;
  TargetKey      : pPGPKey;
  SignKey        : pPGPKey;
  TargetID       : pPGPUserID;
  BufferSize     : PGPSIZE;
  AString        : PChar;
  FullSize       : PGPSIZE;
  s              : string;
  ExpirationDays : PGPUINT32;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeySet := nil;
  Filt := nil;
  FKeySet := nil;
  KeyList := nil;
  Iter := nil;
  SignKey := nil;
  TargetID := nil;
  
  result := Init(Context, KeySet, true, false);

  if result <> 0 then exit;

  { check #1: Key IDs are present & keys are on ring }
  if ZeroLength(KeyHexID) then
    begin
      result := SPGPError_MissingParam;//missing param
      exit;
    end
  else
  if not KeyIsOnRing(context,KeySet,KeyHexID) then
    begin
      result := SPGPError_KeyNotFound;//key not found
      exit;
    end;

  if ZeroLength(SignKeyID) then
    begin
      result := SPGPError_NoSignKey;//no sign key specified
      exit;
    end
  else
  if not KeyIsOnRing(context,KeySet,SignKeyID) then
    begin
      result := SPGPError_SignKeyNotFound;//sign key not found
      exit;
    end;

  { initialize variables }
  if Expires > 0
  then expirationDays := Expires
  else expirationDays := 0;

  bufferSize := 255;
  astring := AllocMem(buffersize);

  try
    { STEP ONE - Get the Signing key }

    if result <> 0 then exit;
    result := GetKeyByID(context,SignKey,StrPas(SignKeyID),KeySet);

    { STEP TWO - Find the Target User ID }

    if result <> 0 then exit;
    result := PGPNewUserIDStringFilter(context, UserID, kPGPMatchEqual{kPGPMatchSubString}, filt);

    if result <> 0 then exit;
    try
    { filter main keyset into fKeySet based on ID/email filter }
    result := PGPFilterKeySet(KeySet, filt, fKeySet);
    if result <> 0 then exit;
    try
      { see what we've got in the net }
      result := PGPCountKeys(fKeySet, n);
      if result <> 0 then exit;
      if n <= 0 then begin
        result := SPGPError_KeyNotFound;
        exit;
        end
      else
      { sort the keys into a keylist }
      result := PGPOrderKeySet(fKeySet, kPGPUserIDOrdering, keylist);
      if result <> 0 then exit;
      try
        result := PGPNewKeyIter(keylist, iter);
        if result <> 0 then exit;
        try
          { must iter by one to move off the iter's default 'null' }
          result := PGPKeyIterNext(iter, Key);
          if (result = kPGPError_EndOfIteration) then
            result := kPGPError_ItemNotFound;
          if result <> 0 then exit;

          // old school
          //result := GetKeyById(context,targetKey,LowerCase(StrPas(KeyHexID)),KeySet);

          // jeune ecole!
          // this technique can be used because ID is always hex
          result := PGPGetKeyIDFromString(KeyHexID, keyID);
          if result <> 0 then exit;
          result :=
          PGPGetKeyByKeyID(
            KeySet,
            keyID,
            kPGPPublicKeyAlgorithm_Invalid,
            targetKey);
                    
          if result <> 0 then exit;

          while PGPCompareKeys(targetKey,Key,2) <> 0 do
          result := PGPKeyIterNext(iter,Key);
          if (result = kPGPError_EndOfIteration) then
            result := kPGPError_ItemNotFound;
          if result <> 0 then exit;

          while PGPKeyIterNextUserID(iter,targetID) = 0 do begin
          StrCopy(astring,'');
          result :=
          PGPGetUserIDStringBuffer(
            targetID,
            kPGPUserIDPropName,
            bufferSize,
            astring,
            fullSize
            );
          if strcomp(UserID,astring) = 0 then BREAK;
          end;

          if result <> 0 then exit;
          result :=
          PGPSignUserID(targetID, SignKey,
            [
            PGPOPassphrase(context, SignKeyPass),
            PGPOExpiration(context, expirationDays),
            PGPOExportable(context, Exportable),
            PGPOSigTrust(context, 0{trustLevel}, kPGPKeyTrust_Complete)
            ]);

          if result <> 0 then exit;

          if Boolean(PGPKeySetNeedsCommit(KeySet))
          then result := PGPCommitKeyringChanges(KeySet);

        finally
          PGPFreeKeyIter(iter);
        end;
      finally
        PGPFreeKeyList(keylist);
      end;
    finally
      PGPFreeKeySet(fkeySet);
    end;
  finally
    PGPFreeFilter(filt);
  end;

  finally
    FreeMem(AString, BufferSize);
    QuitIt(Context, KeySet);
  end;
end;

(* REMOVE SIG *)
function spgpkeysigremove(KeyHexID,UserID,SignKeyID:PChar): LongInt;stdcall;export;
var
  Context        : pPGPContext;
  KeySet         : pPGPKeySet;
  filt           : pPGPFilter;
  fKeySet        : pPGPKeySet;
  keyList        : pPGPKeyList;
  keyID          : TPGPKeyID;
  iter           : pPGPKeyIter;
  n 		         : PGPUInt32;
  targetKeyID    : TPGPKeyID;
  signingKeyID   : TPGPKeyID;
  Key            : pPGPKey;
  targetKey      : pPGPKey;
  SignKey        : pPGPKey;
  targetID       : pPGPUserID;
  bufferSize     : PGPSIZE;
  astring        : PChar;
  fullSize       : PGPSIZE;
  s              : string;
  expirationDays : PGPUINT32;
  shortID        : boolean;
  sig            : pPGPSig;
  pid            : PGPChar;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeySet := nil;
  filt := nil;
  fKeySet := nil;
  keyList := nil;
  iter := nil;
  Key := nil;
  targetKey := nil;
  SignKey := nil;
  targetID := nil;
  sig := nil;

  result := Init(Context, KeySet, true, false);
  if result <> 0 then exit;

  bufferSize := 255;
  astring := AllocMem(buffersize);

  { is this a short ID or a complete User ID? }
  if (LowerCase(Copy(StrPas(UserID),1,2)) = '0x')
  then shortID := true
  else shortID := false;

  try
    { STEP ONE - Find the Target User ID }

    if result <> 0 then exit;
    if LowerCase(Copy(StrPas(UserID),1,2)) = '0x' then
      begin
      { convert ID string to inscrutable TPGPKeyID format }
      result := PGPGetKeyIDFromString(UserID, targetKeyID);
      if result <> 0 then exit;
      { create filter based on ID }
      result := PGPNewKeyIDFilter(context, targetKeyID, filt);
      shortID := true;
      end
    else
    result := PGPNewUserIDStringFilter(context, UserID, kPGPMatchEqual{kPGPMatchSubString}, filt);

    if result <> 0 then exit;
    try
    { filter main keyset into fKeySet based on ID/email filter }
    result := PGPFilterKeySet(KeySet, filt, fKeySet);
    if result <> 0 then exit;
    try
      { see what we've got in the net }
      result := PGPCountKeys(fKeySet, n);
      if result <> 0 then exit;
      if n <= 0 then begin
        result := SPGPError_KeyNotFound;
        exit;
        end
      else
      { sort the keys into a keylist }
      result := PGPOrderKeySet(fKeySet, kPGPUserIDOrdering, keylist);
      if result <> 0 then exit;
      try
        result := PGPNewKeyIter(keylist, iter);
        if result <> 0 then exit;
        try
          { must iter by one to move off the iter's default 'null' }
          result := PGPKeyIterNext(iter, Key);
          if (result = kPGPError_EndOfIteration) then
            result := kPGPError_ItemNotFound;
          if result <> 0 then exit;

          //result := GetKeyById(context,targetKey,LowerCase(StrPas(KeyHexID)),KeySet);

          // this technique can be used because ID is always hex
          result := PGPGetKeyIDFromString(KeyHexID, keyID);
          if result <> 0 then exit;
          result :=
          PGPGetKeyByKeyID(
            KeySet,
            keyID,
            kPGPPublicKeyAlgorithm_Invalid,
            targetKey);

          if result <> 0 then exit;

          while PGPCompareKeys(targetKey,Key,2) <> 0 do
          result := PGPKeyIterNext(iter,Key);

          while PGPKeyIterNextUserID(iter,targetID) = 0 do begin
          StrCopy(astring,'');
          result :=
          PGPGetUserIDStringBuffer(
            targetID,
            kPGPUserIDPropName,
            bufferSize,
            astring,
            fullSize);
          { stop when the extracted id matches our boy }
          if strcomp(UserID,astring) = 0 then BREAK;
          end;

          { PART TWO - REMOVE SIGNATURE }

          if result <> 0 then exit;

          { now we're on the right UserID, iter through the sigs }
          while PGPKeyIterNextUIDSig(iter, sig) = 0 do
          begin
            if result <> 0 then exit;
            result := PGPGetKeyIDOfCertifier(sig, signingKeyID);

            if result <> 0 then exit;
            { extract signing keyID as string }
            result :=
            PGPGetKeyIDString(
              signingKeyID,
              kPGPKeyIDString_Abbreviated,
              pid);
            { if the sig matches, remove it }
            if strcomp(SignKeyID,pid) = 0 then
              result := PGPRemoveSig(sig);
          end;

          if result <> 0 then exit;

          if Boolean(PGPKeySetNeedsCommit(KeySet)) then
            result := PGPCommitKeyringChanges(KeySet);

        finally
          PGPFreeKeyIter(iter);
        end;
      finally
        PGPFreeKeyList(keylist);
      end;
    finally
      PGPFreeKeySet(fkeySet);
    end;
  finally
    PGPFreeFilter(filt);
  end;

  finally
    FreeMem(astring,buffersize);
    QuitIt(Context, KeySet);
  end;
end;

(* REVOKE SIG *)
function spgpkeysigrevoke(UserID,SignKeyID,SignKeyPass:PChar): LongInt;stdcall;export;
var
  Context        : pPGPContext;
  KeySet         : pPGPKeySet;
  filt           : pPGPFilter;
  fKeySet        : pPGPKeySet;
  keyList        : pPGPKeyList;
  iter           : pPGPKeyIter;
  n              : PGPUInt32;
  targetKeyID    : TPGPKeyID;
  signingKeyID   : TPGPKeyID;
  Key            : pPGPKey;
  SignKey        : pPGPKey;
  targetID       : pPGPUserID;
  bufferSize     : PGPSIZE;
  astring        : PChar;
  fullSize       : PGPSIZE;
  s              : string;
  expirationDays : PGPUINT32;
  shortID        : boolean;
  sig            : pPGPSig;
  pid            : PGPChar;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeySet := nil;
  filt := nil;
  fKeySet := nil;
  keyList := nil;
  iter := nil;
  Key := nil;
  SignKey := nil;
  targetID := nil;
  astring := nil;
  sig := nil;

  result := Init(Context, KeySet, true, false);
  if result <> 0 then exit;

  bufferSize := 255;
  //GetMem(astring,buffersize);
  astring := AllocMem(buffersize);

  { is this a short ID or a complete User ID? }
  if (LowerCase(Copy(StrPas(UserID),1,2)) = '0x')
  then shortID := true
  else shortID := false;

  try
    { STEP ONE - Find the Target User ID }

    if result <> 0 then exit;
    if LowerCase(Copy(StrPas(UserID),1,2)) = '0x' then
      begin
      { convert ID string to inscrutable TPGPKeyID format }
      result := PGPGetKeyIDFromString(UserID, targetKeyID);
      if result <> 0 then exit;
      { create filter based on ID }
      result := PGPNewKeyIDFilter(context, targetKeyID, filt);
      shortID := true;
      end
    else
    result := PGPNewUserIDStringFilter(context, UserID, kPGPMatchEqual{kPGPMatchSubString}, filt);

    if result <> 0 then exit;
    try
    { filter main keyset into fKeySet based on ID/email filter }
    result := PGPFilterKeySet(KeySet, filt, fKeySet);
    if result <> 0 then exit;
    try
      { see what we've got in the net }
      result := PGPCountKeys(fKeySet, n);
      if result <> 0 then exit;
      if n <= 0 then begin
        result := SPGPError_KeyNotFound;
        exit;
        end
      else
      { sort the keys into a keylist }
      result := PGPOrderKeySet(fKeySet, kPGPUserIDOrdering, keylist);
      if result <> 0 then exit;
      try
        result := PGPNewKeyIter(keylist, iter);
        if result <> 0 then exit;
        try
          { must iter by one to move off the iter's default 'null' }
          result := PGPKeyIterNext(iter, Key);
          if (result = kPGPError_EndOfIteration) then
            result := kPGPError_ItemNotFound;
          if result <> 0 then exit;

          if shortID
          then result := PGPKeyIterNextUserID(iter,targetID)
          else
          while PGPKeyIterNextUserID(iter,targetID) = 0 do
          begin
            StrCopy(astring,'');
            result :=
            PGPGetUserIDStringBuffer(
              targetID,
              kPGPUserIDPropName,
              bufferSize,
              astring,
              fullSize);
            { stop when the extracted id matches our boy }
            if strcomp(UserID,astring) = 0 then BREAK;
          end;

          { PART TWO - REVOKE SIGNATURE }

          if result <> 0 then exit;

          { now we're on the right UserID, iter through the sigs }
          while PGPKeyIterNextUIDSig(iter, sig) = 0 do
          begin
            if result <> 0 then exit;
            result := PGPGetKeyIDOfCertifier(sig, signingKeyID);

            if result <> 0 then exit;
            { extract signing keyID as string }
            result := PGPGetKeyIDString(signingKeyID,
                                        kPGPKeyIDString_Abbreviated,
                                        pid);
            { if the sig matches ... }
            if strcomp(SignKeyID,pid) = 0 then BREAK;
          end;

          result := PGPRevokeSig(sig, fKeySet, [PGPOPassphrase(context,SignKeyPass)]);

          if result <> 0 then exit;

          if Boolean(PGPKeySetNeedsCommit(KeySet)) then
            result := PGPCommitKeyringChanges(KeySet);

        finally
          PGPFreeKeyIter(iter);
        end;
      finally
        PGPFreeKeyList(keylist);
      end;
    finally
      PGPFreeKeySet(fkeySet);
    end;
  finally
    PGPFreeFilter(filt);
  end;

  finally
    FreeMem(AString, BufferSize);
    QuitIt(Context, KeySet);
  end;
end;

initialization
  // check all pgp units used by this unit
  LibsAllLoaded := pgpUtilitiesLoaded and
                   pgpErrorsLoaded and
                   pgpKeysLoaded and
                   pgpOptionListLoaded;

  PGPversion5 := IsPGP5;
end.

