{************************************************************************}
{ spgpKeyProp - functions to return Key properties                       }
{                                                                        }
{ This unit requires one or more units from the Pascal translations of   }
{ the PGPsdk header files: <http://www.oz.net/~srheller/pgpsdk/>         }
{                                                                        }
{ The original Pascal code is: spgpKeyProp.pas, released 10 Apr 2000.    }
{ The initial developer of the Pascal code is Steven R. Heller           }
{ <srheller@oz.net>                                                      }
{                                                                        }
{ Portions created by Steven R. Heller are                               }
{ Copyright (C) 2000 Steven R. Heller                                    }
{                                                                        }
{ The contents of this file are used with permission, subject to         }
{ the Mozilla Public License Version 1.1 (the "License"); you may        }
{ not use this file except in compliance with the License. You may       }
{ obtain a copy of the License at                                        }
{ http://www.mozilla.org/MPL/MPL-1.1.html                                }
{                                                                        }
{ Software distributed under the License is distributed on an            }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or         }
{ implied. See the License for the specific language governing           }
{ rights and limitations under the License.                              }
{                                                                        }
{************************************************************************}

unit spgpKeyProp;

interface

uses
  // Delphi
  SysUtils, Classes,
  // PGPsdk
  pgpBase, pgpPubTypes, pgpKeys, pgpUtilities, pgpErrors, pgpFeatures,
  // SPGP
  spgpGlobals, spgpKeyUtil, spgpUtil,
  // always last
  PGPsdk;

  function spgpkeyprops(KeyID,KeyProps: PChar; KeyPropsLen: LongInt): LongInt;stdcall;export;
  function spgpkeyringid(BufferOut: PChar; BufferOutLen: LongInt): LongInt;stdcall;export;

  function spgpkeyringidex(BufferOut: PChar; BufferOutLen:LongInt): LongInt;stdcall;export;

  function spgpkeyringtofile(FileOut: PChar): LongInt;stdcall;export;
  function spgpkeyringcount: LongInt;stdcall;export;
  function spgpkeypropuserid(KeyID,BufferOut: PChar;BufferOutLen: LongInt): LongInt;stdcall;export;
  function spgpkeypropsig(UserID,BufferOut: PChar;BufferOutLen: LongInt): LongInt;stdcall;export;

  function spgpkeypropadk(KeyID, ADKProps: PChar; ADKPropsLen: LongInt; var ADKCount: LongInt; Flags: LongInt): LongInt;stdcall;export;
  function spgpkeyproprevocationkey(KeyID, RevKeyProps: PChar; RevKeyPropsLen: LongInt; var RevKeyCount: LongInt; Flags: LongInt): LongInt;stdcall;export;

  function spgpkeyisonring(KeyId: PChar): LongInt;stdcall;export;

implementation

var
  LibsAllLoaded  : boolean;
  PGPversion5    : boolean;
  
function spgpkeyprops(KeyID, KeyProps: PChar; KeyPropsLen: LongInt): LongInt;stdcall;export;
var
  Context        : pPGPContext;
  FilteredKeyList: pPGPKeyList;
  FilteredKeySet : pPGPKeySet;
  Flags          : Longint;
  FoundKey       : pPGPKey;
  Iter           : pPGPKeyIter;
  KeyFilter      : pPGPFilter;
  KeyRec         : TKeyData;
  KeySet         : pPGPKeySet;
  n              : PGPUInt32;
  s              : string;
  t              : shortstring;
  UseFlags       : boolean;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeyFilter := nil;
  KeySet := nil;
  FilteredKeySet := nil;
  FilteredKeyList := nil;
  FoundKey := nil;
  Iter := nil;

  try
    result := Init(Context, KeySet, false, false);
    if result <> 0 then exit;

  try
    // test for flags-string in BufferOut
    UseFlags := not ZeroLength(KeyProps);
    if UseFlags then
    begin
      Flags := StrToInt(KeyProps);
      //UseFlags := not (flags and spgpKeyPropFlag_OldStyle = spgpKeyPropFlag_OldStyle);
      StrCopy(KeyProps,'');
    end;

    result := PGPNewKeySet(context, FilteredKeyset);
    if result <> 0 then exit;

    { 1. convert BufferIn to Keyset }
    result := GetKeysByID(Context, FilteredKeyset, LowerCase(StrPas(KeyID)), KeySet);
    if result <> 0 then exit;

    { 2. for each key in the resulting keyset get keyprops }
    s := '';
    result := PGPCountKeys(FilteredKeyset, n);
    if result <> 0 then exit;

    { '0' should be impossible, since GetKeysByID would've failed
      if none of the keys are on the ring, but we check anyway }
    if n = 0 then
    begin
      result := SPGPError_KeyNotFound;
      exit;
    end;

    result := PGPOrderKeySet(FilteredKeyset, kPGPUserIDOrdering, FilteredKeyList);
    if result <> 0 then exit;

    result := PGPNewKeyIter(FilteredKeyList, Iter);
    if result <> 0 then exit;

    while IsntPGPError(PGPKeyIterNext(Iter, FoundKey)) and
      Assigned(FoundKey) do
    begin
      if UseFlags then
        KeyDataToStr(FoundKey, 0, 0, Flags, t)
      else
      if IsntPGPError(GetKeyData(context,FilteredKeyset,FoundKey,KeyRec)) then
        KeyRecToStr(KeyRec,t,10,true)
      else
        t := #13#10;
      s := s + t;
    end;

    if Length(s) > KeyPropsLen - 1 then
      result := Length(s) + 1
    else
      StrPLCopy(KeyProps, s, KeyPropsLen - 1); //-1 because of terminating #0

  finally
    PGPFreeKeyIter(Iter);
    PGPFreeKeyList(FilteredKeyList);
    PGPFreeKeySet(FilteredKeyset);
    QuitIt(Context, KeySet);
  end;
  
  except
    on E:Exception do
      result := SPGPError_KeySearchError;
  end;
end;

(* RETURN ALL IDs ON KEY RING *)
function spgpkeyringid(BufferOut: PChar; BufferOutLen:LongInt): LongInt;stdcall;export;
var
  Context        : pPGPKey;
  filt           : pPGPFilter;
  fKeySet        : pPGPKeySet;
  Flags          : Longint;
  Iter           : pPGPKeyIter;
  keyList        : pPGPKeyList;
  Key            : pPGPKey;
  KeyRec         : TKeyData;
  KeySet         : pPGPKeySet;
  n              : PGPUInt32;
  s              : string;
  t              : shortstring;
  UseFlags       : boolean;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  filt := nil;
  fKeySet := nil;
  keyList := nil;
  Key := nil;
  KeySet := nil;
  Iter := nil;

  try
    result := Init(Context, KeySet, false, false);
    if result <> 0 then exit;

    // test for flags-string in BufferOut
    UseFlags := not ZeroLength(BufferOut);
    if UseFlags then
    begin
      Flags := StrToInt(BufferOut);
      //UseFlags := not (flags and spgpKeyPropFlag_OldStyle = spgpKeyPropFlag_OldStyle);
      StrCopy(BufferOut,'');
    end;

  try
    { for each key in the keyset, call getkeyprops }
    s := '';
                                     //kPGPKeyIDOrdering
                                     //kPGPUserIDOrdering
    result := PGPOrderKeySet(KeySet, kPGPAnyOrdering, keylist);
    if result <> 0 then exit;

    result := PGPNewKeyIter(keylist, Iter);
    if result <> 0 then exit;

    result := PGPKeyIterRewind(Iter);
    if result <> 0 then exit;

    while IsntPGPError(PGPKeyIterNext(Iter, Key)) and
      Assigned(Key) do
    begin
      t := '';
      if UseFlags then
        KeyDataToStr(Key, 0, 0, Flags, t)
      else
        if IsntPGPError(GetKeyData(Context,fKeySet,Key,KeyRec)) then
          KeyRecToStr(KeyRec, t, 3, true)
        else
          t := #13#10;
      s := s + t;
    end;

    if Length(s) > BufferOutLen - 1 then
      result := Length(s) + 1
    else
      StrPLCopy(BufferOut, s, BufferOutLen - 1);

  finally
    PGPFreeKeyIter(Iter);
    PGPFreeKeyList(KeyList);
    QuitIt(Context,KeySet);
  end;

  except
    on E:Exception do
    result := SPGPError_KeySearchError;
  end;
end;

(* RETURN ALL IDs ON KEY RING -- ALT *)
function spgpkeyringidex(BufferOut: PChar; BufferOutLen:LongInt): LongInt;stdcall;export;
var
  Context        : pPGPKey;
  filt           : pPGPFilter;
  fKeySet        : pPGPKeySet;
  flags          : Longint;
  Iter           : pPGPKeyIter;
  keyList        : pPGPKeyList;
  Key            : pPGPKey;
  KeyRec         : TKeyData;
  KeySet         : pPGPKeySet;
  n              : PGPUInt32;
  s              : string;
  t              : shortstring;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  filt := nil;
  fKeySet := nil;
  keyList := nil;
  Key := nil;
  KeySet := nil;
  Iter := nil;

  try
  result := Init(Context, KeySet, false, false);
  if result <> 0 then exit;

  try
    { for each key in the keyset, call getkeyprops }
    s := '';
                                     //kPGPUserIDOrdering
                                     //kPGPKeyIDOrdering
    result := PGPOrderKeySet(KeySet, kPGPAnyOrdering, keylist);
    if result <> 0 then exit;
    try
      result := PGPNewKeyIter(keylist, Iter);
      if result <> 0 then exit;
      result := PGPKeyIterRewind(Iter);
      if result <> 0 then exit;
      try
        while IsntPGPError(PGPKeyIterNext(Iter, Key)) and
          Assigned(Key) do
        begin
          if not ZeroLength(BufferOut) then
            flags := StrToInt(BufferOut);

          KeyDataToStr(Key, 0, 0, flags, t);
          s := s + t + #13#10;
        end;
      finally
        PGPFreeKeyIter(Iter);
      end;
    finally
      PGPFreeKeyList(keylist);
    end;
    
    if Length(s) > BufferOutLen - 1
    then
      result := Length(s) + 1// kPGPError_OutputBufferTooSmall;
    else
      StrPLCopy(BufferOut, s, BufferOutLen - 1);

    finally
     QuitIt(Context,KeySet);
    end;
  except
    on E:Exception do
    result := SPGPError_KeySearchError;
  end;
end;

(* COUNT KEYS ON KEY-RING *)
function spgpkeyringcount: LongInt;stdcall;export;
var
  Context        : pPGPContext;
  KeySet         : pPGPKeySet;
  n              : PGPUInt32;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

try
  result := Init(Context, KeySet, false, false);
  if result <> 0 then exit;
  try
    result := PGPCountKeys(KeySet, n);
    if result < 0 then exit;
    result := n;
  finally
    QuitIt(Context, KeySet);
  end;
  except
    on E:Exception do
    result := SPGPError_KeySearchError;
  end;
end;

(* EXTRACT USER IDs from KEY *)
function spgpkeypropuserid(KeyID,BufferOut: PChar;BufferOutLen: LongInt): LongInt;stdcall;export;
var
  Context        : pPGPContext;
  Filter         : pPGPFilter;
  FilteredKeySet : pPGPKeySet;
  Iter           : pPGPKeyIter;
  Key            : pPGPKey;
  KeyCount       : PGPUInt32;
  KeyIDRec       : TPGPKeyID;
  KeyIDRef       : pPGPUserID;
  KeyList        : pPGPKeyList;
  KeySet         : pPGPKeySet;
  PropBool       : PGPBoolean;
  PropString     : PChar;
  s              : string;
  UsedSize       : PGPSize;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
  begin
    result := SPGPError_NoPGPsdk;
    exit;
  end;

  s := '';
  Context := nil;
  KeySet := nil;
  Filter := nil;
  FilteredKeySet := nil;
  keyList := nil;
  Iter := nil;
  Key := nil;
  KeyIDRef := nil;

  try
    result := Init(Context, KeySet, false, false);
    if result <> 0 then exit;

    try
      PropString := AllocMem(kPGPMaxUserIDSize);
      
      if LowerCase(Copy(StrPas(KeyID),1,2)) = '0x' then
      begin
        { convert ID string to inscrutable TPGPKeyID format }
        result := PGPGetKeyIDFromString(KeyID, KeyIDRec);
        if result <> 0 then exit;
        { create filter based on ID }
        result := PGPNewKeyIDFilter(context, KeyIDRec, Filter);
      end
      else
      begin
        //result := PGPNewUserIDEmailFilter(context, KeyID, kPGPMatchEqual, filt);
        result := PGPNewUserIDStringFilter(context, KeyID, kPGPMatchSubString, Filter);
      end;

      if result <> 0 then exit;

      { filter main keyset into fKeySet based on ID/email filter }
      result := PGPFilterKeySet(KeySet, Filter, FilteredKeySet);
      if result <> 0 then exit;

      { see what we've got in the net }
      result := PGPCountKeys(FilteredKeySet, KeyCount);
      if result <> 0 then exit;

      // bail out if zero or too many keys
      if KeyCount <> 1 then
      begin
        result := SPGPError_KeyNotFound;
        exit;
      end;

      { sort the keys into a keylist }
      result := PGPOrderKeySet(FilteredKeySet, kPGPUserIDOrdering, keylist);
      if result <> 0 then exit;

      { create a new iter on the sorted keylist }
      result := PGPNewKeyIter(KeyList, Iter);
      if result <> 0 then exit;

      { must Iter by one to move off the Iter's default 'null' }
      if PGPKeyIterNext(Iter, Key) = kPGPError_EndOfIteration then
        result := kPGPError_ItemNotFound;
      if result <> 0 then exit;
 
      { make sure we're back at the very first ID }
      result := PGPKeyIterRewindUserID(Iter);
      if result <> 0 then exit;

      { move through the userids, extracting the id as a string }
      while PGPKeyIterNextUserID(Iter, KeyIDRef) = kPGPError_NoError do
      begin
        StrCopy(PropString, '');

        // if PGP 6+, check for 'attributes' like photos
        if not PGPversion5 then //APIVersion >= $03000000 then
        begin
          result := PGPGetUserIDBoolean(KeyIDRef, kPGPUserIDPropIsAttribute,
            PropBool);
          // if the property isn't an attribute, i.e. is an ID ...  
          if not Boolean(PropBool) then
          begin
            result := PGPGetUserIDStringBuffer(KeyIDRef, kPGPUserIDPropName,
              kPGPMaxUserIDSize, PropString, UsedSize);
            s := s + StrPas(PropString) + #9;
          end;
        end
        else
        begin
          // if PGP55, no photos, so no need to check
          result := PGPGetUserIDStringBuffer(KeyIDRef,
            kPGPUserIDPropName, kPGPMaxUserIDSize, PropString, UsedSize);
          s := s + StrPas(PropString) + #9;
        end;
      end;

    { trim off trailing tabs }
    if s <> '' then
    while s[Length(s)] = #9 do
      s := Copy(s,0,Length(s) -1);

    { check length and copy results to buffer }
    if Length(s) > BufferOutLen -1
    then result := Length(s)+1//kPGPError_OutputBufferTooSmall
    else StrPLCopy(BufferOut,s,BufferOutLen -1);

    finally
      FreeMem(PropString, kPGPMaxUserIDSize);
      PGPFreeKeyIter(Iter);
      PGPFreeKeyList(KeyList);
      PGPFreeKeySet(FilteredKeySet);
      PGPFreeFilter(Filter);
      QuitIt(Context, KeySet);
    end;

  except
    on E:Exception do
    result := SPGPError_KeySearchError;
  end;
end;

(* EXTRACT SIGS *)
function spgpkeypropsig(UserID,BufferOut:PChar;BufferOutLen: LongInt): LongInt;stdcall;export;
var
  Context        : pPGPContext;
  KeySet         : pPGPKeySet;
  filt           : pPGPFilter;
  fKeySet        : pPGPKeySet;
  keyList        : pPGPKeyList;
  Iter           : pPGPKeyIter;
  n              : PGPUInt32;
  targetKeyID    : TPGPKeyID;
  signingKeyID   : TPGPKeyID;
  Key            : pPGPKey;
  SignKey        : pPGPKey;
  TargetID       : pPGPUserID;
  bufferSize     : PGPSize;
  fullSize       : PGPSize;
  astring        : PChar;
  s              : string;
  expirationDays : PGPUINT32;
  sig            : pPGPSig;
  pid            : PGPChar;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeySet := nil;
  filt := nil;
  fKeySet := nil;
  keyList := nil;
  Iter := nil;
  Key := nil;
  SignKey := nil;
  targetID := nil;
  astring := nil;
  sig := nil;

  try
  result := Init(Context, KeySet, false, false);

  if result <> 0 then exit;

  bufferSize := 255;
  astring := AllocMem(buffersize);

  try
    { STEP ONE - Find the Target User ID }
    result := PGPNewUserIDStringFilter(context, UserID, kPGPMatchEqual{kPGPMatchSubString}, filt);

    if result <> 0 then exit;
    try
    { filter main keyset into fKeySet based on ID/email filter }
    result := PGPFilterKeySet(KeySet, filt, fKeySet);
    if result <> 0 then exit;
    try
      { see what we've got in the net }
      result := PGPCountKeys(fKeySet, n);
      if result <> 0 then exit;
      if n <= 0 then begin
        result := SPGPError_KeyNotFound;
        exit;
        end
      else
      { sort the keys into a keylist }
      result := PGPOrderKeySet(fKeySet, kPGPUserIDOrdering, keylist);
      if result <> 0 then exit;
      try
        result := PGPNewKeyIter(keylist, Iter);
        if result <> 0 then exit;
        try
          { must Iter by one to move off the Iter's default 'null' }
          result := PGPKeyIterNext(Iter, Key);
          if (result = kPGPError_EndOfIteration) then
            result := kPGPError_ItemNotFound;
          if result <> 0 then exit;

          while result = 0 do begin
          StrCopy(astring,'');
          result := PGPKeyIterNextUserID(Iter,targetID);
          result := PGPGetUserIDStringBuffer(targetID,
            kPGPUserIDPropName, bufferSize, astring, fullSize);

          { stop when the extracted id matches our boy }
          if strcomp(UserID,astring) = 0 then BREAK;
          end;

          { PART TWO - phew! - EXTRACT SIGNATURES }

          if result <> 0 then exit else
          { now we've Iterated to the right UserID, Iter through the sigs }
          while PGPKeyIterNextUIDSig(Iter, sig) = 0 do
            begin
            if result <> 0 then exit;
            result := PGPGetKeyIDOfCertifier(sig, signingKeyID);

            if result <> 0 then exit;
            result := PGPGetKeyIDString(signingKeyID,
                                        kPGPKeyIDString_Abbreviated,
                                        pid);
            s := s + StrPas(pid) + #9;
            end;
        finally
          PGPFreeKeyIter(Iter);
        end;
      finally
        PGPFreeKeyList(keylist);
      end;
    finally
      PGPFreeKeySet(fkeySet);
    end;
  finally
    PGPFreeFilter(filt);
  end;

  if s <> '' then
  while s[Length(s)] = #9 do
    s := Copy(s,0,Length(s)-1);

  if Length(s) > BufferOutLen -1 then
  result := Length(s) + 1//kPGPError_OutputBufferTooSmall
  else StrPLCopy(BufferOut,s,BufferOutLen-1);

  finally
    FreeMem(astring, buffersize);
    QuitIt(Context, KeySet);
  end;
  except
    on E:Exception do
    result := SPGPError_KeySearchError;
  end;
end;

(* EXTRACT Additional Decryption Key properties *)
function spgpkeypropadk(KeyID, ADKProps: PChar; ADKPropsLen: Longint; var ADKCount: Longint; Flags: LongInt): LongInt;stdcall;export;
var
  ARClass        : PGPByte;
  ARKey          : pPGPKey;
  ARKeyCount     : PGPUint32;
  ARKeyID        : TPGPKeyID;
  Context        : pPGPContext;
  Key            : pPGPKey;
  KeySet         : pPGPKeySet;
  i              : integer;
  KeyAlg         : PGPInt32;
  KeyIDStr       : PGPChar;
  NoFlags        : Longint;
  s              : string;
  TempStr        : shortstring;
  YesFlags       : Longint;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  ARKey := nil;
  Key := nil;
  KeySet := nil;

  ADKCount := 0;
  YesFlags := 0;
  NoFlags := 0;
  s := '';

  result := Init(Context, KeySet, false, false);
  if result <> 0 then exit;

  try
    result := GetKeyById(Context, Key, KeyID, KeySet);
    if result <> 0 then exit;
    
    // check for algorithm
    result := PGPGetKeyNumber(Key, kPGPKeyPropAlgID, KeyAlg);
    if result <> 0 then exit;

    // only DSA keys have ARR
    if KeyAlg <> kPGPPublicKeyAlgorithm_DSA then
    begin
      result := kPGPError_ItemNotFound;
      exit;
    end;

    // look for ARR
    result := PGPCountAdditionalRecipientRequests(Key, ARKeyCount);
    if result <> 0 then exit;

    if ARKeyCount > 0 then
    for i := 0 to ARKeyCount -1 do
    begin
      if PGPversion5 then
      begin
        // PGP55
        // first, get the ARR's Key ID
        result := PGPGetIndexedAdditionalRecipientRequestKeyID(
          Key, i, ARKeyID, ARClass);
        if result <> 0 then exit;

        // get a string from Key ID
        result := PGPGetKeyIDString(ARKeyID,
          kPGPKeyIDString_Abbreviated, KeyIDStr);
        if result <> 0 then exit;

        // if key is on ring, get props ...
        if KeyIsOnRing(Context, KeySet, LowerCase(StrPas(KeyIDStr))) then
        begin
          result := PGPGetIndexedAdditionalRecipientRequest(
            Key, KeySet, i, ARKey, ARClass);
          if result <> 0 then exit;

          if ARClass > 0 then
            YesFlags := spgpKeyPropFlag_HasADK
          else
            NoFlags := spgpKeyPropFlag_HasADK;

          KeyDataToStr(ARKey, YesFlags, NoFlags, Flags, TempStr);
          s := s + TempStr;
        end
        // ... if not, return KeyID
        else
          if ARClass > 0 then
            s := s + KeyIDStr + #9 + '1' + #13#10
          else
            s := s + KeyIDStr + #9 + '0' + #13#10;
      end// if not PGP6
      else
      begin
        // PGP60+
        // this version gets key and id
        result := PGPGetIndexedAdditionalRecipientRequestKey(
          Key, KeySet, i, ARKey, ARKeyID, ARClass);
        if result <> 0 then exit;

        if Assigned(ARKey) then
        begin
          // if ADK flag is set, check for enforcement
          if Flags and spgpKeyPropFlag_HasADK = spgpKeyPropFlag_HasADK then
            if ARClass > 0 then
              YesFlags := spgpKeyPropFlag_HasADK
            else
              NoFlags := spgpKeyPropFlag_HasADK;

          KeyDataToStr(ARKey, YesFlags, NoFlags, Flags, TempStr);
          s := s + TempStr;
        end
        else
        begin
          // get a string from Key ID
          result := PGPGetKeyIDString(ARKeyID,
            kPGPKeyIDString_Abbreviated, KeyIDStr);
          if result <> 0 then exit;

          // if ADK flag is set, check for enforcement
          if flags and spgpKeyPropFlag_HasADK = spgpKeyPropFlag_HasADK then
            if ARClass > 0 then
              s := s + KeyIDStr + #9 + '1' + #13#10
            else
              s := s + KeyIDStr + #9 + '0' + #13#10
          else
            s := s + KeyIDStr + #13#10;
        end;
      end;
    end;// for

    ADKCount := ARKeyCount;

    if Length(s) > ADKPropsLen - 1
    then result := Length(s) + 1
    else StrPLCopy(ADKProps, s, ADKPropsLen - 1);

  finally
    QuitIt(Context, KeySet);
  end;
end;

(* EXTRACT Designated Revocation Key properties *)
function spgpkeyproprevocationkey(KeyID, RevKeyProps: PChar; RevKeyPropsLen: LongInt; var RevKeyCount: LongInt; Flags: LongInt): LongInt;stdcall;export;
var
  Context        : pPGPContext;
  i              : integer;
  TargetKey      : pPGPKey;
  KeySet         : pPGPKeySet;
  TargetKeyAlg   : PGPInt32;
  RevocationKey  : pPGPKey;
  RevocationKeyCount : PGPUint32;
  RevocationKeyID : TPGPKeyID;
  RevocationKeyIDStr : PGPChar;
  s              : string;
  TempStr        : shortstring;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  if PGPversion5 then
    result := kPGPError_IncompatibleAPI;

  if result <> 0 then exit;

  RevocationKey := nil;
  TargetKey := nil;
  KeySet := nil;

  RevocationKeyCount := 0;
  s := '';

  result := Init(Context, KeySet, false, false);
  if result <> 0 then exit;

  try
    result := GetKeyById(Context, TargetKey, KeyID, KeySet);
    if result <> 0 then exit;

    // check for algorithm
    result := PGPGetKeyNumber(TargetKey, kPGPKeyPropAlgID, TargetKeyAlg);
    if result <> 0 then exit;

    // only DSA keys have Revokers
    if TargetKeyAlg <> kPGPPublicKeyAlgorithm_DSA then
    begin
      result := kPGPError_ItemNotFound;
      exit;
    end;

    // look for Revocation Key
    result := PGPCountRevocationKeys(TargetKey, RevocationKeyCount);
    if result <> 0 then exit;

    if RevocationKeyCount > 0 then
    for i := 0 to RevocationKeyCount -1 do
    begin
      // get key and keyid
      result := PGPGetIndexedRevocationKey(TargetKey,
        KeySet, i, RevocationKey, RevocationKeyID);
      if result <> 0 then exit;

      // if key is on ring, get properties ...
      if Assigned(RevocationKey) then
      begin
        KeyDataToStr(RevocationKey, 0, 0, Flags, TempStr);
        s := s + TempStr;
      end                        
      // ... if not, return KeyID
      else
      begin
        result := PGPGetKeyIDString(RevocationKeyID,
          kPGPKeyIDString_Abbreviated, RevocationKeyIDStr);
        if result <> 0 then exit;
        s := s + RevocationKeyIDStr + #13#10;
      end;
    end;
    
    RevKeyCount := RevocationKeyCount;

    if Length(s) > RevKeyPropsLen - 1 then
      result := Length(s) + 1
    else
      StrPLCopy(RevKeyProps, s, RevKeyPropsLen - 1);

  finally
    QuitIt(Context, KeySet);
  end;
end;

function spgpkeyringtofile(FileOut: PChar): LongInt;stdcall;export;
var
  Context        : pPGPContext;
  Flags          : Longint;
  KeySet         : pPGPKeySet;
  OutPut         : TStringList;
  filt           : pPGPFilter;
  fKeySet        : pPGPKeySet;
  keyList        : pPGPKeyList;
  Key            : pPGPKey;
  Iter           : pPGPKeyIter;
  n              : PGPUInt32;
  s              : shortstring;
  i              : integer;
  KeyRec         : TKeyData;
  UseFlags       : boolean;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeySet := nil;
  filt := nil;
  fKeySet := nil;
  keyList := nil;
  Key := nil;
  Iter := nil;

  try
  result := Init(Context,KeySet,false, false);

  if result <> 0 then exit;
{
    // test for flags-string in BufferOut
    UseFlags := not ZeroLength(BufferOut);
    if UseFlags then
    begin
      Flags := StrToInt(BufferOut);
      //UseFlags := not (flags and spgpKeyPropFlag_OldStyle = spgpKeyPropFlag_OldStyle);
      StrCopy(BufferOut,'');
    end;
 }
  // we'll use the above for v3; until then ...
  Flags := 0;
  UseFlags := false;
  
  OutPut := TStringList.Create;

    { for each key in the keyset, call getkeyprops }
    try
      s := '';                            //kPGPUserIDOrdering
      result := PGPOrderKeySet(KeySet, kPGPAnyOrdering, keylist);
      if result <> 0 then exit;
      try
        result := PGPNewKeyIter(keylist, Iter);
        if result <> 0 then exit;
        result := PGPKeyIterRewind(Iter);
        if result <> 0 then exit;
        try
          while IsntPGPError(PGPKeyIterNext(Iter, Key)) and
            Assigned(Key) do
          begin
            s := '';
            if UseFlags then
              KeyDataToStr(Key, 0, 0, Flags, s)
            else
            { suppress errors here to allow skipping a bad key }
            if IsntPGPError(GetKeyData(context, KeySet, Key, KeyRec)) then
              KeyRecToStr(KeyRec, s, 10, false);

            // trim CRLF as needed
            while (s[Length(s)] = #13) or (s[Length(s)] = #10) do
              s := Copy(s, 1, Length(s) - 1);

            OutPut.Add(s);
          end;
        finally
          PGPFreeKeyIter(Iter);
        end;
      finally
        PGPFreeKeyList(keylist);
      end;

    finally
      OutPut.SaveToFile(StrPas(FileOut));
      OutPut.Free;
      QuitIt(Context, KeySet);
    end;
  except
    on E:Exception do
    result := SPGPError_KeySearchError;
  end;
end;

function spgpkeyisonring(KeyId: PChar): LongInt; stdcall; export;
var
  Context        : pPGPContext;
  KeySet         : pPGPKeySet;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  result := Init(Context, KeySet, false, false);
  if result <> 0 then exit;
  try
    if not KeyIsOnRing(Context, KeySet, LowerCase(StrPas(KeyId)))
    then result := -1
    else result := 0;
  finally
    QuitIt(Context, KeySet);
  end;
end;



(*
function spgpsubkeyprops(pKeyID, SubKeyProps: PChar; SubKeyPropsLen: LongInt): LongInt;stdcall;export;
var
  Context        : pPGPContext;
  FilteredKeyList: pPGPKeyList;
  FilteredKeySet:  pPGPKeySet;
  FoundKey:        pPGPKey;
  Iter:            pPGPKeyIter;
  KeyFilter:       pPGPFilter;
  KeyID:           string;
  KeyRec:          TKeyData;
  KeySet         : pPGPKeySet;
  n:               PGPUInt32;
  s:               string;
  t:               shortstring;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeyFilter := nil;
  KeySet := nil;
  FilteredKeySet := nil;
  FilteredKeyList := nil;
  FoundKey := nil;
  Iter := nil;
  try
  result := Init(Context, KeySet, false, false);

  if result <> 0 then exit;

  KeyID := LowerCase(StrPas(pKeyID));

  try
    result := PGPNewKeySet(context, FilteredKeyset);

    if result <> 0 then exit;

    { 1. convert BufferIn to Keyset }
    result := GetKeysByID(Context, FilteredKeyset, KeyID, KeySet);
    if result <> 0 then exit;

  { 2. for each key in the resulting keyset, call getkeyprops }
    try
      s := '';
      result := PGPCountKeys(FilteredKeyset, n);
      if (result <> 0) or (n = 0) then exit
      else
      result := PGPOrderKeySet(FilteredKeyset, kPGPUserIDOrdering, FilteredKeyList);
      if result <> 0 then exit;
      try
        result := PGPNewKeyIter(FilteredKeyList, Iter);
        if result <> 0 then exit;
        try
          while IsntPGPError(PGPKeyIterNext(Iter, FoundKey)) and
            Assigned(FoundKey) do
            begin
            if IsntPGPError(GetKeyData(context,FilteredKeyset,FoundKey,KeyRec)) then begin
              t := '';
              KeyRecToStr(KeyRec,t,10,true);
              s := s + t;
              end
            else
            s := s + '';
            end;
        finally
          PGPFreeKeyIter(Iter);
        end;
      finally
        PGPFreeKeyList(FilteredKeyList);
      end;
    finally
      PGPFreeKeySet(FilteredKeyset);
    end;

    if Length(s) > KeyPropsLen -1
    then result := Length(s) + 1 //kPGPError_OutputBufferTooSmall
    else StrPLCopy(KeyProps,s,KeyPropsLen-1); //-1 because of terminating #0
        { error code if output is larger than caller's buffer }

  finally
   QuitIt(Context, KeySet);
  end;
  except
    on E:Exception do
    result := SPGPError_KeySearchError;
  end;
end;
*)

initialization
  // check all pgp units used by this unit
  LibsAllLoaded := pgpUtilitiesLoaded and
                   pgpErrorsLoaded and
                   pgpKeysLoaded;

  PGPversion5 := IsPGP5;
  //if PGPversion5 then kPGPUserIDPropName := 50;
end.
