{************************************************************************}
{ spgpKeyUtil - internal key-handling functions                          }
{                                                                        }
{ This unit requires one or more units from the Pascal translations of   }
{ the PGPsdk header files: <http://www.oz.net/~srheller/pgpsdk/>         }
{ 								                                                       }
{ The original Pascal code is: spgpKeyUtil.pas, released 10 Apr 2000.    }
{ The initial developer of the Pascal code is Steven R. Heller           }
{ <srheller@oz.net>                                                      }
{ 								                                                       }
{ Portions created by Steven R. Heller are    				                   }
{ Copyright (C) 2000 Steven R. Heller                                    }
{									                                                       }
{ The contents of this file are used with permission, subject to         }
{ the Mozilla Public License Version 1.1 (the "License"); you may        }
{ not use this file except in compliance with the License. You may       }
{ obtain a copy of the License at                                        }
{ http://www.mozilla.org/MPL/MPL-1.1.html 	                             }
{ 									                                                     }
{ Software distributed under the License is distributed on an 	         }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or         }
{ implied. See the License for the specific language governing           }
{ rights and limitations under the License.                              }
{                                                                        }
{************************************************************************}

unit spgpKeyUtil;

interface

uses
  // Delphi
  Classes, SysUtils,   dialogs,
  // PGPsdk
  pgpBase, pgpPubTypes, pgpKeys, pgpErrors, pgpUtilities,
  // SPGP
  spgpUtil, spgpGlobals,
  // always last
  PGPsdk;

  function GetKeyByUserID(const context: pPGPContext;var Key: pPGPKey;const Email: string; target_KeySet: pPGPKeySet):PGPError;
  function GetKeySetByUserID(const context: pPGPContext;Keys: pPGPKeySet;const Email:string;target_KeySet:pPGPKeySet):PGPError;
  function GetKeySetByKeyID(const context: pPGPContext;var Keys: pPGPKeySet;const ID:string;target_KeySet:pPGPKeySet):PGPError;
  function GetKeyByKeyID(const context: pPGPContext;var Key: pPGPKey; const ID: string;target_KeySet:pPGPKeySet):PGPError;
  function KeyIsOnRing(const context: pPGPContext; const target_KeySet: pPGPKeySet;const ID: string): Boolean;
  function KeysAreOnRing(const context: pPGPContext; const target_KeySet: pPGPKeySet;const ID: string): Boolean;
  function GetKeyData(context: pPGPContext; KeySet: pPGPKeySet; Key: pPGPKey; var KeyRec: TKeyData): PGPError;

  { key properties: the saga continues }
  function GetKeyPropKeyTime(const Key: pPGPKey; var t: PGPInt32): PGPError;
  function GetKeyPropKeyTimeStr(const Key: pPGPKey; var s: shortstring): PGPError;
  function GetKeyPropKeyCreationTime(const Key: pPGPKey; var t: PGPInt32): PGPError;
  function GetKeyPropKeyCreationTimeStr(const Key: pPGPKey; var s: shortstring): PGPError;
  function GetKeyPropKeyExpirationTime(const Key: pPGPKey; var t: PGPInt32): PGPError;
  function GetKeyPropKeyExpirationTimeStr(const Key: pPGPKey; var s: shortstring): PGPError;
  function GetKeyPropFingerprint(const Key: pPGPKey; var s: shortstring): PGPError;
  function GetKeyPropKeybits(const Key: pPGPKey; var s: shortstring): PGPError;
  function GetKeyPropUserID(const Key: pPGPKey; var s: shortstring): PGPError;
  function GetKeyPropKeyID(const Key: pPGPKey; var s: shortstring): PGPError;
  function GetKeyPropAlg(const Key: pPGPKey; var t: integer): PGPError;

  function GetKeyPropARR(const Key: pPGPKey; var count: integer; var s: shortstring): PGPError;
  function GetKeyPropRevokers(const Key: pPGPKey; var count: integer; var s: shortstring): PGPError;

  // boolean properties
  function GetKeyPropIsSecret(const Key: pPGPKey; var t: integer): PGPError;
  function GetKeyPropIsAxiomatic(const Key: pPGPKey; var t: integer): PGPError;
  function GetKeyPropIsRevoked(const Key: pPGPKey; var t: integer): PGPError;
  function GetKeyPropIsDisabled(const Key: pPGPKey; var t: integer): PGPError;
  function GetKeyPropIsExpired(const Key: pPGPKey; var t: integer): PGPError;
  function GetKeyPropCanEncrypt(const Key: pPGPKey; var t: integer): PGPError;
  function GetKeyPropCanDecrypt(const Key: pPGPKey; var t: integer): PGPError;
  function GetKeyPropCanSign(const Key: pPGPKey; var t: integer): PGPError;
  function GetKeyPropCanVerify(const Key: pPGPKey; var t: integer): PGPError;
  function GetKeyPropIsSecretShared(const Key: pPGPKey; var t: integer): PGPError;
  function GetKeyPropHasRevoker(const Key: pPGPKey; var t: integer): PGPError;
  function GetKeyPropHasARR(const Key: pPGPKey; var t: integer): PGPError;

  //sub-keys
  function GetSubKeyPropKeyID(const SubKey: pPGPSubKey; var s: shortstring): PGPError;
  function GetSubKeyPropKeybits(const SubKey: pPGPSubKey; var s: shortstring): PGPError;

  function KeyRecToStr(const KeyRec: TKeyData;var s: shortstring; count: byte;crlf: boolean): PGPError;

  function KeyDataToStr(const Key: pPGPKey; const yesflags: Longint; const noflags: Longint; const flags: Longint; var s: shortstring): PGPError;

  function GetKeySetPropsOldStyle(Context: pPGPContext; KeySet: pPGPKeySet; var KeyProps: string; AllProps: boolean): LongInt;
  // with flags
  function GetKeySetProps(Context: pPGPContext; KeySet: pPGPKeySet; var KeyProps: string; const Flags: Longint): LongInt;

  { wrappers }
  function GetKeyByID(const context: pPGPContext;var Key: pPGPKey;const ID: string; target_KeySet: pPGPKeySet):PGPError;
  function GetKeysByID(const context: pPGPContext;var Keys: pPGPKeySet;const ID:string;target_KeySet:pPGPKeySet):PGPError;

implementation

var
  PGPversion5    : boolean;
  
{ wrappers to the most common combinations }
function GetKeyByID(const context: pPGPContext;var Key: pPGPKey;const ID: string; target_KeySet: pPGPKeySet):PGPError;
begin
  if LowerCase(Copy(ID,1,2)) = '0x' then
  result := GetKeyByKeyID(context,Key,ID,target_KeySet)
  else
  result := GetKeyByUserID(context,Key,ID,target_KeySet);
end;

function GetKeysByID(const context: pPGPContext;var Keys: pPGPKeySet;const ID:string;target_KeySet:pPGPKeySet):PGPError;
begin
  if LowerCase(Copy(ID,1,2)) = '0x' then
  result := GetKeySetByKeyID(context,Keys,ID,target_KeySet)
  else
  result := GetKeySetByUserID(context,Keys,ID,target_KeySet);
end;

{ get a single key from an e-mail address}
function GetKeyByUserID(const context: pPGPContext;var Key: pPGPKey;const Email: string; target_KeySet: pPGPKeySet):PGPError;
var
  FilteredKeyList: pPGPKeyList;
  FilteredKeySet : pPGPKeySet;
  Iter           : pPGPKeyIter;
  KeyIDFilter    : pPGPFilter;
  n              : PGPUInt32;
  s              : string;
begin
  KeyIDFilter := nil;
  FilteredKeySet := nil;
  FilteredKeyList := nil;
  Iter := nil;
  
  try

  if Email = '' then
  begin
    result := SPGPError_KeyNotFound;//key not found
    exit;
  end
  else
  s := Email;

  // 1. create a filter
  result := PGPNewUserIDStringFilter(context, pchar(s),
    kPGPMatchSubString{kPGPMatchEqual}, KeyIDFilter);
  if result <> 0 then exit;

  try
    // 2. filter the source keyset, sending the result to FilteredKeySet
    result := PGPFilterKeySet(target_KeySet, KeyIDFilter, FilteredKeySet);
    if result <> 0 then exit;

    try
      // 3. count the keys in FilteredKeySet; zero = no matches
      result := PGPCountKeys(FilteredKeySet, n);
      if result <> 0 then exit;
      if n = 0 then begin
        result := -1;
        exit;
        end;
      // 4. move & sort the found keys into a keylist
      result := PGPOrderKeySet(FilteredKeySet, kPGPUserIDOrdering, FilteredKeyList);
      if result <> 0 then exit;
      try
        // 5. create a new 'iter', whatever that is
        result := PGPNewKeyIter(FilteredKeyList, iter);
        if result <> 0 then exit;

        try
          // 6. walk to the first key, that's the searched-for key
          result := PGPKeyIterNext(iter, Key);
          if (result = kPGPError_EndOfIteration) then
            result := kPGPError_ItemNotFound;

        finally
          // 7. take the little iter to the woods & set it free
          PGPFreeKeyIter(iter);
        end;
      finally
        PGPFreeKeyList(FilteredKeyList);
      end;
    finally
      PGPFreeKeySet(FilteredKeySet);
    end;
  finally
    PGPFreeFilter(KeyIDFilter);
  end;
  except
    on E:Exception do
    result := SPGPError_KeySearchError;
  end;
end;

{ get a keySET from a list of 1 or more e-mail addresses }
function GetKeySetByUserID(const context: pPGPContext;Keys: pPGPKeySet;const Email:string;target_KeySet:pPGPKeySet):PGPError;
var
  filt           : pPGPFilter;
  fKeySet        : pPGPKeySet;
  i              : integer;
  List           : TStringList;
  n              : PGPUInt32;
  s              : string;
begin
  result := kPGPError_NoError;

  filt := nil;
  fKeySet := nil;

  try
  List := TStringList.Create;

  try
  { 1. convert comma-delimited text to array of strings }
  ParseEmailList(Email,List);

  { 2. for each e-mail string in the array, retrieve the appropriate key }
  for i := 0 to List.Count -1 do
    begin

    if List[i] <> ''
    then s := List[i]
    else begin
     result := SPGPError_KeyNotFound;
     exit;
     end;

    result := PGPNewUserIDStringFilter(context,pchar(s),kPGPMatchSubString,filt);
    if result <> 0 then exit;
    try
      result := PGPFilterKeySet(target_KeySet, filt, fKeySet);
      if (result <> 0) or not Assigned(fKeySet) then exit;
      try
        result := PGPCountKeys(fKeySet, n);
        if result <> 0 then exit else
        if n = 0 then begin
          result := SPGPError_KeyNotFound;
          exit;
          end;
        if n > 0 then result := PGPAddKeys(fKeySet,Keys);
        if result <> 0 then exit else
      finally
        PGPFreeKeySet(fkeySet);
      end;
    finally
      PGPFreeFilter(filt);
    end;
  end;

  finally
    List.free;
  end;
  except
    on E:Exception do
    result := SPGPError_KeySearchError;
  end;
end;

{ get keySET from 1 or more key ID numbers }
function GetKeySetByKeyID(const context: pPGPContext;var Keys: pPGPKeySet;const ID:string;target_KeySet:pPGPKeySet):PGPError;
var
  filt: 	  pPGPFilter;
  fKeySet:	pPGPKeySet;
  n:		    PGPUInt32;
  List:	    TStringList;
  KeyID:	  TPGPKeyID;
  i:		    integer;
  s:		    string;
begin
  result := kPGPError_NoError;

  filt := nil;
  fKeySet := nil;

  try
  List := TStringList.Create;

  try
  { 1. convert comma-delimited list to array of strings }
  ParseIDList(ID,List);
  
  { 2. for each ID string in the array, retrieve the appropriate key}
  for i := 0 to List.Count -1 do begin

  if List.Strings[i] <> ''
  then s := List.Strings[i]
  else begin
    result := SPGPError_KeyNotFound;
    exit;
    end;
  { convert ID string to inscrutable TPGPKeyID format }
  result := PGPGetKeyIDFromString(PChar(s), KeyID);
  if result <> 0 then exit;
  result := PGPNewKeyIDFilter(context, KeyID, filt);
  if result <> 0 then exit;
  try
    result := PGPFilterKeySet(target_KeySet, filt, fKeySet);
    if result <> 0 then exit;
    try
      result := PGPCountKeys(fKeySet, n);
      if result <> 0 then exit;
      if n = 0 then begin
        result := SPGPError_KeyNotFound;
        exit;
        end;
      if n > 0 then result := PGPAddKeys(fKeySet,Keys);
      if result <> 0 then exit;
    finally
      PGPFreeKeySet(fkeySet);
    end;
  finally
    PGPFreeFilter(filt);
  end;
end;

finally
  List.Free;
end;
  except
    on E:Exception do
    result := SPGPError_KeySearchError;
  end;

end;

{ get single key from hex ID }
function GetKeyByKeyID(const context: pPGPContext;var Key: pPGPKey; const ID: string;target_KeySet:pPGPKeySet):PGPError;
var
  filt:	    pPGPFilter;
  fKeySet:	pPGPKeySet;
  keyList:	pPGPKeyList;
  iter:	    pPGPKeyIter;
  n:		    PGPUInt32;
  KeyID:	  TPGPKeyID;
begin
  result := kPGPError_NoError;

  filt := nil;
  fKeySet := nil;
  keyList := nil;
  iter := nil;

  try
  { convert ID string to inscrutable TPGPKeyID format }
  result := PGPGetKeyIDFromString(PChar(ID), KeyID);
  if result <> 0 then exit;
  result := PGPNewKeyIDFilter(context, KeyID, filt);
  if result <> 0 then exit;
  try
    result := PGPFilterKeySet(target_KeySet, filt, fKeySet);
    if result <> 0 then exit;
    try
      result := PGPCountKeys(fKeySet, n);
      if result <> 0 then exit;
      result := PGPOrderKeySet(fKeySet, kPGPUserIDOrdering, keylist);
      if result <> 0 then exit;
      try
        result := PGPNewKeyIter(keylist, iter);
        if result <> 0 then exit;
        try
          result := PGPKeyIterNext(iter, Key);
          if (result = kPGPError_EndOfIteration) then
            result := kPGPError_ItemNotFound;
          if result <> 0 then exit;
        finally
          result := PGPFreeKeyIter(iter);
        end;
      finally
        result := PGPFreeKeyList(keylist);
      end;
    finally
      result := PGPFreeKeySet(fkeySet);
    end;
  finally
    result := PGPFreeFilter(filt);
  end;
  except
    on E:Exception do
    result := SPGPError_KeySearchError;
  end;
end;

function KeyIsOnRing(const context: pPGPContext; const target_KeySet: pPGPKeySet;const ID: string): Boolean;
var
  filt:       pPGPFilter;
  fKeySet:    pPGPKeySet;
  n:          PGPUInt32;
  KeyID:      TPGPKeyID;
  s:          string;
  err:        PGPError;
begin
  result := false;
  err := kPGPError_NoError;

  filt := nil;
  fKeySet := nil;
  try
  try
  s := Copy(ID,1,2);
  if s = '0x' then begin
    (* search by Key ID *)
    { convert ID string to inscrutable TPGPKeyID format }
    err := PGPGetKeyIDFromString(PChar(ID),KeyID);
    if err <> 0 then exit;
    err := PGPNewKeyIDFilter(context, KeyID, filt);
    end
  else begin
    s := ID;
    err := {PGPNewUserIDEmailFilter}PGPNewUserIDStringFilter(context, pchar(s), kPGPMatchSubString{kPGPMatchEqual}, filt);
    if err <> 0 then exit;
    end;

  try
    err := PGPFilterKeySet(target_KeySet, filt, fKeySet);
    if (err <> 0) or not Assigned(fKeySet) then exit;
    try
      err := PGPCountKeys(fKeySet, n);
      if n > 0 then result := True;
    finally
      PGPFreeKeySet(fkeySet);
    end;
  finally
    PGPFreeFilter(filt);
  end;

  finally
    if err <> 0 then result := false;
  end;
  except
    on E:Exception do
    result := false;
  end;
end;

function KeysAreOnRing(const context: pPGPContext; const target_KeySet: pPGPKeySet;const ID: string): Boolean;
var
  filt:      pPGPFilter;
  fKeySet:   pPGPKeySet;
  n:         PGPUInt32;
  KeyID:     TPGPKeyID;
  List:      TStringList;
  i:         integer;
  s:         string;
  err:       PGPError;
begin
  result := false;
  err := kPGPError_NoError;

  filt := nil;
  fKeySet := nil;
  try
  List := TStringList.Create;

  try
  { 1. convert comma-delimited string to array of strings }
  s := Copy(ID,1,2);
  if s = '0x'
  then
    ParseIDList(ID,List)//Key ID
  else
    ParseEmailList(ID,List);//User ID

  { 2. for each e-mail string in the array, check KeyIsOnRing }
  for i := 0 to List.Count -1 do begin

  s := Copy(List[i],1,2);
  if s = '0x' then begin
    (* search by Key ID *)
    { convert ID string to inscrutable TPGPKeyID format }
    err := PGPGetKeyIDFromString(PChar(List[i]),KeyID);
    if err <> 0 then exit else
    err := PGPNewKeyIDFilter(context, KeyID, filt);
    end
  else begin
    s := List[i];
    err := PGPNewUserIDStringFilter(context, pchar(s), kPGPMatchSubString{kPGPMatchEqual}, filt);
    //err := PGPNewUserIDEmailFilter(context, pchar(s), {kPGPMatchSubString}kPGPMatchEqual, filt);
    end;

  if err <> 0 then exit;

  try
    err := PGPFilterKeySet(target_KeySet, filt, fKeySet);
    if (err <> 0) or not Assigned(fKeySet) then exit;
    try
      PGPCountKeys(fKeySet, n);
      if n > 0 then result := True
      else begin
        result := false;
        exit;
        end;
    finally
      PGPFreeKeySet(fkeySet);
    end;
  finally
    PGPFreeFilter(filt);
  end;
  end;{for}

  finally
    if err <> 0 then result := false;
    List.Free;
  end;
 except
   on E:Exception do
   result := false;
 end;

end;

function KeyRecInit(var KeyRec: TKeyData): LongInt;
begin
  result := 0;
  try
	with KeyRec do begin
   KeyID     := '';
   UserID    := '';
   Fprint    := '';
   KeyBits   := '';
   KeyAlg    := 0;
   Trust     := 0;
   Validity  := 0;
   DateStr   := '';
   DateInt   := 0;
   ExpireStr := '';
   ExpireInt := 0;
   IsSecret  := PGPFalse;
   IsAxiomatic:= PGPFalse;
   IsRevoked := PGPFalse;
   IsDisabled:= PGPFalse;
   IsExpired := PGPFalse;
   end;
 except
   on E:Exception do
   result := SPGPError_KeySearchError;
 end;
end;

function getkeydata(context: pPGPContext; KeySet: pPGPKeySet; Key: pPGPKey;var KeyRec: TKeyData):PGPError;
begin
  result := kPGPError_NoError;
  try
  if (Key = nil) then begin
    result := SPGPError_KeyNotFound;
    exit;
    end;
  { initialise Key record }
  if KeyRecInit(KeyRec) <> 0 then exit;

  (* PRIVATE KEY? *)
  result := PGPGetKeyBoolean(key,kPGPKeyPropIsSecret,KeyRec.IsSecret);
  if result <> 0 then exit;

  (* KEY ID *)
  result := GetKeyPropKeyID(Key,KeyRec.KeyID);
  if result <> 0 then exit;

  (* USER ID *)
  result := GetKeyPropUserID(Key,KeyRec.UserID);
  if result <> 0 then exit;

  (* KEY SIZE *)
  result := GetKeyPropKeybits(Key,KeyRec.KeyBits);
  if result <> 0 then exit;

  (* CREATION DATE-TIME (STRING)*)
  result := GetKeyPropKeyTimeStr(Key,KeyRec.DateStr);
  if result <> 0 then exit;

  (* CREATION DATE-TIME (INTEGER)*)
  result := GetKeyPropKeyTime(Key,KeyRec.DateInt);
  if result <> 0 then exit;

  (* FINGERPRINT *)
  result := GetKeyPropFingerprint(Key,KeyRec.FPrint);
  if result <> 0 then exit;

  (* KEY TRUST *)
  result := PGPGetKeyNumber(key,kPGPKeyPropTrust,KeyRec.Trust);
  if result <> 0 then exit;

  (* KEY VALIDITY *)
  result := PGPGetKeyNumber(key,kPGPKeyPropValidity,KeyRec.Validity);
  if result <> 0 then exit;

  (* PUBLIC-KEY ALGORITHM *)
  result := GetKeyPropAlg(key,KeyRec.KeyAlg);
  if result <> 0 then exit;

  except
    on E:Exception do
    result := SPGPError_KeySearchError;
  end;
end;

function GetKeySetProps(Context: pPGPContext; KeySet: pPGPKeySet; var KeyProps: string; const Flags: Longint): LongInt;
var
  FoundKey       : pPGPKey;
  Iter           : pPGPKeyIter;
  KeyList        : pPGPKeyList;
  KeyRec         : TKeyData;
  n              : PGPUInt32;
  s              : string;
  t              : shortstring;
begin
  result := kPGPError_NoError;

  FoundKey := nil;
  Iter := nil;
  KeyList := nil;

  try
    s := '';
    result := PGPCountKeys(Keyset, n);
    if result <> 0 then exit;

    { '0' should be impossible, since GetKeysByID would've failed
      if none of the keys are on the ring, but we check anyway }
    if n = 0 then
    begin
      result := SPGPError_KeyNotFound;
      exit;
    end;

    result := PGPOrderKeySet(Keyset, kPGPUserIDOrdering, KeyList);
    if result <> 0 then exit;

  try

    result := PGPNewKeyIter(KeyList, Iter);
    if result <> 0 then exit;

    while IsntPGPError(PGPKeyIterNext(Iter, FoundKey)) and
      Assigned(FoundKey) do
    begin
      if IsntPGPError(KeyDataToStr(FoundKey, 0, 0, Flags, t))
      then s := s + t
      else s := s + #13#10;
    end;

  KeyProps := s;

  finally
    PGPFreeKeyIter(Iter);
    PGPFreeKeyList(KeyList);
    PGPFreeKeySet(Keyset);
  end;

  except
    on E:Exception do
      result := SPGPError_KeySearchError;
  end;
end;

function GetKeyPropKeyTimeStr(const Key: pPGPKey; var s: shortstring): PGPError;
var
  KeyTime: 	PGPTime;
begin
  result := kPGPError_NoError;
  if not Assigned(Key) then result := SPGPError_KeySearchError;
  if result <> 0 then exit;
  try
    result := PGPGetKeyTime(key, kPGPKeyPropCreation, KeyTime);

    if result <> 0 then exit;

    result := LocalTime(PGPGetStdTimeFromPGPTime(KeyTime),s);
  except
    on E:Exception do
    result := SPGPError_KeySearchError;
  end;
end;

function GetKeyPropKeyTime(const Key: pPGPKey; var t: PGPInt32): PGPError;
var
  KeyTime: 	PGPTime;
begin
  result := kPGPError_NoError;
  if not Assigned(Key) then result := SPGPError_KeySearchError;
  if result <> 0 then exit;
  try
    result := PGPGetKeyTime(Key, kPGPKeyPropCreation, KeyTime);

    if result <> 0 then exit;

    t := PGPGetStdTimeFromPGPTime(KeyTime);

    if t <= 0 then
    begin
      t := 0;
      result := -1;
    end;
  except
    on E:Exception do
    result := SPGPError_KeySearchError;
  end;
end;

(* the new style *)
function GetKeyPropKeyCreationTimeStr(const Key: pPGPKey; var s: shortstring): PGPError;
var
  KeyTime: 	PGPTime;
begin
  result := kPGPError_NoError;
  if not Assigned(Key) then result := SPGPError_KeySearchError;
  if result <> 0 then exit;
  try
    result := PGPGetKeyTime(key, kPGPKeyPropCreation, KeyTime);

    if result <> 0 then exit;

    result := LocalTime(PGPGetStdTimeFromPGPTime(KeyTime),s);
  except
    on E:Exception do
    result := SPGPError_KeySearchError;
  end;
end;

function GetKeyPropKeyCreationTime(const Key: pPGPKey; var t: PGPInt32): PGPError;
var
  KeyTime: 	PGPTime;
begin
  result := kPGPError_NoError;
  if not Assigned(Key) then result := SPGPError_KeySearchError;
  if result <> 0 then exit;
  try
    result := PGPGetKeyTime(Key, kPGPKeyPropCreation, KeyTime);

    if result <> 0 then exit;

    t := PGPGetStdTimeFromPGPTime(KeyTime);

  except
    on E:Exception do
    result := SPGPError_KeySearchError;
  end;
end;

function GetKeyPropKeyExpirationTimeStr(const Key: pPGPKey; var s: shortstring): PGPError;
var
  KeyTime: 	PGPTime;
begin
  result := kPGPError_NoError;
  if not Assigned(Key) then result := SPGPError_KeySearchError;
  if result <> 0 then exit;
  try
    result := PGPGetKeyTime(key, kPGPKeyPropExpiration, KeyTime);

    if result <> 0 then exit;

    if KeyTime = kPGPExpirationTime_Never then
      s := ''
    else
      result := LocalTime(PGPGetStdTimeFromPGPTime(KeyTime),s);
  except
    on E:Exception do
    result := SPGPError_KeySearchError;
  end;
end;

function GetKeyPropKeyExpirationTime(const Key: pPGPKey; var t: PGPInt32): PGPError;
var
  KeyTime: 	PGPTime;
begin
  result := kPGPError_NoError;
  if not Assigned(Key) then result := SPGPError_KeySearchError;
  if result <> 0 then exit;
  try
    result := PGPGetKeyTime(Key, kPGPKeyPropExpiration, KeyTime);

    if result <> 0 then exit;

    if KeyTime = kPGPExpirationTime_Never then
      t := 0
    else
      t := PGPGetStdTimeFromPGPTime(KeyTime);

  except
    on E:Exception do
    result := SPGPError_KeySearchError;
  end;
end;

function GetKeyPropFingerprint(const Key: pPGPKey; var s: shortstring):PGPError;
const
  HexChars       : array [0..$F] of char = '0123456789ABCDEF';
var
  b              : byte;
  FingerBytes    : array[0..255] of PGPByte;
  i              : integer;
  TempStr        : shortstring;
  UsedBytes      : PGPSize;
begin
  result := kPGPError_NoError;
  if not Assigned(Key) then result := SPGPError_KeySearchError;
  if result <> 0 then exit;
     
  try
    // extract fingerprint as bytes
    result := PGPGetKeyPropertyBuffer(key, kPGPKeyPropFingerprint,
	    256, @FingerBytes, UsedBytes);
    if result <> 0 then exit;

    // convert to hex
    TempStr := '';
    for i := 0 to UsedBytes - 1 do
    begin
      b := FingerBytes[i];
      TempStr := TempStr + (HexChars[b shr 4] + HexChars[b and $F]);
    end;

    s := TempStr;

  except
    on E:Exception do
    begin
      GetKeyPropUserID(Key, tempstr);
      showmessage(tempstr);
      result := SPGPError_KeySearchError;
    end;
  end;
end;

function GetKeyPropKeybits(const Key: pPGPKey; var s: shortstring): PGPError;
var
  KeyBits,
  SubKeyBits: PGPInt32;
  SubKey:     pPGPSubKey;
  SubKeyFingerBytes : array[0..kPGPMaxUserIDSize-1] of PGPByte;
  FullSize: PGPSize;
	KeyList:    pPGPKeyList;
  KeyIter:    pPGPKeyIter;
  KeyAlg:     PGPInt32;
  fKey:       pPGPKey;
  fKeySet:    pPGPKeySet;
begin
  result := kPGPError_NoError;
  if not Assigned(Key) then result := SPGPError_KeySearchError;
  if result <> 0 then exit;
  try
    { find the algorithm }
    result := PGPGetKeyNumber(Key, kPGPKeyPropAlgID, KeyAlg);

    if result <> 0 then exit;

	  case KeyAlg of
    { RSA has only one key, so one key-size }
	   kPGPPublicKeyAlgorithm_RSA :
    begin
      result := PGPGetKeyNumber(Key,kPGPKeyPropBits,KeyBits);
      s := IntToStr(KeyBits);
		 end;
    { DH/DSS keys have key & sub-key, XXXX/YYYY
     BUT MAYBE NOT: some 'DH/DSS' are really just DSS, e.g.
     the PGP/NAI software release key 0xD77B2094 }
	   kPGPPublicKeyAlgorithm_DSA :
    begin
      result := PGPGetKeyNumber(Key,kPGPKeyPropBits,KeyBits);
      if result <> 0 then exit;
      result := PGPNewSingletonKeySet(Key,fKeySet);
      if result <> 0 then exit;
      try
        result := PGPOrderKeySet(fKeySet,kPGPAnyOrdering,KeyList);
        if result <> 0 then exit;
        try
  	      result := PGPNewKeyIter(KeyList,KeyIter);
          if result <> 0 then exit;
          try
            fKey := Key;
            result := PGPKeyIterNext(KeyIter,fKey);
            if (result = kPGPError_EndOfIteration) then
             result := kPGPError_ItemNotFound;
            if result <> 0 then exit;
              { if there is a sub-key }
		          if IsntPGPError(PGPKeyIterNextSubKey(KeyIter,SubKey))
              then
		            result := PGPGetSubKeyNumber(SubKey,kPGPKeyPropBits,SubKeyBits)
              else
                SubKeyBits := 0;
            if result <> 0 then exit;
            if SubKeyBits >0
            then s := IntToStr(SubKeyBits) + '/' + IntToStr(KeyBits)
            else s := IntToStr(KeyBits);
		       finally
		         PGPFreeKeyIter(KeyIter);
          end;
        finally
          PGPFreeKeyList(KeyList);
        end;
      finally
        PGPFreeKeySet(fKeySet);
      end;
    end;{begin}
    end;{case}
  except
    on E:Exception do
    result := SPGPError_KeySearchError;
  end;
end;

function GetKeyPropUserID(const Key: pPGPKey; var s: shortstring): PGPError;
var
  p              : pointer;
  usedsize       : PGPSize;
begin
  result := kPGPError_NoError;
  p := nil;

  if not Assigned(Key) then result := SPGPError_KeySearchError;
  if result <> 0 then exit;

  try
    GetMem(p, kPGPMaxUserIDSize);
    try
      result := PGPGetPrimaryUserIDNameBuffer(key, kPGPMaxUserIDSize-1, p, usedsize);
      if result <> 0 then exit;
      s := StrPas(p);
    finally
      FreeMem(p, kPGPMaxUserIDSize);
    end;

  except
    on E:Exception do
    result := SPGPError_KeySearchError;
  end;
end;

function GetKeyPropKeyID(const Key: pPGPKey; var s: shortstring): PGPError;
var
  keyid_as_weird: TPGPKeyID;
  keyid_as_str:   PGPChar;
begin
  result := kPGPError_NoError;
  if not Assigned(Key) then result := SPGPError_KeySearchError;
  if result <> 0 then exit;
  try
  { first get the KeyID data ... }
  result := PGPGetKeyIDFromKey(key, keyid_as_weird);
  if result <> 0 then exit;

  { ... now extract the ID string }
  result :=
  PGPGetKeyIDString(keyid_as_weird,
                    kPGPKeyIDString_Abbreviated,
                    keyid_as_str);
  if result <> 0 then exit;

  s := StrPas(keyid_as_str);
  except
    on E:Exception do
    result := SPGPError_KeySearchError;
  end;
end;

function GetKeyPropAlg(const Key: pPGPKey; var t: integer): PGPError;
var
  KeyAlg: PGPInt32;
begin
  result := kPGPError_NoError;
  if not Assigned(Key) then result := SPGPError_KeySearchError;
  if result <> 0 then exit;
  try
    result := PGPGetKeyNumber(Key,kPGPKeyPropAlgID,KeyAlg);
    if result <> 0 then exit;

    case KeyAlg of
      kPGPPublicKeyAlgorithm_Invalid         : t := 0;
	    kPGPPublicKeyAlgorithm_RSA             : t := 1;
      kPGPPublicKeyAlgorithm_RSAEncryptOnly  : t := 2;
      kPGPPublicKeyAlgorithm_RSASignOnly     : t := 3;
      kPGPPublicKeyAlgorithm_ElGamal         : t := 4; // A.K.A.Diffie-Hellman
      kPGPPublicKeyAlgorithm_DSA             : t := 5;
    else
      t := 0;
    end;
  except
    on E:Exception do
    result := SPGPError_KeySearchError;
  end;
end;

function GetKeyPropTrust(const Key: pPGPKey; var t: integer): PGPError;
var
  KeyTrust: PGPInt32;
begin
  result := kPGPError_NoError;
  if not Assigned(Key) then result := SPGPError_KeySearchError;
  if result <> 0 then exit;
  try
    result := PGPGetKeyNumber(Key, kPGPKeyPropTrust, KeyTrust);
    if result <> 0 then exit;
    t := KeyTrust;
  except
    on E:Exception do
    result := SPGPError_KeySearchError;
  end;
end;

function GetKeyPropValidity(const Key: pPGPKey; var t: integer): PGPError;
var
  KeyValidity: PGPInt32;
begin
  result := kPGPError_NoError;
  if not Assigned(Key) then result := SPGPError_KeySearchError;
  if result <> 0 then exit;
  try
    result := PGPGetKeyNumber(Key, kPGPKeyPropValidity, KeyValidity);
    if result <> 0 then exit;
    t := KeyValidity;
  except
    on E:Exception do
    result := SPGPError_KeySearchError;
  end;
end;

function GetKeyPropARR(const Key: pPGPKey; var count: integer; var s: shortstring): PGPError;
var
  ARClass        : PGPByte;
  ARKey          : pPGPKey;
  ARKeyCount     : PGPUint32;
  ARKeyID        : TPGPKeyID;
  DataSize       : PGPSize;
  Filter         : pPGPFilter;
  fKey           : pPGPKey;
  fKeySet        : pPGPKeySet;
  i              : integer;
  Iter           : pPGPKeyIter;
  KeyAlg         : PGPInt32;
  KeyCount       : PGPUInt32;
  KeyIDStr       : PGPChar;
  KeyList        : pPGPKeyList;
  pData          : pointer;
begin
  ARKey := nil;
  Filter := nil;
  fKey := nil;
  fKeySet := nil;
  Iter := nil;
  KeyList := nil;
  pData := nil;

  count := 0;
  s := '';

try
  GetMem(pData, 256);

  try
    // create a new keyset containing the target key
    result := PGPNewSingletonKeySet(Key, fKeySet);

    // move keyset into a keylist
    result := PGPOrderKeySet(fKeySet, kPGPAnyOrdering, KeyList);

    // create a new iter
    result := PGPNewKeyIter(KeyList, Iter);

    // rewind to make sure we're at the start
    result := PGPKeyIterRewind(Iter);

    // iterate through the keylist until
    // kPGPError_EndOfIteration
    fKey := Key;
    while IsntPGPError(PGPKeyIterNext(Iter, fKey)) do
    begin
      // check for algorithm
      result := PGPGetKeyNumber(Key, kPGPKeyPropAlgID, KeyAlg);
      if result <> 0 then exit;
      
      // only DSA keys have ARR
      if KeyAlg <> kPGPPublicKeyAlgorithm_DSA then
      begin
        result := kPGPError_ItemNotFound;
        exit;
      end;

      // look for ARR
      PGPCountAdditionalRecipientRequests(fKey, ARKeyCount);
      if result <> 0 then exit;

      if ARKeyCount > 0 then
      for i := 0 to ARKeyCount -1 do
      begin
        if PGPversion5 then //kPGPsdkApiVersion < $03000000 then
        begin
        // PGP55
          result := PGPGetIndexedAdditionalRecipientRequestKeyID(
            fKey, i, ARKeyID, ARClass);
          if result <> 0 then exit;
        end
        else
        begin
          // PGP60+
          // we only need the KeyID, so fKeySet will do;
          // otherwise we'd need the master keyset
          result := PGPGetIndexedAdditionalRecipientRequestKey(
            fKey, fKeySet, i, ARKey, ARKeyID, ARClass);
          if result <> 0 then exit;
        end;
        // get string from Key ID
        PGPGetKeyIDString(ARKeyID,
          kPGPKeyIDString_Abbreviated, KeyIDStr);

        if i > 0 then
          s := s + ',' + StrPas(KeyIDstr)
        else
          s := StrPas(KeyIDstr);
      end;
      count := ARKeyCount;
    end;

  finally
    FreeMem(pData, 256);
    PGPFreeKeyIter(Iter);
    PGPFreeKeyList(KeyList);
    PGPFreeKeySet(fKeySet);
  end;
except
  on E:Exception do
    result := SPGPError_KeySearchError;
end;
end;

function GetKeyPropRevokers(const Key: pPGPKey; var count: integer; var s: shortstring): PGPError;
var
  DataSize       : PGPSize;
  Filter         : pPGPFilter;
  fKey           : pPGPKey;
  fKeySet        : pPGPKeySet;
  i              : PGPUint32;
  Iter           : pPGPKeyIter;
  KeyAlg         : PGPInt32;
  KeyCount       : PGPUInt32;
  KeyIDStr       : PGPChar;
  KeyList        : pPGPKeyList;
  pData          : pointer;
  RevKey         : pPGPKey;
  RevKeyCount    : PGPUint32;
  RevKeyID       : TPGPKeyID;
begin
  RevKey := nil;
  Filter := nil;
  fKey := nil;
  fKeySet := nil;
  Iter := nil;
  KeyList := nil;
  pData := nil;

  count := 0;
  s := '';

try
  GetMem(pData, 256);

  try
    // no revocation stuff before PGP 6
    if PGPversion5 then //kPGPsdkApiVersion < $03000000 then
    begin
      result := kPGPError_IncompatibleAPI;
      exit;
    end;

    // create a new keyset containing the target key
    result := PGPNewSingletonKeySet(Key, fKeySet);
    if result <> 0 then exit;

    // move keyset into a keylist
    result := PGPOrderKeySet(fKeySet, kPGPAnyOrdering, KeyList);
    if result <> 0 then exit;

    // create a new iter
    result := PGPNewKeyIter(KeyList, Iter);
    if result <> 0 then exit;

    // rewind to make sure we're at the start
    result := PGPKeyIterRewind(Iter);
    if result <> 0 then exit;

    // iterate through the keylist until
    // kPGPError_EndOfIteration
    fKey := Key;
    while IsntPGPError(PGPKeyIterNext(Iter, fKey)) do
    begin
      // check for algorithm
      result := PGPGetKeyNumber(Key, kPGPKeyPropAlgID, KeyAlg);
      if result <> 0 then exit;

      // only DSA keys have Revocation Keys
      if KeyAlg <> kPGPPublicKeyAlgorithm_DSA then exit;

      // look for Revokers
      PGPCountRevocationKeys(fKey, RevKeyCount);
      if result <> 0 then exit;

      if RevKeyCount > 0 then
      for i := 0 to RevKeyCount -1 do
      begin
        // PGP60+
        // we only need the KeyID, so fKeySet will do;
        // otherwise we'd need the master keyset
        result := PGPGetIndexedRevocationKey(
          fKey, fKeySet, i, RevKey, RevKeyID);

        if result <> 0 then exit;

        // get string from Key ID
        PGPGetKeyIDString(RevKeyID,
          kPGPKeyIDString_Abbreviated, KeyIDStr);

        if i > 0 then
          s := s + ',' + StrPas(KeyIDstr)
        else
          s := StrPas(KeyIDstr);
      end;
      count := RevKeyCount;
    end;

  finally
    FreeMem(pData, 256);
    PGPFreeKeyIter(Iter);
    PGPFreeKeyList(KeyList);
    PGPFreeKeySet(fKeySet);
  end;
except
  on E:Exception do
    result := SPGPError_KeySearchError;
end;
end;

function GetKeyPropIsSecret(const Key: pPGPKey; var t: integer): PGPError;
var
  Prop           : PGPBoolean;
begin
  result := kPGPError_NoError;
  t := 0;

  if not Assigned(Key) then result := SPGPError_KeySearchError;
  if result <> 0 then exit;
  try
    result := PGPGetKeyBoolean(Key, kPGPKeyPropIsSecret, Prop);
    if result <> 0 then exit;

    t := Prop;
  except
    on E:Exception do
      result := SPGPError_KeySearchError;
  end;
end;

function GetKeyPropIsAxiomatic(const Key: pPGPKey; var t: integer): PGPError;
var
  Prop           : PGPBoolean;
begin
  result := kPGPError_NoError;
  t := 0;

  if not Assigned(Key) then result := SPGPError_KeySearchError;
  if result <> 0 then exit;
  try
    result := PGPGetKeyBoolean(Key, kPGPKeyPropIsAxiomatic, Prop);
    if result <> 0 then exit;

    t := Prop;
  except
    on E:Exception do
      result := SPGPError_KeySearchError;
  end;
end;

function GetKeyPropIsRevoked(const Key: pPGPKey; var t: integer): PGPError;
var
  Prop           : PGPBoolean;
begin
  result := kPGPError_NoError;
  t := 0;

  if not Assigned(Key) then result := SPGPError_KeySearchError;
  if result <> 0 then exit;
  try
    result := PGPGetKeyBoolean(Key, kPGPKeyPropIsRevoked, Prop);
    if result <> 0 then exit;

    t := Prop;
  except
    on E:Exception do
      result := SPGPError_KeySearchError;
  end;
end;

function GetKeyPropIsDisabled(const Key: pPGPKey; var t: integer): PGPError;
var
  Prop           : PGPBoolean;
begin
  result := kPGPError_NoError;
  t := 0;

  if not Assigned(Key) then result := SPGPError_KeySearchError;
  if result <> 0 then exit;
  try
    result := PGPGetKeyBoolean(Key, kPGPKeyPropIsDisabled, Prop);
    if result <> 0 then exit;

    t := Prop;
  except
    on E:Exception do
      result := SPGPError_KeySearchError;
  end;
end;

function GetKeyPropIsExpired(const Key: pPGPKey; var t: integer): PGPError;
var
  Prop           : PGPBoolean;
begin
  result := kPGPError_NoError;
  t := 0;

  if not Assigned(Key) then result := SPGPError_KeySearchError;
  if result <> 0 then exit;
  try
    result := PGPGetKeyBoolean(Key, kPGPKeyPropIsExpired, Prop);
    if result <> 0 then exit;

    t := Prop;
  except
    on E:Exception do
      result := SPGPError_KeySearchError;
  end;
end;

function GetKeyPropCanEncrypt(const Key: pPGPKey; var t: integer): PGPError;
var
  Prop           : PGPBoolean;
begin
  result := kPGPError_NoError;
  t := 0;

  if not Assigned(Key) then result := SPGPError_KeySearchError;
  if result <> 0 then exit;
  try
    result := PGPGetKeyBoolean(Key, kPGPKeyPropCanEncrypt, Prop);
    if result <> 0 then exit;

    t := Prop;
  except
    on E:Exception do
      result := SPGPError_KeySearchError;
  end;
end;

function GetKeyPropCanDecrypt(const Key: pPGPKey; var t: integer): PGPError;
var
  Prop           : PGPBoolean;
begin
  result := kPGPError_NoError;
  t := 0;

  if PGPversion5 then //kPGPsdkApiVersion < $03000000 then
  begin
    result := kPGPError_IncompatibleAPI;
    exit;
  end;

  if not Assigned(Key) then result := SPGPError_KeySearchError;
  if result <> 0 then exit;
  try
    result := PGPGetKeyBoolean(Key, kPGPKeyPropCanDecrypt, Prop);
    if result <> 0 then exit;

    t := Prop;
  except
    on E:Exception do
      result := SPGPError_KeySearchError;
  end;
end;

function GetKeyPropCanSign(const Key: pPGPKey; var t: integer): PGPError;
var
  Prop           : PGPBoolean;
begin
  result := kPGPError_NoError;
  t := 0;

  if not Assigned(Key) then result := SPGPError_KeySearchError;
  if result <> 0 then exit;
  try
    result := PGPGetKeyBoolean(Key, kPGPKeyPropCanSign, Prop);
    if result <> 0 then exit;

    t := Prop;
  except
    on E:Exception do
      result := SPGPError_KeySearchError;
  end;
end;

function GetKeyPropCanVerify(const Key: pPGPKey; var t: integer): PGPError;
var
  Prop           : PGPBoolean;
begin
  result := kPGPError_NoError;
  t := 0;

  if PGPversion5 then //kPGPsdkApiVersion < $03000000 then
  begin
    result := kPGPError_IncompatibleAPI;
    exit;
  end;

  if not Assigned(Key) then result := SPGPError_KeySearchError;
  if result <> 0 then exit;
  try
    result := PGPGetKeyBoolean(Key, kPGPKeyPropCanVerify, Prop);
    if result <> 0 then exit;

    t := Prop;
  except
    on E:Exception do
      result := SPGPError_KeySearchError;
  end;
end;

function GetKeyPropIsSecretShared(const Key: pPGPKey; var t: integer): PGPError;
var
  Prop           : PGPBoolean;
begin
  result := kPGPError_NoError;
  t := 0;

  if PGPversion5 then //kPGPsdkApiVersion < $03000000 then
  begin
    result := kPGPError_IncompatibleAPI;
    exit;
  end;

  if not Assigned(Key) then result := SPGPError_KeySearchError;
  if result <> 0 then exit;
  try
    result := PGPGetKeyBoolean(Key, kPGPKeyPropIsSecretShared, Prop);
    if result <> 0 then exit;

    t := Prop;
  except
    on E:Exception do
      result := SPGPError_KeySearchError;
  end;
end;

function GetKeyPropHasRevoker(const Key: pPGPKey; var t: integer): PGPError;
var
  RevKeyCount    : PGPUint32;
begin
  result := kPGPError_NoError;
  t := 0;

  if PGPversion5 then //kPGPsdkApiVersion < $03000000 then
  begin
    result := kPGPError_IncompatibleAPI;
    exit;
  end;

  if not Assigned(Key) then result := SPGPError_KeySearchError;
  if result <> 0 then exit;
  try
    result := PGPCountRevocationKeys(Key, RevKeyCount);
    if result <> 0 then exit;

    if RevKeyCount > 0 then t := 1;
  except
    on E:Exception do
      result := SPGPError_KeySearchError;
  end;
end;

function GetKeyPropHasARR(const Key: pPGPKey; var t: integer): PGPError;
var
  ARKeyCount    : PGPUint32;
begin
  result := kPGPError_NoError;
  t := 0;

  if not Assigned(Key) then result := SPGPError_KeySearchError;
  if result <> 0 then exit;
  try
    result := PGPCountAdditionalRecipientRequests(Key, ARKeyCount);
    if result <> 0 then exit;

    if ARKeyCount > 0 then t := 1;
  except
    on E:Exception do
      result := SPGPError_KeySearchError;
  end;
end;

(* SUB-KEY PROPERTIES *)

function GetSubKeyPropKeyID(const SubKey: pPGPSubKey; var s: shortstring): PGPError;
var
  KeyID          : TPGPKeyID;
  KeyIDstr       : PGPChar;
begin
  result := kPGPError_NoError;
  if not Assigned(SubKey) then result := SPGPError_KeySearchError;
  if result <> 0 then exit;
  try
    { first get the KeyID data ... }
    result := PGPGetKeyIDFromSubKey(SubKey, KeyID);
    if result <> 0 then exit;

    { ... now extract the ID string }
    result :=
    PGPGetKeyIDString(KeyID,
                      kPGPKeyIDString_Abbreviated,// first 8 chars
                      KeyIDstr);
    if result <> 0 then exit;

    s := StrPas(KeyIDstr);
  except
    on E:Exception do
      result := SPGPError_KeySearchError;
  end;
end;

function GetSubKeyPropKeybits(const SubKey: pPGPSubKey; var s: shortstring): PGPError;
var
  SubKeyBits     : PGPInt32;
begin
  result := kPGPError_NoError;
  s := '0';

  if not Assigned(SubKey) then result := SPGPError_KeySearchError;
  if result <> 0 then exit;

  try
    result := PGPGetSubKeyNumber(SubKey, kPGPKeyPropBits, SubKeyBits);
    if result <> 0 then exit;
    s := IntToStr(SubKeyBits);

  except
    on E:Exception do
      result := SPGPError_KeySearchError;
  end;
end;

function GetSubKeyPropFingerprint(const SubKey: pPGPSubKey; var s: shortstring):PGPError;
const
  HexChars       : array [0..$F] of char = '0123456789ABCDEF';
var
  b              : byte;
  FingerBytes    : array[0..255] of PGPByte;
  i              : integer;
  TempStr        : shortstring;
  UsedBytes      : PGPSize;
begin
  result := kPGPError_NoError;
  if not Assigned(SubKey) then result := SPGPError_KeySearchError;
  if result <> 0 then exit;

  try
    result := PGPGetSubKeyPropertyBuffer(SubKey, kPGPKeyPropFingerprint,
	    256, @FingerBytes, UsedBytes);
    if result <> 0 then exit;

    TempStr := '';

    for i := 0 to UsedBytes - 1 do
    begin
      b := FingerBytes[i];
      TempStr := TempStr + (HexChars[b shr 4] + HexChars[b and $F]);
    end;

    s := TempStr;

  except
    on E:Exception do
      result := SPGPError_KeySearchError;
  end;
end;

(* UTILITIES *)

(* takes a key record, puts properties into s, up to count; optionally adds CRLF  *)
function KeyRecToStr(const KeyRec: TKeyData;var s: shortstring;count: byte;crlf: boolean): PGPError;
begin
  result := kPGPError_NoError;

  if not (count > 0) then
    result := SPGPError_KeySearchError;

  if result <> 0 then exit;

  try
    s := '';

    if (KeyRec.IsSecret = PGPTrue)
    then s := s + '1' + #9
    else s := s + '0' + #9;
    if count >= 2 then
    s := s + KeyRec.KeyID;
    if count >= 3 then
    s := s + #9 + KeyRec.UserID;
    if count >= 4 then
    s := s + #9 + KeyRec.KeyBits;
    if count >= 5 then
    s := s + #9 + KeyRec.DateStr;
    if count >= 6 then
    s := s + #9 + IntToStr(KeyRec.DateInt);
    if count >= 7 then
    s := s + #9 + KeyRec.FPrint;
    if count >= 8 then
    s := s + #9 + IntToStr(KeyRec.Trust);
    if count >= 9 then
    s := s + #9 + IntToStr(KeyRec.Validity);
    if count >= 10 then
    s := s + #9 + IntToStr(KeyRec.KeyAlg);

    if crlf then s := s + #13#10;

  except
    on E:Exception do
    begin
      result := SPGPError_KeySearchError;
      s := '';
    end;
  end;
end;

(*
  yesflags are properties which should be set to yes
  noflags are properties which should be set to no
  flags are properties to query
*)
function KeyDataToStr(const Key: pPGPKey; const yesflags: Longint; const noflags: Longint; const flags: Longint; var s: shortstring): PGPError;
var
  t              : shortstring;
  x              : integer;
begin
  result := kPGPError_NoError;
  s := '';

  try
     // string properties
    if flags and spgpKeyPropFlag_KeyID = spgpKeyPropFlag_KeyID then
    begin
      GetKeyPropKeyID(Key, t);
      s := s + #9 + t;
    end;

    if flags and spgpKeyPropFlag_UserID = spgpKeyPropFlag_UserID then
    begin
      GetKeyPropUserID(Key, t);
      s := s + #9 + t;
    end;

    if flags and spgpKeyPropFlag_Fingerprint = spgpKeyPropFlag_Fingerprint then
    begin
      GetKeyPropFingerprint(Key, t);
      s := s + #9 + t;
    end;

    if flags and spgpKeyPropFlag_CreationTimeStr = spgpKeyPropFlag_CreationTimeStr then
    begin
      GetKeyPropKeyCreationTimeStr(Key, t);
      s := s + #9 + t;
    end;

    if flags and spgpKeyPropFlag_ExpirationTimeStr = spgpKeyPropFlag_ExpirationTimeStr then
    begin
      GetKeyPropKeyExpirationTimeStr(Key, t);
      s := s + #9 + t;
    end;

    // number properties
    if flags and spgpKeyPropFlag_Keybits = spgpKeyPropFlag_Keybits then
    begin
      GetKeyPropKeybits(Key, t);
      s := s + #9 + t;
    end;

    if flags and spgpKeyPropFlag_KeyAlg = spgpKeyPropFlag_KeyAlg then
    begin
      GetKeyPropAlg(Key, x);
      s := s + #9 + IntToStr(x);
    end;

    if flags and spgpKeyPropFlag_Trust = spgpKeyPropFlag_Trust then
    begin
      GetKeyPropTrust(Key, x);
      s := s + #9 + IntToStr(x);
    end;

    if flags and spgpKeyPropFlag_Validity = spgpKeyPropFlag_Validity then
    begin
      GetKeyPropValidity(Key, x);
      s := s + #9 + IntToStr(x);
    end;

    if flags and spgpKeyPropFlag_CreationTime = spgpKeyPropFlag_CreationTime then
    begin
      GetKeyPropKeyCreationTime(Key, x);
      s := s + #9 + IntToStr(x);
    end;

    if flags and spgpKeyPropFlag_ExpirationTime = spgpKeyPropFlag_ExpirationTime then
    begin
      GetKeyPropKeyExpirationTime(Key, x);
      s := s + #9 + IntToStr(x);
    end;

    // boolean properties
    if flags and spgpKeyPropFlag_IsSecret = spgpKeyPropFlag_IsSecret then
    begin
      GetKeyPropIsSecret(Key, x);
      s := s + #9 + IntToStr(x);
    end;

    if flags and spgpKeyPropFlag_IsAxiomatic = spgpKeyPropFlag_IsAxiomatic then
    begin
      GetKeyPropIsAxiomatic(Key, x);
      s := s + #9 + IntToStr(x);
    end;

    if flags and spgpKeyPropFlag_IsRevoked = spgpKeyPropFlag_IsRevoked then
    begin
      GetKeyPropIsRevoked(Key, x);
      s := s + #9 + IntToStr(x);
    end;

    if flags and spgpKeyPropFlag_IsDisabled = spgpKeyPropFlag_IsDisabled then
    begin
      GetKeyPropIsDisabled(Key, x);
      s := s + #9 + IntToStr(x);
    end;

    if flags and spgpKeyPropFlag_IsExpired = spgpKeyPropFlag_IsExpired then
    begin
      GetKeyPropIsExpired(Key, x);
      s := s + #9 + IntToStr(x);
    end;

    if flags and spgpKeyPropFlag_IsSecretShared = spgpKeyPropFlag_IsSecretShared then
    begin
      GetKeyPropIsSecretShared(Key, x);
      s := s + #9 + IntToStr(x);
    end;

    if flags and spgpKeyPropFlag_CanEncrypt = spgpKeyPropFlag_CanEncrypt then
    begin
      GetKeyPropCanEncrypt(Key, x);
      s := s + #9 + IntToStr(x);
    end;

    if flags and spgpKeyPropFlag_CanDecrypt = spgpKeyPropFlag_CanDecrypt then
    begin
      GetKeyPropCanDecrypt(Key, x);
      s := s + #9 + IntToStr(x);
    end;

    if flags and spgpKeyPropFlag_CanSign = spgpKeyPropFlag_CanSign then
    begin
      GetKeyPropCanSign(Key, x);
      s := s + #9 + IntToStr(x);
    end;

    if flags and spgpKeyPropFlag_CanVerify = spgpKeyPropFlag_CanVerify then
    begin
      GetKeyPropCanVerify(Key, x);
      s := s + #9 + IntToStr(x);
    end;

    if flags and spgpKeyPropFlag_HasRevoker = spgpKeyPropFlag_HasRevoker then
    begin
      GetKeyPropHasRevoker(Key, x);
      s := s + #9 + IntToStr(x);
    end;

    if flags and spgpKeyPropFlag_HasADK = spgpKeyPropFlag_HasADK then
    begin
      if YesFlags and spgpKeyPropFlag_HasADK = spgpKeyPropFlag_HasADK then
        x := 1
      else
      if NoFlags and spgpKeyPropFlag_HasADK = spgpKeyPropFlag_HasADK then
        x := 0
      else
      GetKeyPropHasARR(Key, x);
      s := s + #9 + IntToStr(x);
    end;

    if flags and spgpKeyPropFlag_HasSubKey = spgpKeyPropFlag_HasSubKey then
    begin
      GetKeyPropKeyBits(Key, t);
      if Pos('/', t) <> 0 then
        x := 1
      else
        x := 0;
      s := s + #9 + IntToStr(x);
    end;

    // remove leading tab, add CRLF
    while s[1] = #9 do
      s := Copy(s, 2, Length(s));
    s := s + #13#10;

  except
    on E:Exception do
    begin
      result := SPGPError_KeySearchError;
      s := '';
    end;
  end;
end;

function GetKeySetPropsOldStyle(Context: pPGPContext; KeySet: pPGPKeySet; var KeyProps: string; AllProps: boolean): LongInt;
var
  FoundKey       : pPGPKey;
  Iter           : pPGPKeyIter;
  KeyFilter      : pPGPFilter;
  KeyList        : pPGPKeyList;
  KeyRec         : TKeyData;
  n              : PGPUInt32;
  s              : string;
  t              : shortstring;
begin
  result := kPGPError_NoError;

  FoundKey := nil;
  Iter := nil;
  KeyFilter := nil;
  KeyList := nil;

  try

  result := PGPCountKeys(Keyset, n);
  if result <> 0 then exit else
  if n = 0 then exit
  else
    result := PGPOrderKeySet(Keyset, kPGPAnyOrdering, keylist);
    if result <> 0 then exit;

  try
    result := PGPNewKeyIter(keylist, iter);
    if result <> 0 then exit;

    while IsntPGPError(PGPKeyIterNext(iter, FoundKey)) and
      Assigned(FoundKey) do
    begin
      result := GetKeyData(Context,keyset,FoundKey,KeyRec);
      //PGPCountAdditionalRecipientRequests(Key,n);
      if IsntPGPError(result)then
      begin
        if not AllProps then
          KeyRecToStr(KeyRec,t,3,true)
        else
          KeyRecToStr(KeyRec,t,10,true);
        s := s + t;
      end
      else
        s := s + '' + #13#10;
      end;

    KeyProps := s;
  finally
    PGPFreeKeyIter(Iter);
    PGPFreeKeyList(KeyList);
    PGPFreeKeySet(Keyset);
  end;

  except
    on E:Exception do
      result := SPGPError_KeySearchError;
  end;
end;

initialization
  PGPversion5 := IsPGP5;

end.

