{************************************************************************}
{ spgpMisc - weirdness & miscellanea                                     }
{                                                                        }
{ This unit requires one or more units from the Pascal translations of   }
{ the PGPsdk header files: <http://www.oz.net/~srheller/pgpsdk/>         }
{ 								                                                       }
{ The original Pascal code is: spgpMisc.pas, released 23 Jun 2000.       }
{ The initial developer of the Pascal code is Steven R. Heller           }
{ <srheller@oz.net>                                                      }
{ 								                                                       }
{ Portions created by Steven R. Heller are    				                   }
{ Copyright (C) 2000 Steven R. Heller                                    }
{									                                                       }
{ The contents of this file are used with permission, subject to         }
{ the Mozilla Public License Version 1.1 (the "License"); you may        }
{ not use this file except in compliance with the License. You may       }
{ obtain a copy of the License at                                        }
{ http://www.mozilla.org/MPL/MPL-1.1.html 	                             }
{ <http://www.mozilla.org/MPL/MPL-1.1.html> 	                           }
{ 									                                                     }
{ Software distributed under the License is distributed on an 	         }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or         }
{ implied. See the License for the specific language governing           }
{ rights and limitations under the License.                              }
{                                                                        }
{************************************************************************}

unit spgpMisc;

interface

uses
  // Delphi
  SysUtils, Windows,
  // PGPsdk
  pgpBase, pgpPubTypes, pgpErrors, pgpUtilities, pgpClient,
    pgpUserInterface,
  // SPGP
  spgpGlobals,
  // always last
  PGPsdk;

  function spgpestimatepassphrasequality(passphrase: PChar): Longint; stdcall;export;
  function spgpgeterrorstring(theError: Longint; BufferOut: PChar):longint;stdcall;export;
  function spgppgppath(path: PChar): LongInt;stdcall;export;
  function spgpwraptext(Text, WrappedText: pChar; WrappedTextLen, Column: Longint): Longint;stdcall;export;

implementation

var
  LibsAllLoaded  : boolean;
  PGPversion5    : boolean;

// return semi-lucid error-strings based on PGPError codes
function spgpgeterrorstring(theError: Longint; BufferOut: PChar):longint;stdcall;export;
begin
  result := PGPsdkInit;
  if result <> 0 then exit;
  try
    result := PGPGetErrorString(theError, 255, BufferOut);
    if result <> 0 then BufferOut := '';
  finally
    PGPsdkCleanup;
  end;
end;

(* get the path to the installed PGP *)
function spgppgppath(path: PChar): LongInt;stdcall;export;
var
  Context        : pPGPContext;
  Loaded         : boolean;
  TempPath       : array[0..MAX_PATH - 1] of char;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;

  Loaded := false;
  result := PGPsdkInit;
  if result <> 0 then exit;
  Loaded := true;

  result := PGPNewContext(kPGPsdkApiVersion, Context);
  if result <> 0 then exit;

  try
    if PGPversion5 then
      result := PGPcomdlgGetPGPPath(TempPath, MAX_PATH)
    else
    begin
      PGPclInitLibrary(Context);
      result := PGPclGetPGPPath(TempPath, MAX_PATH);
    end;
    if result <> 0 then exit;

    StrLCopy(Path, TempPath, MAX_PATH);

  finally
    if Assigned(PGPcomdlgCloseLibrary) then
      PGPcomdlgCloseLibrary;
    if Assigned(PGPclCloseLibrary) then
      PGPclCloseLibrary;
    PGPFreeContext(Context);
    if Loaded then PGPsdkCleanup;
  end;
end;

function spgpwraptext(Text, WrappedText: pChar; WrappedTextLen, Column: Longint): Longint;stdcall;export;
var
  Context        : pPGPContext;
  KeySet         : pPGPKeySet;
  OutText        : pChar;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeySet := nil;
  OutText := nil;

  result := PGPsdkInit;
  if result <> 0 then exit;

  try
    result := PGPNewContext(kPGPsdkAPIVersion, Context);
    if result <> 0 then exit;
    //result := PGPsdkLoadDefaultPrefs(Context);

    // open keys for read/write
    //result := PGPOpenDefaultKeyRings(Context, kPGPKeyRingOpenFlags_Mutable, KeySet));

    OutText := StrAlloc(WrappedTextLen);

    if PGPversion5 then
      result := PGPcomdlgWrapBuffer(Text, Column, OutText)
    else
    begin
      PGPclInitLibrary(Context);
      result := PGPclWrapBuffer(Text, Column, OutText);
    end;
    if result <> 0 then exit;

    { error code if output is larger than caller's buffer }
    { is that possible in this case, since OutText was created
      with WrappedTextLen as a maximum size? }
    if StrLen(OutText) > WrappedTextLen - 1
    then result := StrLen(OutText) + 1
    else StrLCopy(WrappedText, OutText, WrappedTextLen);

  finally
    if PGPversion5 then
    begin
      PGPcomdlgFreeWrapBuffer(OutText);
      PGPcomdlgCloseLibrary;
    end
    else
    begin
      PGPclFreeWrapBuffer(OutText);
      PGPclCloseLibrary;
    end;
    PGPFreeContext(Context);
    PGPsdkCleanup;
  end;
end;

function spgpestimatepassphrasequality(passphrase: PChar): Longint; stdcall;export;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;
  if result <> 0 then exit;

  result := PGPsdkInit;
  if result <> 0 then exit;

  try
    // version check
    if PGPversion5 then
      result := PGPcomdlgPassphraseQuality(passphrase)
    else
      result := PGPEstimatePassphraseQuality(passphrase);
  finally
    PGPsdkCleanup;
  end;
end;

initialization
  // check all pgp units used by this unit
  LibsAllLoaded := pgpUtilitiesLoaded and
                   pgpErrorsLoaded and
                   pgpClientLoaded;

  PGPversion5 := IsPGP5;
end.
