{************************************************************************
{ spgpPreferences - get/set PGPsdk preferences
{
{ This unit requires one or more units from the Pascal translations of
{ the PGPsdk header files: <http://www.oz.net/~srheller/pgpsdk/>
{
{ The original Pascal code is: spgpPreferences.pas, released 10 Apr 2000.
{ The initial developer of the Pascal code is Steven R. Heller
{ <srheller@oz.net>
{
{ Portions created by Steven R. Heller are
{ Copyright (C) 2000 Steven R. Heller
{
{ The contents of this file are used with permission, subject to
{ the Mozilla Public License Version 1.1 (the "License"); you may
{ not use this file except in compliance with the License. You may
{ obtain a copy of the License at
{ http://www.mozilla.org/MPL/MPL-1.1.html
{ <http://www.mozilla.org/MPL/MPL-1.1.html>
{
{ Software distributed under the License is distributed on an
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
{ implied. See the License for the specific language governing
{ rights and limitations under the License.
{
{************************************************************************}

unit spgpPreferences;

interface

uses
  // Delphi
  SysUtils, Windows,
  // PGPsdk
  pgpBase, pgpPubTypes, pgpOptionList, pgpSDKPrefs, pgpErrors,
    pgpUtilities, pgpKeys, pgpFeatures, pgpClient,
  // SPGP
  spgpGlobals, spgpKeyUtil, spgpUtil,
  // always last
  PGPsdk;

type
  pPreferenceRec = ^TPreferenceRec;
  TPreferenceRec = record
    PublicKeyring  : array[0..MAX_PATH - 1] of char;
    PrivateKeyring : array[0..MAX_PATH - 1] of char;
    RandomSeedFile : array[0..MAX_PATH - 1] of char;
    GroupsFile     : array[0..MAX_PATH - 1] of char;
    DefaultKeyID   : array[0..9] of char;
  end;

  function spgpsetpreference(Preference: LongInt; Value: PChar):LongInt;stdcall;export;
  function spgpgetpreference(Preference: LongInt; Value: PChar):LongInt;stdcall;export;

  function spgppreferencesdialog(Flags, WindowHandle: Longint): Longint; stdcall; export;

  function spgpgetpreferences(Prefs: pPreferenceRec; Flags: Longint):LongInt;stdcall;export;
  function spgpsetpreferences(Prefs: pPreferenceRec; Flags: Longint):LongInt;stdcall;export;

implementation

var
  LibsAllLoaded  : boolean;
  PGPversion5    : boolean;

function spgpgetpreferences(Prefs: pPreferenceRec; Flags: Longint):LongInt;stdcall;export;
var
  Context        : pPGPContext;
  DataOut        : Pointer;
  FileSpec       : pPGPFileSpec;
  KeyID          : TPGPKeyID;
  KeyIDStr       : PGPChar;
  Loaded         : boolean;
  pDataOut       : PChar;
  SizeOut        : PGPSize;
  Version        : PGPUint32;
begin
// must change lousy variable names one of these days
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  DataOut := nil;
  FileSpec := nil;
  pDataOut := nil;

  Loaded := false;
  result := PGPsdkInit;
  if result <> 0 then exit;
  Loaded := true;

  try
    result := PGPNewContext(kPGPsdkAPIVersion, Context);
    if result <> 0 then exit;

    result := PGPsdkLoadDefaultPrefs(Context);
    if result <> 0 then exit;

    // Pubring
    if Flags and PGPPrefsFlag_PublicKeyring = PGPPrefsFlag_PublicKeyring then
    begin
      result := PGPsdkPrefGetFileSpec(Context, kPGPsdkPref_PublicKeyring, FileSpec);
      if result <> 0 then exit;
      result := PGPGetFullPathFromFileSpec(FileSpec, pDataOut);
      StrLCopy(Prefs.PublicKeyring, pDataOut, 255);
      StrCopy(pDataOut, '');
    end;
    if result <> 0 then exit;

    // Secring
    if Flags and PGPPrefsFlag_PrivateKeyring = PGPPrefsFlag_PrivateKeyring then
    begin
      result := PGPsdkPrefGetFileSpec(Context, kPGPsdkPref_PrivateKeyring, FileSpec);
      if result <> 0 then exit;
      result := PGPGetFullPathFromFileSpec(FileSpec, pDataOut);
      StrLCopy(Prefs.PrivateKeyring, pDataOut, 255);
      StrCopy(pDataOut, '');
    end;
    if result <> 0 then exit;

    //randseed.bin
    if Flags and PGPPrefsFlag_RandomSeedFile = PGPPrefsFlag_RandomSeedFile then
    begin
      result := PGPsdkPrefGetFileSpec(Context, kPGPsdkPref_RandomSeedFile, FileSpec);
      if result <> 0 then exit;
      result := PGPGetFullPathFromFileSpec(FileSpec, pDataOut);
      StrLCopy(Prefs.RandomSeedFile, pDataOut, 255);
      StrCopy(pDataOut, '');
    end;
    if result <> 0 then exit;

    //groups file
    if not PGPversion5 then // PGP6+ only
    if Flags and PGPPrefsFlag_GroupsFile = PGPPrefsFlag_GroupsFile then
    begin
      result := PGPsdkPrefGetFileSpec(Context, kPGPsdkPref_GroupsFile, FileSpec);
      if result <> 0 then exit;
      result := PGPGetFullPathFromFileSpec(FileSpec, pDataOut);
      StrLCopy(Prefs.GroupsFile, pDataOut, 255);
      StrCopy(pDataOut, '');
    end;
    if result <> 0 then exit;

    // Private Key
    if Flags and PGPPrefsFlag_DefaultKeyID = PGPPrefsFlag_DefaultKeyID then
    begin
      result := PGPsdkPrefGetData(Context, kPGPsdkPref_DefaultKeyID, DataOut, sizeOut);
      if result = kPGPError_ItemNotFound then exit;
      if result <> 0 then exit;
      { put rawdata into Tpgpkeyid format }
      result := PGPImportKeyID(DataOut, KeyID);
      if result <> 0 then exit;
      { put Tpgpkeyid into abbreviated string format }
      result := PGPGetKeyIDString(KeyID, kPGPKeyIDString_Abbreviated, KeyIDStr);
      StrLCopy(Prefs.DefaultKeyID, KeyIDStr, 10);
    end;
    if result <> 0 then exit;

  finally
    PGPFreeData(DataOut);
    PGPFreeData(pDataOut);
    PGPFreeFileSpec(FileSpec);
    PGPFreeContext(Context);
    if Loaded then PGPsdkCleanup;
  end;
end;

function spgpsetpreferences(Prefs: pPreferenceRec; Flags: Longint):LongInt;stdcall;export;
var
  FileSpec       : pPGPFileSpec;
  Loaded         : boolean;
  Key            : pPGPKey;
  KeyID          : TPGPKeyID;
  ExportedKeyData : PGPByteArray;
  ExportedKeyDataLength : PGPSize;
  KeySet         : pPGPKeySet;
  Context        : pPGPContext;
begin
// must change lousy variable names one of these days
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  FileSpec := nil;
  KeySet := nil;

  Loaded := false;
  result := PGPsdkInit;
  if result <> 0 then exit;
  Loaded := true;

  try
    result := PGPNewContext(kPGPsdkAPIVersion, Context);
    if result <> 0 then exit;

    result := PGPsdkLoadDefaultPrefs(Context);
    if result <> 0 then exit;

    // default secret key
    if Flags and PGPPrefsFlag_DefaultKeyID = PGPPrefsFlag_DefaultKeyID then
    begin
      result := PGPOpenDefaultKeyRings(Context, 0, KeySet);
      if result <> 0 then exit;

      if not KeyIsOnRing(Context, KeySet, Prefs.DefaultKeyID) then
      begin
        result := SPGPError_KeyNotFound;
        exit;
      end;

      { 1. get TpgpkeyID from hex id string }
      result := PGPGetKeyIDFromString(Prefs.DefaultKeyID, KeyID);
      if result <> 0 then exit;

      { 2. export weird ID to weirder data format }
      result := PGPExportKeyID(KeyID, ExportedKeyData, ExportedKeyDataLength);
      if result <> 0 then exit;

      { 3. let 'er rip }
      result := PGPsdkPrefSetData(Context, kPGPsdkPref_DefaultKeyID, @ExportedKeyData, ExportedKeyDataLength);
      if result <> 0 then exit;
    end;

    // PublicKeyring
    if Flags and PGPPrefsFlag_PublicKeyring = PGPPrefsFlag_PublicKeyring then
    begin
      result := PGPNewFileSpecFromFullPath(Context, Prefs.PublicKeyring, FileSpec);
      if result <> 0 then exit;
      try
        result := PGPsdkPrefSetFileSpec(Context, kPGPsdkPref_PublicKeyring, FileSpec);
        if result <> 0 then exit;
      finally
        PGPFreeFileSpec(FileSpec);
      end;
    end;
    
    // PrivateKeyring
    if Flags and PGPPrefsFlag_PrivateKeyring = PGPPrefsFlag_PrivateKeyring then
    begin
      result := PGPNewFileSpecFromFullPath(Context, Prefs.PrivateKeyring, FileSpec);
      if result <> 0 then exit;
      try
        result := PGPsdkPrefSetFileSpec(Context, kPGPsdkPref_PrivateKeyring, FileSpec);
        if result <> 0 then exit;
      finally
        PGPFreeFileSpec(FileSpec);
      end;
    end;

    // kPGPsdkPref_RandomSeedFile
    if Flags and PGPPrefsFlag_RandomSeedFile = PGPPrefsFlag_RandomSeedFile then
    begin
      result := PGPNewFileSpecFromFullPath(Context, Prefs.RandomSeedFile, FileSpec);
      if result <> 0 then exit;
      try
        result := PGPsdkPrefSetFileSpec(Context, kPGPsdkPref_RandomSeedFile, FileSpec);
        if result <> 0 then exit;
      finally
        PGPFreeFileSpec(FileSpec);
      end;
    end;

    // kPGPsdkPref_GroupsFile
    if not PGPversion5 then // PGP6+ only
    if Flags and PGPPrefsFlag_GroupsFile = PGPPrefsFlag_GroupsFile then
    begin
      result := PGPNewFileSpecFromFullPath(Context, Prefs.GroupsFile, FileSpec);
      if result <> 0 then exit;
      try
        result := PGPsdkPrefSetFileSpec(Context, kPGPsdkPref_GroupsFile, FileSpec);
        if result <> 0 then exit;
      finally
        PGPFreeFileSpec(FileSpec);
      end;
    end;

  finally
    PGPsdkSavePrefs(Context);
    PGPFreeKeySet(KeySet);
    PGPFreeContext(Context);
    if Loaded then PGPsdkCleanup;
  end;
end;

function spgppreferencesdialog(Flags, WindowHandle: Longint): Longint; stdcall; export;
var
  Context        : pPGPContext;
  KeySet         : pPGPKeySet;
begin
  result := kPGPError_NoError;

  if not pgpClientLoaded then
    result := SPGPError_NoPGPsdk;
  if result <> 0 then exit;

  result := Init(Context, KeySet, false, false);
  if result <> 0 then exit;
  try
    if Assigned(PGPclInitLibrary) then
      PGPclInitLibrary(Context);

    if PGPversion5 then
      result := PGPcomdlgPreferences(Context, WindowHandle, Flags)
    else
      result := PGPclPreferences(Context, WindowHandle, Flags, KeySet)
  finally
    if Assigned(PGPcomdlgCloseLibrary) then
      PGPcomdlgCloseLibrary;
    if Assigned(PGPclCloseLibrary) then
      PGPclCloseLibrary;
    QuitIt(Context, KeySet);
  end;
end;

(* OBSOLETE *)

(* set a preference: public or secret key-ring,
   randseed.bin file, default signing key *)
function spgpsetpreference(Preference: LongInt; Value: PChar):LongInt;stdcall;export;
var
  Pref           : PGPsdkPrefSelector;
  p              : pointer;
  Size           : PGPSize;
  Ref            : pPGPFileSpec;
  Loaded         : boolean;
  Version        : PGPUINT32;
  Key            : pPGPKey;
  ID             : TPGPKeyID;
  ExpData        : PGPByteArray;
  ExpLength      : PGPSIZE;
  KeySet         : pPGPKeySet;
  Context        : pPGPContext;
begin
// must change lousy variable names one of these days
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeySet := nil;
  Ref := nil;

  Loaded := false;
  result := PGPsdkInit;
  if result <> 0 then exit;
  Loaded := true;

  result := PGPNewContext(kPGPsdkAPIVersion, Context);
  if result <> 0 then exit;

  result := PGPsdkLoadDefaultPrefs(Context);
  if result <> 0 then exit;

  { convert pref to prefselector type }
  case Preference of
    0: pref := kPGPsdkPref_PublicKeyring;
    1: pref := kPGPsdkPref_PrivateKeyring;
    2: pref := kPGPsdkPref_RandomSeedFile;
    3: pref := kPGPsdkPref_DefaultKeyID;
    4: pref := kPGPsdkPref_GroupsFile// pgp6
  else
    pref := kPGPsdkPref_PublicKeyring;
  end;

  try
    { default key uses the setdata function and
      requires the keyrings. don't try opening the rings
      for the other prefs; unnecessary and may confuse in
      the case of kPGPsdkPref_PublicKeyring, etc. }
    if pref = kPGPsdkPref_DefaultKeyID then
    begin
      result := PGPOpenDefaultKeyRings(Context, 0, KeySet);
      if result <> 0 then exit;

      if not KeyIsOnRing(Context, KeySet, Value) then
      begin
        result := SPGPError_KeyNotFound;
        exit;
      end;
      { 1. get TpgpkeyID from Value }
      result := PGPGetKeyIDFromString(Value,id);
      if result <> 0 then exit;

      { 2. export weird ID to weirder data format }
      result := PGPExportKeyID(id,expDATA,expLength);
      if result <> 0 then exit;

      { 3. let 'er rip }
      result := PGPsdkPrefSetData(Context,Pref,@expData,expLength);
      if result <> 0 then exit;
    end
    else

    { these preferences use setfilespec }
    begin
      result := PGPNewFileSpecFromFullPath(Context,Value,ref);
      if result <> 0 then exit;

      try
        result := PGPsdkPrefSetFileSpec(Context,pref,ref);
        if result <> 0 then exit;

      finally
        PGPFreeFileSpec(ref);
      end;
    end;

    result := PGPsdkSavePrefs(Context);
    if result <> 0 then exit;
    { reload preferences so that changes take effect? }
    result := PGPsdkLoadDefaultPrefs(Context);

  finally
    PGPFreeKeySet(KeySet);
    PGPFreeContext(Context);
    if Loaded then PGPsdkCleanup;
  end;
end;

(* get a preference: public or secret key-ring,
   randseed.bin file, default private key *)
function spgpgetpreference(Preference: LongInt; Value: PChar):LongInt;stdcall;export;
var
  Context        : pPGPContext;
  Loaded         : boolean;
  Pref           : PGPsdkPrefSelector;
  DataOut        : Pointer;
  SizeOut        : PGPSize;
  Ref            : pPGPFileSpec;
  pDataOut       : PChar;
  Version        : PGPUint32;
  ID             : TPGPKeyID;
  OutString      : PGPChar;
begin
// must change lousy variable names one of these days
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  DataOut := nil;
  pDataOut := nil;
  Ref := nil;

  Loaded := false;
  result := PGPsdkInit;
  if result <> 0 then exit;
  Loaded := true;

  result := PGPNewContext(kPGPsdkAPIVersion, Context);
  if result <> 0 then exit;

  { convert pref to prefselector type }
  case Preference of
    0: pref := kPGPsdkPref_PublicKeyring;
    1: pref := kPGPsdkPref_PrivateKeyring;
    2: pref := kPGPsdkPref_RandomSeedFile;
    3: pref := kPGPsdkPref_DefaultKeyID;
    4: pref := kPGPsdkPref_GroupsFile// pgp 6
  else
    pref := kPGPsdkPref_PublicKeyring;
  end;

  try
    result := PGPsdkLoadDefaultPrefs(Context);

    if result <> 0 then exit;

    if Pref = kPGPsdkPref_DefaultKeyID then
    begin
      result := PGPsdkPrefGetData(Context, Pref, DataOut, sizeOut);
      if result <> 0 then exit;
      { put rawdata into Tpgpkeyid format }
      result := PGPImportKeyID(DataOut, ID);
      if result <> 0 then exit;
      { put Tpgpkeyid into abbreviated string format }
      result := PGPGetKeyIDString(id, kPGPKeyIDString_Abbreviated, outstring);
      StrLCopy(Value,outstring,16);
    end
    else
    begin
      result := PGPsdkPrefGetFileSpec(Context, Pref, Ref);
      if result <> 0 then exit;
      result := PGPGetFullPathFromFileSpec(ref, pDataOut);
      StrLCopy(Value, pDataOut, MAX_PATH -1);
    end;

  finally
    PGPFreeData(DataOut);
    PGPFreeData(pDataOut);
    PGPFreeFileSpec(Ref);
    PGPFreeContext(Context);
    if Loaded then PGPsdkCleanup;
  end;
end;

initialization
  // check all pgp units used by this unit
  LibsAllLoaded := pgpUtilitiesLoaded and
                   pgpErrorsLoaded and
                   pgpKeysLoaded and
                   pgpOptionListLoaded and
                   pgpSDKPrefsLoaded and
                   pgpFeaturesLoaded;

  PGPversion5 := IsPGP5;
end.
