{************************************************************************
{ spgpSimple - super-simple encryption & signing functions
{ this unit requires the PGP 6.5 version of PGPsc.dll
{
{ This unit requires one or more units from the Pascal translations of
{ the PGPsdk header files: <http://www.oz.net/~srheller/pgpsdk/>
{
{ The original Pascal code is: spgpSimple.pas, released 3 Sept 2000.
{ The initial developer of the Pascal code is Steven R. Heller
{ <srheller@oz.net>
{
{ Portions created by Steven R. Heller are
{ Copyright (C) 2000 Steven R. Heller
{
{ The contents of this file are used with permission, subject to
{ the Mozilla Public License Version 1.1 (the "License"); you may
{ not use this file except in compliance with the License. You may
{ obtain a copy of the License at
{ http://www.mozilla.org/MPL/MPL-1.1.html
{
{ Software distributed under the License is distributed on an
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
{ implied. See the License for the specific language governing
{ rights and limitations under the License.
{
{************************************************************************}

unit spgpSimple;

interface

uses
  // Delphi
  Windows, Classes, SysUtils, ShellAPI,
  // PGPsdk
  pgpErrors, pgpSC,
    PGPsdk,
  // SPGP
  spgpGlobals;

function spgpclipboard(Flags, Parent: LongInt): Longbool; stdcall; export;
function spgpfilelist(Files: PChar; Flags, Parent: LongInt): Longbool; stdcall; export;
function spgpfiledrop(Files: Longint; Flags, Parent: LongInt): Longbool; stdcall; export;
function spgplaunch(Flags, Parent: LongInt): Longbool; stdcall; export;

implementation

const
  PGP_Encrypt      = $00000001;
  PGP_Sign         = $00000002;
  PGP_Decrypt      = $00000004;
  PGP_AddKeys      = $00000008;
  PGP_Wipe         = $00000010;

  PGP_LaunchKeys   = $00000100;
  PGP_LaunchTools  = $00000200;
  PGP_LaunchDisk   = $00000400;
  PGP_LaunchNet    = $00000800;
  PGP_LaunchAbout  = $00001000;
  PGP_LaunchViewer = $00002000;
  PGP_LaunchWipe   = $00004000;
  PGP_LaunchPrefs  = $00008000;
  PGP_LaunchHelp   = $00010000;

function test(p: pchar): longint;
begin
  PGPpath(p);
end;
{ CATCH-ALL CLIPBOARD FUNCTION }
function spgpclipboard(
         Flags   : LongInt;
         Parent  : LongInt
         )       : Longbool; stdcall; export;
var
  AddKeys        : boolean;
  Decrypt        : boolean;
  Encrypt        : BOOL;
  PGPsc          : pPGPsc;
  PGPtls         : pPGPtls;
  Sign           : BOOL;
  Wipe           : boolean;
begin
  result := false;
  if not PGPscLoaded or (Flags = 0) then exit;

  Encrypt := (PGP_Encrypt and Flags = PGP_Encrypt);
  Sign    := (PGP_Sign and Flags = PGP_Sign);
  Decrypt := (PGP_Decrypt and Flags = PGP_Decrypt);
  AddKeys := (PGP_AddKeys and Flags = PGP_AddKeys);
  Wipe    := (PGP_Wipe and Flags = PGP_Wipe);

  result := InitPGPsc(Parent, PGPsc, PGPtls);
  if not result then exit;

  try
    // add keys, ignore everything else
    if AddKeys then
    begin
      result := AddKeyClipboard(Parent, PGPsc, PGPtls);
      exit;
    end;

    // decode anything, including keys
    if Decrypt then
    begin
      result :=
      DecryptClipboard(Parent, PChar('PGP'), PGPsc, PGPtls);
      exit;
    end;

    if Wipe then
    begin
      result :=
      ClipboardWipe(Parent, PGPsc);
      exit;
    end;

    // encode (encrypt, sign, or just armor)
    result :=
    EncryptClipboard(Parent, PChar('PGP'), PGPsc, PGPtls, Encrypt, Sign);

  finally
    UninitPGPsc(Parent, PGPsc, PGPtls);
  end;
end;

{ CATCH-ALL FILE-LIST FUNCTION }
function spgpfilelist(
         Files   : PChar;
         Flags   : LongInt;
         Parent  : LongInt
         )       : Longbool; stdcall; export;
var
  AddKeys        : boolean;
  Decrypt        : boolean;
  Encrypt        : BOOL;
  FList          : pFileList;
  i              : integer;
  List           : TStringList;
  PGPsc          : pPGPsc;
  PGPtls         : pPGPtls;
  Sign           : BOOL;
  Wipe           : boolean;
begin
  result := false;
  FList := nil;

  if not PGPscLoaded or (Flags = 0) then exit;

  Encrypt := (PGP_Encrypt and Flags = PGP_Encrypt);
  Sign    := (PGP_Sign and Flags = PGP_Sign);
  Decrypt := (PGP_Decrypt and Flags = PGP_Decrypt);
  AddKeys := (PGP_AddKeys and Flags = PGP_AddKeys);
  Wipe    := (PGP_Wipe and Flags = PGP_Wipe);

  result := InitPGPsc(Parent, PGPsc, PGPtls);
  if not result then exit;

  List := TStringList.Create;

  try
    // put list of files into stringlist, an easy way to
    // get comma-delimited strings into an array
    List.CommaText := StrPas(Files);

    // put files into a filelist, which will be freed by the
    // function to which it is passed
    for i := 0 to List.Count -1 do
      if FileExists(List[i]) then
        AddToFileList(FList, PChar(List[i]), nil);

    // add keys, ignore everything else
    if AddKeys then
    begin
      result := AddKeyFileList(Parent, PGPsc, PGPtls, FList);
      exit;
    end;

    // decode anything, including keys
    if Decrypt then
    begin
      result :=
      DecryptFileList(Parent, PChar('PGP'), PGPsc, PGPtls, FList);
      exit;
    end;

    if Wipe then
    begin
      result :=
      WipeFileList(Parent, PGPsc, FList, TRUE);
      exit;
    end;

    // encode (encrypt, sign, or just armor)
    result :=
    EncryptFileList(Parent, PChar('PGP'), PGPsc, PGPtls, FList, Encrypt, Sign);

  finally
    List.Free;
    UninitPGPsc(Parent, PGPsc, PGPtls);
  end;
end;

{ CATCH-ALL HDROP FUNCTION }
function spgpfiledrop(
         Files   : Longint;
         Flags   : LongInt;
         Parent  : LongInt
         )       : Longbool; stdcall; export;
var
  AddKeys        : boolean;
  Decrypt        : boolean;
  Encrypt        : BOOL;
  PGPsc          : pPGPsc;
  PGPtls         : pPGPtls;
  Sign           : BOOL;
  Wipe           : boolean;
begin
  result := false;

  if not PGPscLoaded or (Flags = 0) then exit;

  Encrypt := (PGP_Encrypt and Flags = PGP_Encrypt);
  Sign    := (PGP_Sign and Flags = PGP_Sign);
  Decrypt := (PGP_Decrypt and Flags = PGP_Decrypt);
  AddKeys := (PGP_AddKeys and Flags = PGP_AddKeys);
  Wipe    := (PGP_Wipe and Flags = PGP_Wipe);

  result := InitPGPsc(Parent, PGPsc, PGPtls);
  if not result then exit;

  try
    // add keys, ignore everything else
    if AddKeys then
    begin
      result := AddKeyFileList(Parent, PGPsc, PGPtls, HDropToFileList(Files));
      exit;
    end;

    // decode anything, including keys
    if Decrypt then
    begin
      result :=
      DecryptFileList(Parent, PChar('PGP'), PGPsc, PGPtls, HDropToFileList(Files));
      exit;
    end;

    if Wipe then
    begin
      result :=
      WipeFileList(Parent, PGPsc, HDropToFileList(Files), TRUE);
      exit;
    end;

    // encode (encrypt, sign, or just armor)
    result :=
    EncryptFileList(Parent, PChar('PGP'), PGPsc, PGPtls, HDropToFileList(Files), Encrypt, Sign);

  finally
    UninitPGPsc(Parent, PGPsc, PGPtls);
  end;
end;

{ LAUNCH MISC. PGP COMPONENTS }
function spgplaunch(
         Flags   : LongInt;
         Parent  : LongInt
         )       : Longbool; stdcall; export;
var
  Path           : array[0..MAX_PATH-1] of char;
  PGPsc          : pPGPsc;
  PGPtls         : pPGPtls;
begin
  result := false;

  if not PGPscLoaded or (Flags = 0) then exit;

  result := InitPGPsc(Parent, PGPsc, PGPtls);
  if not result then exit;

  try
    // View the clipboard
    if PGP_LaunchViewer and Flags = PGP_LaunchViewer then
      LaunchInternalViewer(PGPsc, Parent);

    // Wipe free space on disk
    if PGP_LaunchWipe and Flags = PGP_LaunchWipe then
      FreeSpaceWipeWizard(Parent, PGPsc);

    // preferences dialog
    if PGP_LaunchPrefs and Flags = PGP_LaunchPrefs then
      PGPscPreferences(Parent, PGPsc, PGPtls);

    // Use secure viewer and/or anti-tempest font
    //function TempestViewer(PGPsc: pPGPsc; Parent: HWND; pInput: PChar; InputSize: DWORD; Warn: BOOL): Cardinal; cdecl external PGPlibName;

    // splash screen
    if PGP_LaunchAbout and Flags = PGP_LaunchAbout then
      result := PGPscAbout(Parent, PGPsc);

    // help file
    if PGP_LaunchHelp and Flags = PGP_LaunchHelp then
    begin
      PGPpath(Path);
      StrCat(Path,PChar('pgp.hlp'));
      ShellExecute(Parent, 'open', Path, nil, nil, SW_SHOWNORMAL);
    end;


    // Launch Keys/Tools
    if PGP_LaunchNet and Flags = PGP_LaunchNet then
      result := DoLaunchNet(Parent);
    if PGP_LaunchKeys and Flags = PGP_LaunchKeys then
      result := DoLaunchKeys(Parent);
    if PGP_LaunchTools and Flags = PGP_LaunchTools then
      result := DoLaunchTools(Parent);
    if PGP_LaunchDisk and Flags = PGP_LaunchDisk then
      result := DoLaunchDisk(Parent);
  finally
    UninitPGPsc(Parent, PGPsc, PGPtls);
  end;
end;

initialization

  
end.
