{************************************************************************}
{ spgpUIDialogs - user-interface dialog functions                        }
{                                                                        }
{ This unit requires one or more units from the Pascal translations of   }
{ the PGPsdk header files: <http://www.oz.net/~srheller/pgpsdk/>         }
{ 								                                                       }
{ The original Pascal code is: spgpUIDialogs.pas, released 10 Apr 2000.  }
{ The initial developer of the Pascal code is Steven R. Heller           }
{ <srheller@oz.net>                                                      }
{ 								                                                       }
{ Portions created by Steven R. Heller are    				                   }
{ Copyright (C) 2000 Steven R. Heller                                    }
{									                                                       }
{ The contents of this file are used with permission, subject to         }
{ the Mozilla Public License Version 1.1 (the "License"); you may        }
{ not use this file except in compliance with the License. You may       }
{ obtain a copy of the License at                                        }
{ http://www.mozilla.org/MPL/MPL-1.1.html 	                             }
{ 									                                                     }
{ Software distributed under the License is distributed on an 	         }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or         }
{ implied. See the License for the specific language governing           }
{ rights and limitations under the License.                              }
{                                                                        }
{************************************************************************}

unit spgpUIDialogs;

interface

uses
  // Delphi
  SysUtils, Windows,
  // PGPsdk
  pgpBase, pgpPubTypes, pgpUserInterface, pgpErrors,
    pgpUtilities, pgpOptionList, pgpKeys, pgpClient, pgpTLS,
  // SPGP
  spgpGlobals, spgpKeyUtil, spgpUtil,
  // always last
  PGPsdk;

  function spgpuirecipientsdialog(RecipientHexID: PChar; RecipientHexIDLen: Longint; NewCaption, Reserved1: PChar; DisplayMarginalValidity, Reserved2, Reserved3, Reserved4, Reserved5, WinHandle: Longint): Longint; stdcall;export;
  function spgpuisigningpassphrasedialog(SignKeyHexID, SignKeyPass, DefaultKeyID: PChar; FindMatchingKey,WinHandle: Longint): Longint; stdcall;export;
  function spgpuiconfirmationpassphrasedialog(Passphrase: PChar; ShowPassphraseQuality, MinimumPassphraseQuality, MinimumPassphraseLength, WinHandle: Longint): Longint; stdcall;export;
  function spgpuikeypassphrasedialog(KeyID, Passphrase: PChar; WinHandle: Longint): Longint; stdcall;export;
  function spgpuiselectkeysdialog(KeyID, KeyProps, Prompt: PChar; KeyPropsLen, ShowAll, Flags, WinHandle: LongInt):LongInt;stdcall;export;

implementation

var
  LibsAllLoaded  : boolean;
  PGPversion5    : boolean;
  
function spgpuirecipientsdialog(
         RecipientHexID          : PChar;  // buffer for recipient's keyid
         RecipientHexIDLen       : Longint;// size of buffer
         NewCaption              : PChar;  // new dialog caption, max 127+null
         Reserved1               : PChar;
         DisplayMarginalValidity : Longint;//
         Reserved2               : Longint;// reserved for
         Reserved3               : Longint;// the %&#!$*@
         Reserved4               : Longint;// checkboxes
         Reserved5               : Longint;//
         WinHandle               : Longint // handle to calling app or form
         ): Longint;stdcall;export;
var
  Context        : pPGPContext;
  FoundKey       : pPGPKey;
  KeyIter        : pPGPKeyIter;
  KeyList        : pPGPKeyList;
  KeySet 			   : pPGPKeySet;
  Options        : pPGPOptionList;
  RecipientKeys  : pPGPKeySet;
  s              : string;
  t              : shortstring;
  tick           : PGPUInt32;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeySet := nil;
  Options := nil;
  RecipientKeys := nil;
  s := '';

  // version check: UI not available before PGP 6
  if PGPversion5 then
    result := kPGPError_IncompatibleAPI;
  if result <> 0 then exit;

  result := Init(Context,KeySet,false,false);
  if result <> 0 then exit;
  // PGPsdkUILibInit not used in 6.0.x
  if Assigned(PGPsdkUILibInit) then
    result := PGPsdkUILibInit;
  if result <> 0 then exit;

  try
    // build basic dialog options
    result := PGPBuildOptionList(Context, Options,
      [
      PGPOUIParentWindowHandle(Context, WinHandle),
      PGPOUIDisplayMarginalValidity(Context,
        PGPBoolean(DisplayMarginalValidity))
      ]);

    if result <> 0 then exit;

    // add optional options
    if not ZeroLength(NewCaption) then
      result := PGPAppendOptionList(Options,
        [
        PGPOUIWindowTitle(Context, NewCaption)
        ]);

    result := PGPRecipientDialog(Context,
      KeySet, PGPTrue, RecipientKeys, [Options]);

    if result <> 0 then exit;

    { for each key in the recipient keyset, get keyprops }
    result := PGPOrderKeySet(RecipientKeys, kPGPUserIDOrdering, KeyList);
    if result <> 0 then exit;
    try
      result := PGPNewKeyIter(KeyList, KeyIter);
      if result <> 0 then exit;
      result := PGPKeyIterRewind(KeyIter);
      if result <> 0 then exit;
      try
        while (PGPKeyIterNext(KeyIter, FoundKey) = 0) and
          Assigned(FoundKey) do
        begin
          t := '';
          result := GetKeyPropKeyID(FoundKey,t);
          if result <> 0 then exit;
          s := s + t + #9;
        end;
	    finally
	      PGPFreeKeyIter(KeyIter);
      end;
    finally
	    PGPFreeKeyList(KeyList);
    end;

    { trim off trailing tab }
    if s <> '' then
      while s[Length(s)] = #9 do
        s := Copy(s,0,Length(s) -1);

    s := s + #13#10;

    if Length(s) > RecipientHexIDLen - 1 then
      result := Length(s) + 1
    else
      StrPLCopy(RecipientHexID, s, RecipientHexIDLen - 1);

  finally
    PGPFreeKeyset(RecipientKeys);
    PGPFreeOptionList(Options);
    // PGPsdkUILibCleanup not used in 6.0.x
    if Assigned(PGPsdkUILibCleanup)
      then PGPsdkUILibCleanup;
    QuitIt(Context,KeySet);
  end;
end;

function spgpuisigningpassphrasedialog(
         SignKeyHexID    : PChar;  // the selected key's ID, 8 bytes max
         SignKeyPass     : PChar;  // the selected key's passphrase, 256 bytes max
         DefaultKeyID    : PChar;  // show this key as the default choice, 256 bytes
         FindMatchingKey : Longint;// search for a key matching a bad passphrase?
         WinHandle       : Longint // handle of the calling app or form
         ): Longint;stdcall;export;
var
  Context        : pPGPContext;
  DefaultKey     : pPGPKey;
  KeySet			   : pPGPKeySet;
  LocalPass      : PChar;
  SelectedKey    : pPGPKey;
  t              : shortstring;
  YeKeyID        : TPGPKeyID;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeySet := nil;
  DefaultKey := nil;
  SelectedKey := nil;

  result := Init(Context,KeySet,false,false);
  if result <> 0 then exit;

  // PGPsdkUILibInit not used in 6.0.x
  if Assigned(PGPsdkUILibInit) then
    result := PGPsdkUILibInit;
  if result <> 0 then exit;

  try
    //GetMem(LocalPass, 256);
    
    if DefaultKeyID <> '' then
    result := GetKeyByID(Context,DefaultKey,LowerCase(StrPas(DefaultKeyID)),KeySet);

    // not a critical failure
    if result = SPGPError_KeyNotFound then result := 0;
    if result <> 0 then exit;

    if PGPversion5 then
    begin
      if Assigned(DefaultKey) then
        result := PGPGetKeyIDFromKey(DefaultKey, YeKeyID);
      if result <> 0 then exit;

      result :=
      PGPcomdlgGetPhrase(
        Context,
        WinHandle,
        PChar('&Enter passphrase for the above key:'),
        LocalPass,
        KeySet,
        @YeKeyID,
        1,
        SelectedKey,
        nil,//@Options,
        PGPCL_RSAENCRYPT);
    end
    else
      result :=
      PGPSigningPassphraseDialog(Context,
        KeySet, SelectedKey,
        [
        PGPOUIDefaultKey(Context, DefaultKey),// can be nil
        PGPOUIFindMatchingKey(Context, PGPBoolean(FindMatchingKey)),
        PGPOUIParentWindowHandle(Context, WinHandle),
        PGPOUIOutputPassphrase(Context, LocalPass)
        ]);
      if result <> 0 then exit;

    StrCopy(SignKeyPass, LocalPass);

    result := GetKeyPropKeyID(SelectedKey, t);
    if result <> 0 then exit;

    StrPCopy(SignKeyHexID, t);
  finally
    if PGPversion5 then
      PGPcomdlgFreePhrase(LocalPass)
    else
      PGPFreeData(LocalPass);
    if Assigned(PGPcomdlgCloseLibrary)
      then PGPcomdlgCloseLibrary;

    // PGPsdkUILibCleanup not used in 6.0.x
    if Assigned(PGPsdkUILibCleanup)
      then PGPsdkUILibCleanup;
    QuitIt(Context,KeySet);
  end;
end;

function spgpuiconfirmationpassphrasedialog(
         Passphrase               : PChar;
         ShowPassphraseQuality    : Longint;
         MinimumPassphraseQuality : Longint;
         MinimumPassphraseLength  : Longint;
         WinHandle                : Longint
         ): Longint; stdcall;export;
var
  Context        : pPGPContext;
  KeySet 			   : pPGPKeySet;
  LocalPass      : PChar;
  Options        : pPGPOptionList;
  PluralOfChar   : shortstring;
  SelectedKey    : pPGPKey;
  YeKeyID        : TPGPKeyID;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeySet := nil;
  LocalPass := nil;
  Options := nil;
  SelectedKey := nil;

  result := Init(Context,KeySet,false,false);
  if result <> 0 then exit;
  
  // PGPsdkUILibInit not used in 6.0.x
  if Assigned(PGPsdkUILibInit) then
    result := PGPsdkUILibInit;
  if result <> 0 then exit;

  try
    if PGPversion5 then
    begin
      //GetMem(LocalPass, 256);
      result :=
      PGPcomdlgGetPhrase(
        Context,
        WinHandle,
        PChar('&Enter conventional encryption passphrase:'),
        LocalPass,
        KeySet,
        @YeKeyID,
        0,
        SelectedKey,
        nil,
        PGPCL_ENCRYPTION);
    end
    else
    begin
      result := PGPBuildOptionList(Context, Options,
        [
        PGPOUIOutputPassphrase(Context, LocalPass),
        PGPOUIParentWindowHandle(Context, WinHandle),
        PGPOUIWindowTitle(Context, PChar('PGP - Enter Passphrase')),
        PGPOUIShowPassphraseQuality(Context, ShowPassphraseQuality),
        PGPOUIMinimumPassphraseQuality(Context, MinimumPassphraseQuality)
        ]);
      if result <> 0 then exit;

      if MinimumPassphraseLength > 0 then
      begin
        if MinimumPassphraseLength = 1 then
          PluralOfChar := 'character'
        else
          PluralOfChar := 'characters';

        result :=
        PGPAppendOptionList(Options,
          [
          PGPOUIDialogPrompt(Context, PChar('&Passphrase (minimum '+IntToStr(MinimumPassphraseLength)+' '+PluralOfChar+'):')),
          PGPOUIMinimumPassphraseLength(Context, MinimumPassphraseLength)
          ]);
      end;
      result := PGPConfirmationPassphraseDialog(Context, [Options]);
    end;

    if result <> 0 then exit;

    StrCopy(Passphrase, LocalPass);

  finally
    PGPFreeOptionList(Options);
    if PGPversion5 then
      PGPcomdlgFreePhrase(LocalPass)
    else
      PGPFreeData(LocalPass);
    if Assigned(PGPcomdlgCloseLibrary)
      then PGPcomdlgCloseLibrary;

    // PGPsdkUILibCleanup not used in 6.0.x
    if Assigned(PGPsdkUILibCleanup)
      then PGPsdkUILibCleanup;
    QuitIt(Context,KeySet);
  end;
end;

function spgpuikeypassphrasedialog(
         KeyID         : PChar;// key to get passphrase for
         Passphrase    : PChar;
         WinHandle     : Longint
         ): Longint;stdcall;export;
var
  Context        : pPGPContext;
  Key            : pPGPKey;
  KeySet         : pPGPKeySet;
  LocalPass      : PChar;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  Key := nil;
  KeySet := nil;
  LocalPass := nil;

  // version check: UI not available before PGP 6
  if PGPversion5 then
    result := kPGPError_IncompatibleAPI;
  if result <> 0 then exit;

  result := Init(Context,KeySet,false,false);
  if result <> 0 then exit;
  
  // PGPsdkUILibInit not used in 6.0.x
  if Assigned(PGPsdkUILibInit) then
    result := PGPsdkUILibInit;
  if result <> 0 then exit;

  try
    result := GetKeyByID(Context, Key, KeyID, KeySet);
    if result <> 0 then exit;

    result := PGPKeyPassphraseDialog(Context, Key,
      [
      PGPOUIOutputPassphrase(Context, LocalPass),
      PGPOUIParentWindowHandle(Context, WinHandle)
      ]);

    if result <> 0 then exit;

    StrCopy(Passphrase, LocalPass);

  finally
      PGPFreeData(LocalPass);
    // PGPsdkUILibCleanup not used in 6.0.x
    if Assigned(PGPsdkUILibCleanup)
      then PGPsdkUILibCleanup;
    QuitIt(Context,KeySet);
  end;
end;

function spgpuiselectkeysdialog(KeyID, KeyProps, Prompt: PChar; KeyPropsLen, ShowAll, Flags, WinHandle: LongInt):LongInt;stdcall;export;
var
  Context        : pPGPContext;
  KeySet         : pPGPKeySet;
  KeySetToDisplay : pPGPKeySet;
  KeySetSelected : pPGPKeySet;
  TempKeyProps   : string;
  TlsContext     : pPGPtlsContext;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeySet := nil;
  KeySetSelected := nil;
  KeySetToDisplay := nil;
  TlsContext := nil;

  result := Init(Context, KeySet, false, false);
  if result <> 0 then exit;

  try
    // if ShowAll then use the whole keyset 
    if ShowAll = 1 then
      KeySetToDisplay := KeySet
    else
    begin
      result := PGPNewKeySet(Context, KeySetToDisplay);
      if result <> 0 then exit;

      result := GetKeysById(Context, KeySetToDisplay, LowerCase(StrPas(KeyID)), KeySet);
      if result <> 0 then exit;
    end;
    
    if PGPversion5 then
    begin
      result := kPGPError_IncompatibleAPI;
      {
      result := PGPNewKeySet(Context, KeySetSelected);
      if result <> 0 then exit;

      // a new thread to change child's caption & button
      VandalThread := TVandal.Create(True);
      VandalThread.FreeOnTerminate := True;

      try
        VandalThread.TargetHandle := WinHandle;
        VandalThread.Resume;

        result :=
        PGPkmQueryAddKeys(Context,
          WinHandle,
          KeySetToDisplay,
          KeySetSelected
        );
      except
        on E: exception do
          VandalThread.Free;
      end;
      }
    end
    else
    begin
      result := PGPNewTLSContext(Context, tlsContext);
      if result <> 0 then exit;

      PGPclInitLibrary(Context);

      result :=
      PGPclSelectKeys(Context,
        TlsContext,
        WinHandle,
        Prompt,
        KeySetToDisplay,
        KeySet,
        KeySetSelected
      );
    end;
    if result <> 0 then exit;

    result := GetKeySetProps(Context, KeySetSelected, TempKeyProps, Flags);
    if result <> 0 then exit;

    if Length(TempKeyProps) > KeyPropsLen - 1
    then result := Length(TempKeyProps) + 1
    else StrPLCopy(KeyProps, TempKeyProps, KeyPropsLen - 1);

  finally
    if not PGPversion5 then
    begin
      PGPclCloseLibrary;
      PGPFreeTLSContext(tlsContext);
    end;
    PGPFreeKeySet(KeySetToDisplay);
    QuitIt(Context, KeySet);
  end;
end;

initialization
  // check all pgp units used by this unit
  LibsAllLoaded := pgpUtilitiesLoaded and
                   pgpErrorsLoaded and
                   pgpKeysLoaded and
                   pgpOptionListLoaded;
                   //pgpClientLoaded;
                   //pgpUserInterfaceLoaded;

  PGPversion5 := IsPGP5;
end.

