{************************************************************************}
{ spgpUtil - internal utility functions                                  }
{                                                                        }
{ This unit requires one or more units from the Pascal translations of   }
{ the PGPsdk header files: <http://www.oz.net/~srheller/pgpsdk/>         }
{ 								                                                       }
{ The original Pascal code is: spgpUtil.pas, released 10 Apr 2000.       }
{ The initial developer of the Pascal code is Steven R. Heller           }
{ <srheller@oz.net>                                                      }
{ 								                                                       }
{ Portions created by Steven R. Heller are    				                   }
{ Copyright (C) 2000 Steven R. Heller                                    }
{									                                                       }
{ The contents of this file are used with permission, subject to         }
{ the Mozilla Public License Version 1.1 (the "License"); you may        }
{ not use this file except in compliance with the License. You may       }
{ obtain a copy of the License at                                        }
{ http://www.mozilla.org/MPL/MPL-1.1.html 	                             }
{ 									                                                     }
{ Software distributed under the License is distributed on an 	         }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or         }
{ implied. See the License for the specific language governing           }
{ rights and limitations under the License.                              }
{                                                                        }
{************************************************************************}

unit spgpUtil;

{$S-}

interface

uses
	SysUtils, Classes, Forms, Windows, pgpBase;

type
  STRING2 = string[2];

   //function BytesToHex(const bytes: pchar): string;
   function BytesToHex(const bytes: array of PGPByte): string;
   function FingerAsHex(const bytes: shortstring;const len: integer;var s: shortstring): LongInt;
   function ByteToHex(b : byte): STRING2;
   function LocalTime(UnixSeconds: LongInt;var s: shortstring): LongInt;
   { StringCount returns the number of occurances of Substr in S }
   function StringCount(Substr,S : string) : integer;
   { StringChop returns a substring found with a delimiter
    (use StringCount to find 'count' - the number of delimiters
    in the main string) }
   function StringChop(text,delim : string; count : integer) : string;
   procedure ParseEmailList(const EmailList: string; var List: TStringList);
   procedure ParseIDList(const IDList: string; var List: TStringList);
   function OffsetFromUTC: TDateTime;
   function ZeroLength(const str: PChar): boolean;

implementation

{
function BytesToHex(const bytes: pchar): string;
var
	 i: 		integer;
  temp1:	STRING2;
  temp2:	string;
begin
  result := '';
	  for i := 0 to length(bytes) -1 do
    begin
      temp1 := ByteToHex(Ord(bytes[i]));
      temp2 := concat(temp2,temp1);
		end;
  result := Copy(temp2,1,Length(temp2));
end;
}
function BytesToHex(const bytes: array of PGPByte): string;
var
  i              : integer;
  temp1          : STRING2;
  temp2          : string;
begin
  result := '';
	for i := 0 to high(bytes) -1 do
  begin
    temp1 := ByteToHex(Ord(bytes[i]));
    temp2 := concat(temp2,temp1);
	end;
  result := Copy(temp2,1,Length(temp2));
end;

function FingerAsHex(const bytes: shortstring;const len: integer;var s: shortstring): LongInt;
var
  i: 		integer;
  temp1:	STRING2;
  temp2:	string;
begin
  result := 0;
  try
	  for i := 1 to len do
      begin
        temp1 := ByteToHex(Ord(bytes[i]));
        temp2 := concat(temp2,temp1);
		  end;
    s := Copy(temp2,1,Length(temp2));
  except
    result := -910;
    s := '';
    exit;
  end;
end;

function ByteToHex(b : byte):STRING2;
const
  HexChars: array [0..$F] of char = '0123456789ABCDEF';
begin
  result := '';
  try
    ByteToHex :=( HexChars[ b shr 4] + HexChars[ b and $F] );
  except
    result := '';
  end;
end;

function LocalTime(UnixSeconds: LongInt;var s: shortstring):LongInt;
var
  UnixTime,Time:  TDateTime;
  tz_info:        TTimeZoneInformation;
  diff:           longint;
const
  TIME_ZONE_ID_INVALID = DWORD($FFFFFFFF);
  TIME_ZONE_ID_UNKNOWN  = 0;
  TIME_ZONE_ID_STANDARD = 1;
  TIME_ZONE_ID_DAYLIGHT = 2;
begin
result := 0;

  try
  { Convert a UNIX time to a Delphi TDateTime }
  UnixTime := EncodeDate(1970, 1, 1) + UnixSeconds / (24.0*60.0*60.0);
  except
    result := -901;
    s := '';
    exit;
  end;
  
  try
	 { find the difference between local & GMT }
  case GetTimeZoneInformation(tz_info) of
    TIME_ZONE_ID_INVALID:   
      begin
        result := -902;
        exit;
      end;
    TIME_ZONE_ID_UNKNOWN:   diff := tz_info.Bias;
    TIME_ZONE_ID_DAYLIGHT:  diff := tz_info.Bias + tz_info.DaylightBias;
    TIME_ZONE_ID_STANDARD:  diff := tz_info.Bias + tz_info.StandardBias;
    else
    begin
      result := -902;
      exit;
    end;
  end;
  
// old method:
{ 
  case GetTimeZoneInformation(tz_info) of
    1: diff := tz_info.StandardBias + tz_info.Bias;
    2: diff := tz_info.DaylightBias + tz_info.Bias;
    else diff := 0;
  end; }
  
  except
      result := -902;
      exit;
  end;
      { bias @ PDT = 480 minutes
      diff div by 1440 min. per day = the fractional
      number of days to subtract from GMT }

  try    
  { subtract difference }
  Time := (UnixTime - (diff / 1440));
  except
    result := -903;
    s := '';
    exit;
  end;
  
  try
  { Format a TDateTime to a string in ctime format }
  s := FormatDateTime('ddd mmm dd hh:nn:ss yyyy', Time);
  except
    result := -904;
    s := '';
    exit;
  end;
end;

function OffsetFromUTC: TDateTime;
var
  iBias: Integer;
	tmez: TTimeZoneInformation;
const
  // CB3 and D3 do not have all of these
  TIME_ZONE_ID_INVALID = DWORD($FFFFFFFF);
  TIME_ZONE_ID_UNKNOWN  = 0;
  TIME_ZONE_ID_STANDARD = 1;
  TIME_ZONE_ID_DAYLIGHT = 2;
begin
  Case GetTimeZoneInformation(tmez) of
    TIME_ZONE_ID_INVALID:
      raise Exception.Create('Failed attempting to retrieve time zone information.');
    TIME_ZONE_ID_UNKNOWN:
      raise Exception.Create('Time Zone ID Unknown.');
    TIME_ZONE_ID_DAYLIGHT:
      iBias := - (tmez.Bias + tmez.DaylightBias);
    TIME_ZONE_ID_STANDARD:
      iBias := - (tmez.Bias + tmez.StandardBias);
    else
      raise Exception.Create('Failed attempting to retrieve time zone information.');
  end;
  Result := EncodeTime(Abs(iBias) div 60, Abs(iBias) mod 60, 0, 0);
  if iBias < 0 then
    Result := -Result;
end;

{ this function simply reports the number of occurances of
  substr in s }
function StringCount(Substr,S : string) : integer;
var
  i,count,substr_len : integer;
begin
  count := 0;
  substr_len := length(Substr);
  for i := 1 to length(S) do
    begin
      if Substr = copy(S,i,substr_len) then
        count := count +1;
    end;
  result := count;
end;

{ Then armed with the string and your knowledge of
	the number of your designated delimeters you can use this: }
function StringChop(text,delim : string; count : integer) : string;
var
  start,stop,i : integer;
  temptext : string;
begin
start := 0;
if count >0 then
  begin
    temptext := text;
    if count = 1 then
      begin
        start := 1;
        stop := pos (delim,text);
      end
    else
      begin
        for i:= 1 to count -1 do
          begin
            start := pos(delim,temptext)+length(delim);
            temptext := copy(temptext,start,length(temptext));
          end;
        stop := pos (delim,temptext);
      end;
    if start >0 then
      begin
         if stop = 0 then stop := length(text);
         result := copy (temptext,1,stop);
      end
    else
      result := text;
  end
else
  result := text;
end;

procedure ParseEmailList(const EmailList: string; var List: TStringList);
var
  i: integer;
begin
  if EmailList <> '' then
  List.CommaText := EmailList;

  { look for blanks }
  for i := List.Count -1 downto 0 do
    if List[i] = '' then List.Delete(i);

  { strip off brackets }
  for i := 0 to List.Count -1 do
  begin
    while List[i][1] = '<' do
      List[i] := Copy(List[i],2,Length(List[i]));
    while List[i][Length(List[i])] = '>' do
      List[i] := Copy(List[i],1,Length(List[i])-1);
  end;

end;

procedure ParseIDList(const IDList: string; var List: TStringList);
var
  i: integer;
begin
  if IDList <> '' then
  begin
    List.CommaText := IDList;

    { look for blank lines or lines without hex-prefix }
    for i := List.Count -1 downto 0 do
      if (Pos('0x',List[i]) = 0) or (List[i] = '') then
        List.Delete(i);
  end;
end;

(* is a pchar really empty? *)
function ZeroLength(const Str: PChar): boolean;
begin
  result := StrLen(Str) = 0;
end;

end.
